/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/


package org.opensi.compta;

import java.sql.*;
import java.util.*;

/*
 * Classe Ecriture : reprsentation d'une criture comptable
 */

public class Ecriture {


	private String codeJournal;
  private long dateEcriture;
  private int numEcriture;
	private int numero;

  private Hashtable<Integer, Operation> operations = new Hashtable<Integer, Operation>(3);
  private List<Operation> aSupprimer = new ArrayList<Operation>();

	private boolean encours = false;
  private boolean modifDateEcriture = false;
  private boolean modifCodeJournal = false;


  public Ecriture() {
    dateEcriture = 0;
    codeJournal = "";
    numEcriture = 0;
		numero = 0;
  }


  public Ecriture(String codeJournal, long dateEcriture) {

    this.codeJournal = codeJournal;
		this.dateEcriture = dateEcriture;
    numEcriture = 0;
		numero = 0;
  }


  public Ecriture(String codeJournal, long dateEcriture, int numEcriture) {

    this.codeJournal = codeJournal;
		this.dateEcriture = dateEcriture;
    this.numEcriture = numEcriture;
		numero = 0;
  }


	public Ecriture(String codeJournal, long dateEcriture, int numEcriture, int numero) {

    this.codeJournal = codeJournal;
		this.dateEcriture = dateEcriture;
    this.numEcriture = numEcriture;
		this.numero = numero;
  }




//	 charge une criture existante dans un objet Ecriture
	public static Ecriture loadEcriture(int numEcriture, Connection con, String base) throws SQLException {

		Ecriture ecr = null;

		Statement stt = con.createStatement();
		String reqEcriture = "select * from "+ base +".ECRITURE where Num_Ecriture="+ numEcriture;

		ResultSet rset = stt.executeQuery(reqEcriture);

		if (rset.next()) {

			ecr = new Ecriture(rset.getString("Code_Journal"), rset.getLong("Date_Ecriture"), numEcriture, rset.getInt("Numero"));

			Statement stt2 = con.createStatement();
			String reqOperations = "select * from "+ base +".OPERATION where Num_Ecriture="+ numEcriture +" order by Num_Ligne";

			ResultSet rsOp = stt2.executeQuery(reqOperations);

			while (rsOp.next()) {

				Operation op = new Operation(	rsOp.getInt("Op_Id"),
																			rsOp.getInt("Num_Ligne"),
																			rsOp.getString("Numero_Compte"),
																			rsOp.getDouble("Montant_D"),
																			rsOp.getDouble("Montant_C"),
																			rsOp.getString("Libelle"),
																			rsOp.getString("Num_Piece"),
																			rsOp.getString("Contrepartie"),
																			rsOp.getString("Type"),
																			rsOp.getString("Raccourci"),
																			rsOp.getString("Lettre"),
																			rsOp.getLong("Date_Op"),
																			rsOp.getLong("Date_Echeance"),
																			rsOp.getInt("Pointage") );

				ecr.addOp(op);

			}


			rsOp.close();
			stt2.close();
		}

		rset.close();
		stt.close();

		return ecr;
	}


	public String getCodeJournal() {	return codeJournal;	}

	public long getDateEcriture() { return dateEcriture; }

	public boolean getEnCours() { return encours; }

	public int getNumEcriture() { return numEcriture;	}

	public int getNumero() { return numero; }

	public Hashtable getOperations() { return operations; }

	public int getSize() { return operations.size(); }


	public SortedSet getOpIds() {

    return new TreeSet<Integer>(operations.keySet());
	}

	public Operation getOp(int index) {

		return operations.get(new Integer(index));
	}


  public void setEnCours() {

    encours = true;
  }


	public void setDateEcriture(long dateEcriture) {

		this.dateEcriture = dateEcriture;
    modifDateEcriture = true;
	}


	public void setCodeJournal(String codeJournal) {

		this.codeJournal = codeJournal;
    modifCodeJournal = true;
	}


	public void setNumEcriture(int numEcriture) {

		this.numEcriture = numEcriture;
	}




	public void addOp(Operation operation) {

    if (operation.getNumLigne() == 0) {
      int i = operations.size();
      while (operations.containsKey(new Integer(i))) {
        i++;
      }
      operation.setNumLigne(i);
    }
    operations.put(new Integer(operation.getNumLigne()), operation);
  }


  public void insertOpAt(Operation operation, int index) {

    int i = operations.size();
    while (operations.containsKey(new Integer(i))) {
      i++;
    }
    while (i > index) {
      Operation opaux = getOp(i-1);
      opaux.setNumLigne(i);
      operations.put(new Integer(i),opaux);
      i--;
    }
    operations.put(new Integer(index),operation);
  }


  public void addASupprimer(int index) {

    Integer cle = new Integer(index);
    Operation op = operations.remove(cle);
    aSupprimer.add(op);
    int i = index;
    while (operations.containsKey(new Integer(i+1))) {
      Operation opaux = operations.remove(new Integer(i+1));
      opaux.setNumLigne(i);
      operations.put(new Integer(i),opaux);
      i++;
    }
  }


  public boolean isStable() {

    Enumeration listeop = operations.elements();
    int total_debit = 0;
    int total_credit = 0;
    while (listeop.hasMoreElements()) {
      Operation op = (Operation)listeop.nextElement();
      if (op.isIncomplete()) {
        return false;
      } else {
        total_debit += op.getMontantCentD();
        total_credit += op.getMontantCentC();
      }
    }
    return (total_debit == total_credit);
  }


	public double totalDebit() {

    Enumeration listeop = operations.elements();
    int total_debit = 0;
    while (listeop.hasMoreElements()) {
      Operation op = (Operation)listeop.nextElement();
      total_debit += op.getMontantCentD();
    }
    double totd = new Integer(total_debit).doubleValue();
    return (totd/100);
  }


  public double totalCredit() {

    Enumeration listeop = operations.elements();
    int total_credit = 0;
    while (listeop.hasMoreElements()) {
      Operation op = (Operation)listeop.nextElement();
      total_credit += op.getMontantCentC();
    }
    double totc = new Integer(total_credit).doubleValue();
    return (totc/100);
  }


  public void insertDB(Connection c, String base, String baseDossier) {

		if (!isStable()) {
      System.out.println("Ecriture dsquilibre");
    }
		else if (codeJournal.equals("")) {
      System.out.println("Journal non renseign");
    }
		else if (dateEcriture == 0) {
      System.out.println("Date non renseigne");
    }
		else if (numEcriture == 0) {
      newEcriture(c, base, baseDossier);

    }
		else {
      updateEcriture(c, base, baseDossier);
    }
  }


  private void newEcriture(Connection con, String base, String baseDossier) {
    try {

			String attNum = "select coalesce(max(Numero),0)+1 from "+ base +".ECRITURE where Code_Journal='"+ codeJournal +"' and Date_Ecriture="+ dateEcriture;

      Statement stt = con.createStatement();

			ResultSet rsNum = stt.executeQuery(attNum);
			rsNum.next();
			this.numero = rsNum.getInt(1);

			String newEcriture = "insert into "+ base +".ECRITURE (Code_Journal, Date_Ecriture, Vide, Numero) values ('"+ codeJournal +"',"+ dateEcriture +", 0,"+ numero +")";

      stt.executeUpdate(newEcriture, Statement.RETURN_GENERATED_KEYS);

			ResultSet rsId = stt.getGeneratedKeys();

			rsId.next();
			numEcriture = rsId.getInt(1);

      stt.close();

      Enumeration listeop = operations.elements();
      while (listeop.hasMoreElements()) {
        Operation op = (Operation)listeop.nextElement();
        op.insertDB(con, base, baseDossier, numEcriture);

      }

      majAvancement(con, base, operations.size());

      encours = false;

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  private void updateEcriture(Connection con, String base, String baseDossier) {
    try {

      if (isStable()) {

				Statement stt = con.createStatement();

				if (this.numero==0) {
					String attNum = "select coalesce(max(Numero),0)+1 from "+ base +".ECRITURE where Code_Journal='"+ codeJournal +"' and Date_Ecriture="+ dateEcriture;
					ResultSet rsNum = stt.executeQuery(attNum);
					rsNum.next();
					this.numero = rsNum.getInt(1);
				}

        String majEcriture = "update "+ base +".ECRITURE set Code_Journal='"+ codeJournal +"',Date_Ecriture="+ dateEcriture +",Vide=0, Numero="+ numero +" where Num_Ecriture="+ numEcriture;

        stt.executeUpdate(majEcriture);

        int nbop = 0;
        Iterator iASupprimer = aSupprimer.iterator();

				while (iASupprimer.hasNext()) {
          Operation op = (Operation)iASupprimer.next();
          if (!op.isNew()) {
            nbop--;
            op.deleteDB(con, base, baseDossier);
          }
          iASupprimer.remove();
        }

        Enumeration listeop = operations.elements();

				while (listeop.hasMoreElements()) {
          Operation op = (Operation)listeop.nextElement();
          if (op.isNew()) nbop++;
          op.insertDB(con, base, baseDossier, numEcriture);
        }
        majAvancement(con, base, nbop);
        encours = false;
      }

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  private void majAvancement(Connection con, String base, int nbop) {
    try {

      Statement stt = con.createStatement();

      String reqAvancement = "select * from "+ base +".NB_OP_JOURNAUX where Code_Journal='"+ codeJournal +"' and Periode='"+ dateEcriture +"'";

      ResultSet rset = stt.executeQuery(reqAvancement);

			String majAvancement = "";

			if (rset.next()) {
        majAvancement = "update "+ base +".NB_OP_JOURNAUX set Nb_Op=Nb_Op+("+ nbop +") where Code_Journal='"+ codeJournal +"' and Periode='"+ dateEcriture +"'";
      }
			else {
        majAvancement = "insert into "+ base +".NB_OP_JOURNAUX(Code_Journal, Periode, Nb_Op) values ('"+ codeJournal +"',"+ dateEcriture +","+ nbop +")";
      }

      stt.executeUpdate(majAvancement);

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  public void supprimer(Connection con, String base, String baseDossier) {
    try {

      supprimerOperations(con, base, baseDossier);
      String supEcriture = "delete from "+ base +".ECRITURE where Num_Ecriture="+ numEcriture;
      Statement stt = con.createStatement();
      stt.executeUpdate(supEcriture);

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  public void supprimerOperations(Connection con, String base, String baseDossier) {

		int nbop = 0;

		Iterator iASupprimer = aSupprimer.iterator();

		while (iASupprimer.hasNext()) {
      Operation op = (Operation)iASupprimer.next();
      if (!op.isNew()) {
        nbop--;
        op.deleteDB(con, base, baseDossier);
      }
      iASupprimer.remove();
    }

    Enumeration listeop = operations.keys();

    while (listeop.hasMoreElements()) {
      Integer id = (Integer)listeop.nextElement();
      Operation op = operations.remove(id);
      if (!op.isNew()) {
        nbop--;
        op.deleteDB(con, base, baseDossier);
      }
    }
    majAvancement(con, base, nbop);
  }


	// pour debugger
	public void printEcriture() {

		System.out.println("Ecriture\n");

		Enumeration listeop = operations.elements();

    while (listeop.hasMoreElements()) {
      Operation op = (Operation)listeop.nextElement();

			op.printOperation();
    }
	}


} // fin Ecriture
