/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/


package org.opensi.compta;

import java.sql.*;

import org.opensi.util.tools.Arrondi;


/*
 * Classe Operation : reprsentation d'une ligne d'criture comptable
 */

public class Operation {

  private long dateOp;
  private int numLigne;
  private String numeroCompte;
	private int montantD;
	private int montantC;
  private String libelle;
  private String numPiece;
  private String contrepartie;
  private String type;
  private String raccourci;
	private long dateEcheance;
	private String lettre;
  private int opId;
	private int pointage;

	private Arrondi rd = new Arrondi(2);


  public Operation() {

		this.opId = 0;
    this.numLigne = 0;
   	this.numeroCompte = "";
    this.montantD = 0;
    this.montantC = 0;
    this.libelle = "";
    this.numPiece = "";
    this.contrepartie = "";
    this.type = "";
    this.raccourci = "";
		this.lettre = "";
		this.dateOp = 0;
    this.dateEcheance = 0;
		this.pointage = 0;
  }

	
	public Operation(int opId,
                   int numLigne,
                   String numeroCompte,
									 double montantD,
                   double montantC,
                   String libelle,
									 String numPiece,
                   String contrepartie,
                   String type,
									 String raccourci,
                   String lettre,
									 long dateOp,
									 long dateEcheance,
									 int pointage) {

    this.opId = opId;
    this.numLigne = numLigne;
    this.numeroCompte = numeroCompte;
    this.montantD = (int)(rd.round(montantD * 100));
    this.montantC = (int)(rd.round(montantC * 100));
    this.libelle = libelle;
    this.numPiece = numPiece;
    this.contrepartie = contrepartie;
    this.type = type;
		this.raccourci = raccourci;
    this.lettre = lettre;
		this.dateOp = dateOp;
		this.dateEcheance = dateEcheance;
		this.pointage = pointage;
  }




	/** retourne true si l'opration n'est pas encore dans la db */
	public boolean isNew() {

		return (opId == 0);
	}


	public boolean isIncomplete() {
    if (dateOp == 0) {
      return true;
    }
    if (numeroCompte.equals("")) {
      return true;
    }
    if ((montantD == 0) && (montantC == 0)) {
      return true;
    }
    return false;
  }



	public String getContrepartie()	{ return contrepartie; }

	public long getDateEcheance()	{ return dateEcheance; }

	public long getDateOp() {	return dateOp; }

	public String getLettre() { return lettre; }

	public String getLibelle() { return libelle; }

	public double getMontantD() { return rd.round((double) montantD/100); }

	public double getMontantC() { return rd.round((double) montantC/100); }

	public double getMontantCentD() { return montantD; }

	public double getMontantCentC() { return montantC; }

	public String getNumeroCompte()	{	return numeroCompte; }

	public int getNumLigne() { return numLigne;	}

	public String getNumPiece()	{	return numPiece; }

	public long getOpId() { return opId; }

	public String getRaccourci() { return raccourci; }

	public String getType()	{ return type; }
	
	public int getPointage() { return pointage; }


	public void setOpId(int OpId) {

		this.opId = OpId;
	}


	public void setContrepartie(String contrepartie) {

		this.contrepartie = contrepartie;
	}


	public void setDateEcheance(long dateEcheance) {

		this.dateEcheance = dateEcheance;
	}


	public void setDateOp(long dateOp) {

		this.dateOp = dateOp;
	}


	public void setLettre(String lettre) {

	  this.lettre = lettre;
  }


	public void setLibelle(String libelle) {

		this.libelle = libelle;
	}


	public void setMontantD(int montantD)	{

		this.montantD = montantD;
	}


	public void setMontantD(double montantD) {

		this.montantD = (int)(rd.round(montantD * 100));
	}


	public void setMontantC(int montantC)	{

		this.montantC = montantC;
	}


	public void setMontantC(double montantC) {

		this.montantC = (int)(rd.round(montantC * 100));
	}


	public void setNumeroCompte(String numeroCompte) {

		this.numeroCompte = numeroCompte;
	}


	public void setNumLigne(int numLigne)	{

		this.numLigne = numLigne;
	}


	public void setNumPiece(String numPiece) {

		this.numPiece = numPiece;
	}


	public void setRaccourci(String raccourci) {

		this.raccourci = raccourci;
	}


	public void setType(String type) {

		this.type = type;
	}




  public void deleteDB(Connection con, String base, String baseDossier) {

    if (!isNew()) {
      deleteOperation(con, base, baseDossier);
    }
  }

  public void insertDB(Connection con, String base, String baseDossier, int numEcriture) {

    if (isNew()) {
      newOperation(con, base, baseDossier, numEcriture);
    } else {
      updateOperation(con, base, baseDossier);
    }
  }





  private void newOperation(Connection con, String base, String baseDossier, int numEcriture) {
    try {

      if (!isIncomplete()) {

        double debit = getMontantD();
        double credit = getMontantC();

        String newOperation = "insert into "+ base +".OPERATION (Num_Ecriture, Num_Ligne, Numero_Compte, Montant_D, Montant_C,"
														+ "Libelle, Num_Piece, Contrepartie, Type, Raccourci, Lettre, Date_Op, Date_Echeance) values (?,?,?,?,?,?,?,?,?,?,?,?,?)";

				PreparedStatement psNewOp = con.prepareStatement(newOperation, Statement.RETURN_GENERATED_KEYS);

				psNewOp.setInt(1, numEcriture);
				psNewOp.setInt(2, numLigne);
				psNewOp.setString(3, numeroCompte);
				psNewOp.setDouble(4, debit);
				psNewOp.setDouble(5, credit);
				psNewOp.setString(6, libelle);
				psNewOp.setString(7, numPiece);
				psNewOp.setString(8, contrepartie);
				psNewOp.setString(9, type);
				psNewOp.setString(10, raccourci);
				psNewOp.setString(11, lettre);
				psNewOp.setLong(12, dateOp);
				psNewOp.setLong(13, dateEcheance);

        psNewOp.executeUpdate();

				ResultSet rsId = psNewOp.getGeneratedKeys();
				rsId.next();
				opId = rsId.getInt(1);

				psNewOp.close();

        updateCompte(con, base, baseDossier, debit, credit, numeroCompte);
      }

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  private void updateOperation(Connection con, String base, String baseDossier) {
    try {

      String anc_numeroCompte = "";
      double anc_debit = 0;
      double anc_credit = 0;

      String reqOperation = "select Numero_Compte, Montant_D, Montant_C from "+ base +".OPERATION where Op_Id="+ opId;
      Statement stt = con.createStatement();
      ResultSet rset = stt.executeQuery(reqOperation);

      if (rset.next()) {
        anc_numeroCompte = rset.getString("Numero_Compte");
        anc_debit = 0 - rset.getDouble("Montant_D");
        anc_credit = 0 - rset.getDouble("Montant_C");
      }

			stt.close();

      double debit = getMontantD();
      double credit = getMontantC();

      String majOperation = "update "+ base +".OPERATION set "
													+ "Num_Ligne=?, Numero_Compte=?, Montant_D=?, Montant_C=?, Libelle=?, Num_Piece=?,"
													+ "Contrepartie=?, Type=?, Raccourci=?, Lettre=?, Date_Op=?, Date_Echeance=? "
													+ "where Op_Id=?";

			PreparedStatement psMajOp = con.prepareStatement(majOperation);

			psMajOp.setInt(1, numLigne);
			psMajOp.setString(2, numeroCompte);
			psMajOp.setDouble(3, debit);
			psMajOp.setDouble(4, credit);
			psMajOp.setString(5, libelle);
			psMajOp.setString(6, numPiece);
			psMajOp.setString(7, contrepartie);
			psMajOp.setString(8, type);
			psMajOp.setString(9, raccourci);
			psMajOp.setString(10, lettre);
			psMajOp.setLong(11, dateOp);
			psMajOp.setLong(12, dateEcheance);
			psMajOp.setInt(13, opId);

			psMajOp.executeUpdate();

			psMajOp.close();

      updateCompte(con, base, baseDossier, debit, credit, numeroCompte);
      updateCompte(con, base, baseDossier, anc_debit, anc_credit, anc_numeroCompte);

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  private void deleteOperation(Connection con, String base, String baseDossier) {
    try {

      String numeroCompte = "";
      double debit = 0;
      double credit = 0;

      String reqOperation = "select Numero_Compte, Montant_D, Montant_C from "+ base +".OPERATION where Op_Id="+ opId;
      Statement stt = con.createStatement();
      ResultSet rset = stt.executeQuery(reqOperation);

      if (rset.next()) {
        numeroCompte = rset.getString("Numero_Compte");
        debit = 0 - rset.getDouble("Montant_D");
        credit = 0 - rset.getDouble("Montant_C");
      }

      String supOperation = "delete from "+ base +".OPERATION where Op_Id="+ opId;

      stt.executeUpdate(supOperation);

			stt.close();

      updateCompte(con, base, baseDossier, debit, credit, numeroCompte);

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


  private void updateCompte(Connection con, String base, String baseDossier, double debit, double credit, String numeroCompte) {

    double solde = credit - debit;

    String majDebit = "Total_Debit=Total_Debit";
    if (debit >= 0) {
      majDebit += "+"+ debit;
    }
		else {
			majDebit += debit;
		}

    String majCredit = "Total_Credit=Total_Credit";
    if (credit >= 0) {
      majCredit += "+"+ credit;
    }
		else {
			majCredit += credit;
		}

    String majSolde = "Solde=Solde";
    if (solde >= 0) {
      majSolde += "+"+ solde;
    }
		else {
			majSolde += solde;
		}

    try {

			Statement stt = con.createStatement();

      String sqlReq = "select Numero_Compte, Type_Compte from "+ baseDossier +".COMPTE where Numero_Compte='"+ numeroCompte +"'";
      String comptegen = "";

      ResultSet rset = stt.executeQuery(sqlReq);

      if (rset.next()) {
        comptegen = rset.getString("Type_Compte");
      }


      String majCompte = "update "+ base +".SOLDE_COMPTE set "+ majCredit +","+ majDebit + ","+ majSolde +" where Numero_Compte='"+ numeroCompte +"'";

      if (comptegen.equals("C")) {
        majCompte += " or Numero_Compte='41100000'";
      }
			else if (comptegen.equals("F")) {
        majCompte += " or Numero_Compte='40100000'";
      }

      stt.executeUpdate(majCompte);

    } catch (SQLException sqle) {
      sqle.printStackTrace();
    }
  }


	// pour debugger
	public void printOperation() {

		System.out.println("Numero compte =\t"+ this.numeroCompte);
		System.out.println("Libelle =\t"+ this.libelle);
		System.out.println("Montant D =\t"+ this.montantD);
		System.out.println("Montant_C =\t"+ this.montantC);
	}


} // fin Operation
