/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Edition du formulaire de dclaration de TVA
 */

package org.opensi.compta.actions.etats;

import java.awt.Color;
import java.io.FileOutputStream;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

import org.experlog.openeas.api.Action;
import org.experlog.openeas.api.ServletRequest;
import org.experlog.openeas.api.Session;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.PageSize;
import com.lowagie.text.Phrase;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;


public class DeclarationTVA extends Action {


	private DecimalFormat numFormat = new DecimalFormat("0.00");


	private static final Font font11G = new Font(Font.HELVETICA , 10, Font.BOLD);
	private static final Font font11GB = new Font(Font.HELVETICA , 9, Font.BOLD, Color.WHITE);
	private static final Font font12GB = new Font(Font.HELVETICA , 11, Font.BOLD, Color.WHITE);
	private static final Font font8G = new Font(Font.HELVETICA , 8, Font.BOLD);
	private static final Font font9G = new Font(Font.HELVETICA , 9, Font.BOLD);
	private static final Font font8N = new Font(Font.HELVETICA , 8, Font.NORMAL);
	private static final Font font6N = new Font(Font.HELVETICA , (float)6.5, Font.NORMAL);
	private static final Font font7I = new Font(Font.HELVETICA , 7, Font.ITALIC);
	private static final Font font7G = new Font(Font.HELVETICA , 7, Font.BOLD);
	private static final Font font7N = new Font(Font.HELVETICA , 7, Font.NORMAL);


	private static final Color BLEU = new Color(52,84,193);
	private static final Color CIEL = new Color(215,226,255);
	private static final Color ROSE = new Color(255,180,195);


	Document document;

	PdfPCell cellText;
	PdfPCell cellForm;
	PdfPCell cellCiel;
	PdfPCell cellRose;


	public boolean serverSide() { return true; }

	public boolean service(Session s, ServletRequest r, Object info) throws Exception {



		PdfPTable pptInit = new PdfPTable(1);


		cellText = new PdfPCell(pptInit.getDefaultCell());
		cellText.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellText.setBorder(PdfPCell.NO_BORDER);
		cellText.setBorderColor(BLEU);
		cellText.setPaddingLeft(8);

		cellForm = new PdfPCell(pptInit.getDefaultCell());
		cellForm.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);
		cellForm.setBorderColor(BLEU);

		cellCiel = new PdfPCell(pptInit.getDefaultCell());
		cellCiel.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellCiel.setBorder(PdfPCell.LEFT);
		cellCiel.setBackgroundColor(CIEL);
		cellCiel.setBorderColor(BLEU);
		cellCiel.setPaddingRight(0);

		cellRose = new PdfPCell(pptInit.getDefaultCell());
		cellRose.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);
		cellRose.setBackgroundColor(ROSE);
		cellRose.setBorderColor(BLEU);
		cellRose.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);


		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		dfs.setGroupingSeparator(' ');

		numFormat.setDecimalFormatSymbols(dfs);
		numFormat.setNegativeSuffix("");
		numFormat.setNegativePrefix("-");
		numFormat.setPositiveSuffix(" ");
		numFormat.setGroupingUsed(true);
		numFormat.setGroupingSize(3);

		document = new Document(PageSize.A4, 18, 18, 18, 18);

		String chemin = s.getAppConfiguration().get("opensi.absdir") + "/pdf/";

		PdfWriter.getInstance(document, new FileOutputStream(chemin + "declare.pdf"));

		//MyPageEvent event = new MyPageEvent(s, dossierId, debutExercice, finExercice);

		//writer.setPageEvent(event);

		document.open();

		// page 1/2






		document.add(titreBleu("MODALIT\u00C9S DE D\u00C9CLARATION ET DE PAIEMENT (voir notice)", 0));

		putCadre2();

		putCadre3();


		document.newPage();

		// page 2/2

		cellText = new PdfPCell(pptInit.getDefaultCell());
		cellText.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellText.setBorder(PdfPCell.NO_BORDER);
		cellText.setBorderColor(BLEU);
		cellText.setPaddingLeft(8);

		cellForm = new PdfPCell(pptInit.getDefaultCell());
		cellForm.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);
		cellForm.setBorderColor(BLEU);

		cellCiel = new PdfPCell(pptInit.getDefaultCell());
		cellCiel.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellCiel.setBorder(PdfPCell.LEFT);
		cellCiel.setBackgroundColor(CIEL);
		cellCiel.setBorderColor(BLEU);
		cellCiel.setPaddingRight(0);

		cellRose = new PdfPCell(pptInit.getDefaultCell());
		cellRose.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);
		cellRose.setBackgroundColor(ROSE);
		cellRose.setBorderColor(BLEU);
		cellRose.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);



		putNeant();

		putSpace(document,5);

		document.add(titreBleu("A MONTANT DES OP\u00C9RATIONS R\u00C9ALIS\u00C9ES", 3));

		putCadreA();

		document.add(titreBleu("B D\u00C9COMPTE DE LA TVA \u00C0 PAYER", 3));

		putCadreB();

		putCadreB2();

		putCreditTaxe();

		putTexteBas();

		/*stt.close();
		s.closeConnection(con, null);*/

		document.close();

		return true;


	} // fin processRequest



	private void putCadre2() throws DocumentException {

		PdfPTable pptCadre2 = new PdfPTable(1);
		pptCadre2.setWidthPercentage(100);

		PdfPCell cell = new PdfPCell(pptCadre2.getDefaultCell());

		cell.setFixedHeight(25);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setPaddingLeft(10);
		cell.setBorderColor(BLEU);

		Phrase p = new Phrase();
		p.add(new Chunk("L'arrondissement des bases et des cotisations s'effectue \u00E0 l'euro le plus proche ", font8G));
		p.add(new Chunk("(cf. les r\u00E8gles d'arrondissement, page 2 de la notice n 3310 NOT-K).", font8N));

		cell.setPhrase(p);

		pptCadre2.addCell(cell);

		document.add(pptCadre2);
	}


	private void putCadre3() throws DocumentException {

		PdfPTable pptCadre3 = new PdfPTable(8);
		pptCadre3.setWidthPercentage(100);

		int widths[] = {23,10,15,5,7,15,10,15};
		pptCadre3.setWidths(widths);

		//ligne 1
		PdfPCell cell = new PdfPCell(titreBleu("PAIEMENT, DATE, SIGNATURE", 0));
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);
		cell.setColspan(2);
		pptCadre3.addCell(cell);

		//cell = new PdfPCell(pptCadre3.getDefaultCell());

		cellRose.setColspan(6);
		cellRose.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cellRose.setPhrase(new Phrase("R\u00C9SERV\u00C9 \u00C0 L'ADMINISTRATION", font9G));
		pptCadre3.addCell(cellRose);

		//ligne2
		cellText.setPhrase(new Phrase("Date :", font6N));
		pptCadre3.addCell(cellText);
		cellText.setPhrase(new Phrase("Signature :", font6N));
		pptCadre3.addCell(cellText);

		cellCiel.setPhrase(new Phrase("Somme :", font6N));
		pptCadre3.addCell(cellCiel);

		cellCiel.setPhrase(new Phrase("Date :", font6N));
		pptCadre3.addCell(cellCiel);

		cellText.setColspan(2);
		cellText.setPhrase(new Phrase("", font6N));
		pptCadre3.addCell(cellText);

		cellCiel.setColspan(2);
		cellCiel.setPhrase(new Phrase("Pnalits :", font6N));
		pptCadre3.addCell(cellCiel);

		//ligne 3

		cellText.setPhrase(new Phrase("", font6N));
		pptCadre3.addCell(cellText);

		cellText.setColspan(1);
		cellText.setPhrase(new Phrase("", font6N));
		pptCadre3.addCell(cellText);

		cell = new PdfPCell(pptCadre3.getDefaultCell());
		cell.setColspan(2);
		cell.setPhrase(new Phrase("N PEC......", font6N));
		pptCadre3.addCell(cell);

		cell.setColspan(1);
		cell.setPhrase(new Phrase(""));
		pptCadre3.addCell(cell);

		cell.setPhrase(new Phrase("Taux 5%", font6N));
		pptCadre3.addCell(cell);

		cell.setPhrase(new Phrase("9005", font6N));
		pptCadre3.addCell(cell);

		document.add(pptCadre3);
	}



	private void putNeant() throws DocumentException {

		PdfPTable pptNeant = new PdfPTable(3);
		pptNeant.setWidthPercentage(100);
		float[] widths = {88,5,7};
		pptNeant.setWidths(widths);

		PdfPCell cell = new PdfPCell(pptNeant.getDefaultCell());

		cell.setBorderColor(BLEU);
		cell.setFixedHeight(16);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);

		String texte = "Si vous n'avez  remplir aucune ligne de ce formulaire (dclaration \u00AB nant \u00BB), veuillez cocher la case  droite";

		cell.setPaddingLeft(5);
		cell.setPhrase(new Phrase(texte, font9G));
		pptNeant.addCell(cell);
		cell.setPaddingLeft(0);

		cell.setBackgroundColor(ROSE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setPhrase(new Phrase("0010", font8N));
		pptNeant.addCell(cell);

		cell = new PdfPCell(makeCase());
		cell.setPadding(3);
		cell.setPaddingLeft(12);
		cell.setPaddingRight(12);
		cell.setBorderColor(BLEU);

		pptNeant.addCell(cell);

		document.add(pptNeant);
	}


	private void putCadreA() throws DocumentException {

		PdfPTable pptCadreA = new PdfPTable(2);
		pptCadreA.setWidthPercentage(100);

		PdfPCell c;
		PdfPCell cell = new PdfPCell(pptCadreA.getDefaultCell());

		float[] widths = {(float)4.8,(float)55.2,10,30};

		PdfPTable pptGauche = new PdfPTable(4);
		pptGauche.setWidths(widths);

		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPhrase(new Phrase(""));
		cell.setBackgroundColor(CIEL);
		pptGauche.addCell(cell);

		cell.setColspan(3);
		cell.setPhrase(new Phrase("OP\u00C9RATIONS IMPOSABLES (H.T.)", font11G));
		pptGauche.addCell(cell);

		cell.setColspan(1);

		// new
		pptGauche.addCell(caseCiel("01"));
		c = caseText("Ventes, prestations de services"+ repT(7));
		c.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED_ALL);
		pptGauche.addCell(c);
		pptGauche.addCell(caseForm(""));
		pptGauche.addCell(caseForm(""));

		pptGauche.addCell(caseCiel("02"));
		c = caseText("Autres oprations imposables"+ repT(8));
		c.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED_ALL);
		pptGauche.addCell(c);
		pptGauche.addCell(caseRose("0981"));
		pptGauche.addCell(caseForm(""));

		pptGauche.addCell(caseCiel("03"));
		pptGauche.addCell(caseSimple("Acquisitions intracommunautaires"+ repT(5)));
		pptGauche.addCell(caseRose("0031"));
		pptGauche.addCell(caseForm(""));

		pptGauche.addCell(caseCiel(""));

		pptGauche.addCell(caseSimple("(dont ventes  distance et/ou oprations"));
		pptGauche.addCell(caseRose(""));
		pptGauche.addCell(caseSimple(""));

		pptGauche.addCell(caseCiel(""));

		pptGauche.addCell(caseSimple("de montage :"));
		pptGauche.addCell(caseRose(""));
		pptGauche.addCell(caseSimple(""));

		pptGauche.addCell(caseCiel(""));

		pptGauche.addCell(caseSimple("....................)"));
		pptGauche.addCell(caseRose(""));
		pptGauche.addCell(caseSimple(""));

		pptGauche.addCell(caseCiel("3A"));

		pptGauche.addCell(caseSimple("Livraisons de gaz naturel ou"));
		pptGauche.addCell(caseRose(""));
		pptGauche.addCell(caseSimple(""));

		pptGauche.addCell(caseCiel(""));

		pptGauche.addCell(caseSimple("d'lectricit imposables en France"+ repT(5)));
		pptGauche.addCell(caseRose("0030"));
		pptGauche.addCell(caseSimple(""));

		pptGauche.addCell(caseCiel("3B"));

		pptGauche.addCell(caseSimple("Rgularisations"+ repT(18)));
		pptGauche.addCell(caseRose("0036"));
		pptGauche.addCell(caseSimple(""));

		pptGauche.addCell(caseCiel(""));

		pptGauche.addCell(caseSimple("(Important : cf. notice)"));
		pptGauche.addCell(caseRose(""));
		pptGauche.addCell(caseSimple(""));

		PdfPTable pptDroite = new PdfPTable(4);
		pptDroite.setWidths(widths);

		pptDroite.addCell(caseCiel(""));
		cell.setColspan(3);
		cell.setPhrase(new Phrase("OP\u00C9RATIONS NON IMPOSABLES", font11G));
		pptDroite.addCell(cell);
		cell.setColspan(1);

		pptDroite.addCell(caseCiel("04"));

		pptDroite.addCell(caseSimple("Exportations hors CE"));
		pptDroite.addCell(caseRose("0032"));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel("05"));

		pptDroite.addCell(caseSimple("Autres oprations non imposables"));
		pptDroite.addCell(caseRose("0033"));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel("06"));

		pptDroite.addCell(caseSimple("Livraisons intracommunautaires"));
		pptDroite.addCell(caseRose("0034"));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel("6A"));

		pptDroite.addCell(caseSimple("Livraisons de gaz naturel ou d'lectricit"));
		pptDroite.addCell(caseRose(""));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel(""));

		pptDroite.addCell(caseSimple("non imposables en France"));
		pptDroite.addCell(caseRose("0029"));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel("07"));

		pptDroite.addCell(caseSimple("Achats en franchise"));
		pptDroite.addCell(caseRose("0037"));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel(""));

		pptDroite.addCell(caseSimple(""));
		pptDroite.addCell(caseRose(""));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel(""));

		pptDroite.addCell(caseSimple(""));
		pptDroite.addCell(caseRose(""));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel("7B"));

		pptDroite.addCell(caseSimple("Rgularisations"));
		pptDroite.addCell(caseRose("0039"));
		pptDroite.addCell(caseSimple(""));

		pptDroite.addCell(caseCiel(""));

		pptDroite.addCell(caseSimple("(Important : cf. notice"));
		pptDroite.addCell(caseRose(""));
		pptDroite.addCell(caseSimple(""));


		cell = new PdfPCell(pptGauche);
		cell.setPadding(0);
		cell.setBorderColor(BLEU);
		//cell.setBorder(PdfPCell.NO_BORDER);

		pptCadreA.addCell(cell);

		cell = new PdfPCell(pptDroite);
		cell.setPadding(0);
		cell.setBorderColor(BLEU);
		//cell.setBorder(PdfPCell.);

		pptCadreA.addCell(cell);

		document.add(pptCadreA);

	}



	private void putCadreB() throws DocumentException {

		PdfPTable pptCadreB = new PdfPTable(5);
		pptCadreB.setWidthPercentage(100);
		float[] widths = {(float)2.4,(float)62.6,5,15,15};
		pptCadreB.setWidths(widths);


		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellCiel.setBorder(PdfPCell.NO_BORDER);
		cellCiel.setPhrase(new Phrase("TVA BRUTE", font11G));
		pptCadreB.addCell(cellCiel);

		cellCiel.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cellCiel.setBorder(PdfPCell.BOTTOM);
		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellCiel.setBorder(PdfPCell.LEFT + PdfPCell.BOTTOM);
		cellCiel.setPhrase(new Phrase("Base hors taxe", font11G));
		pptCadreB.addCell(cellCiel);

		cellCiel.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellCiel.setPhrase(new Phrase("Taxe due", font11G));
		pptCadreB.addCell(cellCiel);
		cellCiel.setBorder(PdfPCell.LEFT);

		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Oprations ralises en France mtropolitaine", font8G));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("08", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Taux normal 19,6 %", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0206", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("09", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Taux rduit 5,5 %", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0105", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("9B", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("......", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Oprations ralises dans les DOM", font8G));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("10", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Taux normal 8,5 %", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0201", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("11", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Taux rduit 2,1 %", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0100", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("12", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase(".........", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Oprations imposables  un autre taux", font8G));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("(France mtropolitaine ou DOM)", font8G));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("13", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Ancien taux", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0900", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("14", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Oprations imposables  un particulier (dcompte effectu sur annexe 3310 A)", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellRose.setPhrase(new Phrase("0950", font8N));
		pptCadreB.addCell(cellRose);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);


		document.add(pptCadreB);
	}



	private void putCadreB2() throws DocumentException {

		PdfPTable pptCadreB = new PdfPTable(4);
		pptCadreB.setWidthPercentage(100);
		float[] widths = {(float)2.4,(float)77.6,5,15};
		pptCadreB.setWidths(widths);


		cellCiel.setPhrase(new Phrase("15", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("TVA antrieurement dduite  reverser", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setBorder(PdfPCell.BOX - PdfPCell.BOTTOM);
		cellRose.setPhrase(new Phrase("0600", font8N));
		pptCadreB.addCell(cellRose);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);



		// B3....

		PdfPTable pptCadreB3 = new PdfPTable(3);
		pptCadreB3.setWidthPercentage(100);
		float[] widthsB3 = {(float)2.4,(float)47.6,50};
		pptCadreB3.setWidths(widthsB3);

		PdfPTable pptGauche = new PdfPTable(1);
		pptGauche.setWidthPercentage(100);

		PdfPCell cell = new PdfPCell(pptGauche.getDefaultCell());
		cell.setBackgroundColor(ROSE);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPhrase(new Phrase("La ligne 11 ne concerne que les DOM.\nLes autres oprations relevant du taux\nde 2,1 % continuent d'tre dclares\nsur l'annexe 3310 A.", font11G));

		pptGauche.addCell(cell);

		float[] widthsD = {(float)4.8,(float)55.2,10,30};

		PdfPTable pptDroite = new PdfPTable(4);
		pptDroite.setWidths(widthsD);

		cellCiel.setPhrase(new Phrase("16", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Total de la TVA brute due (lignes 08  15)", font7G));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase(""));
		pptDroite.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);


		cellCiel.setPhrase(new Phrase("17", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Dont TVA sur acquisitions intracommunautaires", font7N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("0035", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptDroite.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("18", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Dont TVA sur oprations  destination de Monaco", font7N));
		pptDroite.addCell(cellText);

		cellRose.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellRose.setPhrase(new Phrase("0038", font8N));
		pptDroite.addCell(cellRose);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase(""));
		pptDroite.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);



		cellCiel.setPhrase(new Phrase(""));
		pptCadreB3.addCell(cellCiel);


		cell = new PdfPCell(pptGauche);
		cell.setPadding(10);
		cell.setPaddingRight(40);
		cell.setBorder(PdfPCell.NO_BORDER);

		pptCadreB3.addCell(cell);

		cell = new PdfPCell(pptDroite);
		cell.setPadding(0);
		cell.setBorder(PdfPCell.NO_BORDER);
		//cell.setBorderColor(BLEU);
		//cell.setBorder(PdfPCell.LEFT);

		pptCadreB3.addCell(cell);


		cell = new PdfPCell(pptCadreB3);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);
		cell.setColspan(4);

		pptCadreB.addCell(cell);

		// B3....


		cellCiel.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellCiel);

		cellCiel.setColspan(3);
		cellCiel.setBorder(PdfPCell.RIGHT);
		cellCiel.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cellCiel.setPhrase(new Phrase("TVA D\u00C9DUCTIBLE", font11G));
		pptCadreB.addCell(cellCiel);
		cellCiel.setBorder(PdfPCell.LEFT);

		cellText.setColspan(1);


		cellCiel.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cellCiel.setColspan(1);
		cellCiel.setPhrase(new Phrase("19", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Biens constituant des immobilisations", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setBorder(PdfPCell.BOX - PdfPCell.BOTTOM);
		cellRose.setPhrase(new Phrase("0703", font8N));
		pptCadreB.addCell(cellRose);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.BOTTOM);
		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);


		cellCiel.setPhrase(new Phrase("20", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Autres biens et services", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0702", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("21", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Autre TVA  dduire", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("0059", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("(dont rgularisation sur de la TVA collecte [cf. notice]..........)", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setPhrase(new Phrase("", font8N));
		pptCadreB.addCell(cellRose);

		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("22", font8N));
		pptCadreB.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Report du crdit apparaissant ligne 27 de la prcdente dclaration", font8N));
		pptCadreB.addCell(cellText);

		cellRose.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellRose.setPhrase(new Phrase("8001", font8N));
		pptCadreB.addCell(cellRose);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase(""));
		pptCadreB.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);


		// B4....

		pptCadreB3 = new PdfPTable(3);
		pptCadreB3.setWidthPercentage(100);
		//int[] widthsB3 = {3,47,50};
		pptCadreB3.setWidths(widthsB3);

		pptGauche = new PdfPTable(2);
		pptGauche.setWidthPercentage(100);
		//int[] widthsG = {};
		//pptGauche.setWidths(widthsG);

		cell = new PdfPCell(pptGauche.getDefaultCell());
		//cell.setPaddingLeft(10);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPaddingRight(0);
		cell.setPhrase(new Phrase("Indiquer ici le pourcentage de dduction\napplicable pour la priode s'il est diffrent\nde 100 %", font7I));

		pptGauche.addCell(cell);

		//cell.setPhrase(new Phrase("% ", font8N));

		cell = new PdfPCell(makeCasePourcent());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(10);
		cell.setPaddingLeft(10);
		cell.setPaddingRight(50);

		pptGauche.addCell(cell);

		//int[] widthsD = {6,52,12,30};

		pptDroite = new PdfPTable(4);
		pptDroite.setWidths(widthsD);

		cellCiel.setPhrase(new Phrase("24", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setColspan(2);
		cellText.setPhrase(new Phrase("Total TVA dductible (lignes 19  22)", font8G));
		pptDroite.addCell(cellText);
		cellText.setColspan(1);

		//cellForm.setBorder(PdfPCell.BOX - cellForm.);
		cellForm.setPhrase(new Phrase(""));
		pptDroite.addCell(cellForm);
		//cellForm.setBorder(PdfPCell.LEFT + cellForm.RIGHT);


		cellCiel.setPhrase(new Phrase("23", font8N));
		pptCadreB3.addCell(cellCiel);


		cell = new PdfPCell(pptGauche);
		cell.setPadding(5);
		cell.setBorder(PdfPCell.NO_BORDER);

		pptCadreB3.addCell(cell);

		cell = new PdfPCell(pptDroite);
		cell.setPadding(0);
		cell.setBorderColor(BLEU);
		cell.setBorder(PdfPCell.LEFT);

		pptCadreB3.addCell(cell);


		cell = new PdfPCell(pptCadreB3);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);
		cell.setColspan(4);

		pptCadreB.addCell(cell);

		// B4....


		document.add(pptCadreB);
	}





	private void putCreditTaxe() throws DocumentException {


		PdfPTable pptCT = new PdfPTable(2);
		pptCT.setWidthPercentage(100);


		PdfPCell cell = new PdfPCell(pptCT.getDefaultCell());

		float[] widths = {(float)4.8,(float)55.2,10,30};

		PdfPTable pptGauche = new PdfPTable(4);
		pptGauche.setWidths(widths);


		cellCiel.setPhrase(new Phrase("25", font8N));
		pptGauche.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Crdit de TVA (ligne 24 - ligne 16)", font8N));
		pptGauche.addCell(cellText);

		cellRose.setPhrase(new Phrase("0705", font8N));
		pptGauche.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellForm);

		cellCiel.setPhrase(new Phrase("26", font8N));
		pptGauche.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Remboursement demand", font8N));
		pptGauche.addCell(cellText);

		cellRose.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellForm);

		cellCiel.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellCiel);

		cellText.setPhrase(new Phrase("sur formulaire n 3519 joint", font8N));
		pptGauche.addCell(cellText);

		cellRose.setPhrase(new Phrase("8002", font8N));
		pptGauche.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellForm);

		cellCiel.setPhrase(new Phrase("27", font8N));
		pptGauche.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Crdit  reporter (ligne 25 - ligne 26)", font8N));
		pptGauche.addCell(cellText);

		cellRose.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellRose.setPhrase(new Phrase("8003", font8N));
		pptGauche.addCell(cellRose);
		cellRose.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);


		cellCiel.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellCiel);

		cellText.setColspan(3);
		cellText.setPhrase(new Phrase("(Cette somme est  reporter ligne 22\nde la prochaine dclaration)", font7I));
		pptGauche.addCell(cellText);


		cellCiel.setBorder(PdfPCell.LEFT + PdfPCell.BOTTOM);
		cellCiel.setPhrase(new Phrase("", font8N));
		pptGauche.addCell(cellCiel);
		cellCiel.setBorder(PdfPCell.LEFT);

		cellText.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED);
		cellText.setBorder(PdfPCell.BOTTOM);
		cellText.setPhrase(new Phrase("Attention ! Une situation de TVA crditrice (ligne 25 servie) ne dispense pas du paiement des taxes assimiles dclares ligne 29.", font8G));
		pptGauche.addCell(cellText);
		cellText.setColspan(1);
		cellText.setBorder(PdfPCell.NO_BORDER);
		cellText.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);


		PdfPTable pptDroite = new PdfPTable(4);
		pptDroite.setWidths(widths);



		cellCiel.setPhrase(new Phrase("28", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("TVA nette due (ligne 16 - ligne 24)", font8N));
		pptDroite.addCell(cellText);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cellCiel.setPhrase(new Phrase("29", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Taxes assimiles calcules sur", font8N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Annexe n 3310 A", font8N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("9979", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);



		cellCiel.setPhrase(new Phrase("30", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Sommes  imputer, exprimes en euros,", font8N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);

		cellCiel.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("y compris acompte congs", font8N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("9989", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);


		cellCiel.setPhrase(new Phrase("31", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("Sommes  ajouter, exprimes en euros,", font8N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);

		cellCiel.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellCiel);

		cellText.setPhrase(new Phrase("y compris acompte congs", font8N));
		pptDroite.addCell(cellText);

		cellRose.setPhrase(new Phrase("9999", font8N));
		pptDroite.addCell(cellRose);

		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);


		cellCiel.setBorder(PdfPCell.LEFT + PdfPCell.TOP);
		cellCiel.setPhrase(new Phrase("32", font8N));
		pptDroite.addCell(cellCiel);
		cellCiel.setBorder(PdfPCell.LEFT);

		cellText.setBorder(PdfPCell.TOP);
		cellText.setPhrase(new Phrase("Total  payer (lignes 28 + 29 - 30 + 31)", font8G));
		pptDroite.addCell(cellText);
		cellText.setBorder(PdfPCell.NO_BORDER);

		cellForm.setColspan(2);
		cellForm.setBorder(PdfPCell.BOX - PdfPCell.BOTTOM);
		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);


		cellCiel.setBorder(PdfPCell.LEFT + PdfPCell.BOTTOM);
		cellCiel.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellCiel);
		cellCiel.setBorder(PdfPCell.LEFT);

		cellText.setBorder(PdfPCell.BOTTOM);
		cellText.setPhrase(new Phrase("(N'oubliez pas de joindre le rglement correspondant)", font7I));
		pptDroite.addCell(cellText);
		cellText.setBorder(PdfPCell.NO_BORDER);

		cellForm.setBorder(PdfPCell.BOX - PdfPCell.TOP);
		cellForm.setPhrase(new Phrase("", font8N));
		pptDroite.addCell(cellForm);
		cellForm.setColspan(1);
		cellForm.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);

		cell = new PdfPCell(titreBleu("CR\u00C9DIT", 1));
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);

		pptCT.addCell(cell);

		cell = new PdfPCell(titreBleu("TAXE \u00C0 PAYER", 1));
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);

		pptCT.addCell(cell);

		cell = new PdfPCell(pptGauche);
		cell.setPadding(0);
		//cell.setBorderColor(BLEU);
		cell.setBorder(PdfPCell.NO_BORDER);

		pptCT.addCell(cell);

		cell = new PdfPCell(pptDroite);
		cell.setPadding(0);
		//cell.setBorderColor(BLEU);
		cell.setBorder(PdfPCell.NO_BORDER);

		pptCT.addCell(cell);

		document.add(pptCT);



	}

	private PdfPCell caseCiel(String str) {

		PdfPCell cell	= new PdfPCell(new Phrase(str, font8N));

		cell.setBackgroundColor(CIEL);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPaddingLeft(0);
		cell.setPaddingRight(0);

		return cell;
	}



	private PdfPCell caseRose(String str) {

		PdfPCell cell	= new PdfPCell(new Phrase(str, font8N));

		cell.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);
		cell.setBackgroundColor(ROSE);
		cell.setBorderColor(BLEU);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);

		return cell;
	}

	private PdfPCell caseSimple(String str) {

		PdfPCell cell	= new PdfPCell(cellText);
		cell.setPhrase(new Phrase(str, font8N));

		return cell;
	}

	private PdfPCell caseText(String str) {

		PdfPCell cell	= new PdfPCell(cellText);
		cell.setPhrase(new Phrase(str, font8N));

		return cell;
	}

	private PdfPCell caseForm(String str) {

		PdfPCell cell	= new PdfPCell(cellForm);
		cell.setPhrase(new Phrase(str, font8N));

		return cell;
	}







	private PdfPTable titreBleu(String titre, int nbH) throws DocumentException {

		PdfPTable pptTitre = new PdfPTable(1);
		pptTitre.setWidthPercentage(100);

		PdfPCell cell = new PdfPCell(pptTitre.getDefaultCell());
		cell.setBorderColor(BLEU);
		cell.setBackgroundColor(BLEU);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setPaddingLeft(6);
		cell.setPaddingTop(1);
		cell.setPaddingBottom(3);

		Phrase p;

		if (nbH>0 && nbH<titre.length()) {
			Chunk chk1 = new Chunk(titre.substring(0,nbH), font12GB);
			Chunk chk2 = new Chunk(titre.substring(nbH,titre.length()), font11GB);
			p = new Phrase();
			p.add(chk1);
			p.add(chk2);
		}
		else {
			p = new Phrase(titre, font11GB);
		}

		cell.setPhrase(p);
		pptTitre.addCell(cell);

		return pptTitre;
	}


	private void putTexteBas() throws DocumentException {

		PdfPTable pptTexte = new PdfPTable(1);
		pptTexte.setWidthPercentage(100);

		PdfPCell cell = new PdfPCell(pptTexte.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPaddingLeft(0);
		cell.setPaddingRight(0);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED);

		String texte = "Les dispositions des articles 34, 35, 36 de la loi n 78-17 du 6 janvier 1978 modifie "
			+"relative  l'informatique, aux fichiers et aux liberts s'appliquent : elles garantissent, "
			+"pour les donnes vous concernant, auprs du centre des impts, un droit de rectification.\n"
			+"Si vous ralisez des oprations intracommunautaires, pensez  la dclaration d'changes de "
			+"biens  souscrire auprs de la Direction Gnrale des Douanes et des Droits indirects (cf. "
			+"notice de la dclaration CA3).";

		cell.setPhrase(new Phrase(texte, font6N));
		pptTexte.addCell(cell);

		document.add(pptTexte);
	}



	private void putSpace(Document document, int space) throws DocumentException {

		PdfPTable sectionSpace = new PdfPTable(1);
		sectionSpace.setWidthPercentage(100);

		PdfPCell cell = new PdfPCell(sectionSpace.getDefaultCell());

		cell.setPhrase(new Phrase(""));
		cell.setFixedHeight(space);
		cell.setBorder(PdfPCell.NO_BORDER);
		sectionSpace.addCell(cell);

		document.add(sectionSpace);
	}


	private PdfPTable makeCase() {

		PdfPTable pptCase = new PdfPTable(1);

		PdfPCell cell = new PdfPCell(pptCase.getDefaultCell());
		cell.setBorder(PdfPCell.BOX);

		cell.setPhrase(new Phrase(""));

		pptCase.addCell(cell);

		return pptCase;
	}


	private PdfPTable makeCasePourcent() {

		PdfPTable pptCase = new PdfPTable(1);

		PdfPCell cell = new PdfPCell(pptCase.getDefaultCell());
		cell.setBorder(PdfPCell.BOX);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		cell.setVerticalAlignment(PdfPCell.ALIGN_BOTTOM);
		cell.setPaddingRight(4);

		cell.setPhrase(new Phrase("%", font8N));

		pptCase.addCell(cell);

		return pptCase;
	}



	private String repT(int nb) {

		String str = "";

		for (int i=0;i<nb;i++) {
			str += " .";
		}

		return str;
	}



} // fin DeclarationTVA
