/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Gnration d'un avoir au format standard
 */

package org.opensi.compta.actions.etats;


import java.awt.Color;
import java.sql.Date;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;


import org.opensi.util.pdf.MultiPagesDocument;
import org.opensi.util.pdf.PdfWidths;
import org.opensi.util.pdf.TableBuilder;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.Phrase;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;


public class PdfPlanComptableDossier extends MultiPagesDocument {


	// dfinition des polices
private static final Font fontAdrFacture = new Font(Font.HELVETICA, 11, Font.BOLD);
private static final Font fontAdrSociete = new Font(Font.HELVETICA, 10, Font.NORMAL);
private static final Font fontAdrSocieteGras = new Font(Font.HELVETICA, 11, Font.BOLD);
private static final Font fontAdrLiv = new Font(Font.HELVETICA, 8, Font.NORMAL);
private static final Font fontAdrContact = new Font(Font.HELVETICA, 7, Font.NORMAL);
private static final Font fontTitre = new Font(Font.HELVETICA , 16, Font.BOLD, Color.WHITE);
private static final Font fontWeb = new Font(Font.HELVETICA, 10, Font.ITALIC);
private static final Font fontCadre1 = new Font(Font.HELVETICA, 10, Font.BOLD);
private static final Font fontCadre1Petit = new Font(Font.HELVETICA, 8, Font.NORMAL);
private static final Font fontCadre1PetitGras = new Font(Font.HELVETICA, 8, Font.BOLD);
private static final Font fontEnteteColonne = new Font(Font.HELVETICA, 9, Font.BOLD);
private static final Font fontInfosFact = new Font(Font.HELVETICA , 10, Font.NORMAL);
private static final Font fontLegale = new Font(Font.HELVETICA , 7, Font.NORMAL);
private static final Font fontLine = new Font(Font.HELVETICA , 8, Font.NORMAL);
private static final Font fontModalites = new Font(Font.HELVETICA , 10, Font.BOLD);
private static final Font fontTotaux = new Font(Font.HELVETICA , 10, Font.NORMAL);
private static final Font fontTTC = new Font(Font.HELVETICA , 10, Font.BOLD);
private static final Font fontArtNom = new Font(Font.HELVETICA, 9, Font.ITALIC);
private static final Font fontSmallTitre = new Font(Font.HELVETICA , 8, Font.BOLD+Font.ITALIC, Color.WHITE);


private DecimalFormat numFormat = new DecimalFormat("0.00");
private DecimalFormat numFormat4 = new DecimalFormat("0.00##");
private DecimalFormat numFormatCap = new DecimalFormat("0.##");
private SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");


private float[] widthsCols;
private int nbCols;


private boolean numCompte;
private boolean intitule;
private boolean type;
private boolean contrepartie;

private DataPlanComptableDossier lignesdata;





  public PdfPlanComptableDossier(DataPlanComptableDossier lignesdata, String tempdir) throws Exception {

		super(tempdir);
		this.lignesdata=lignesdata;
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		dfs.setGroupingSeparator(' ');
		
		numFormat.setDecimalFormatSymbols(dfs);
		numFormat.setNegativeSuffix("");
		numFormat.setNegativePrefix("-");
		numFormat.setPositiveSuffix(" ");
		numFormat.setGroupingUsed(true);
		numFormat.setGroupingSize(3);		
		
		numFormat4.setDecimalFormatSymbols(dfs);
		numFormat4.setGroupingUsed(true);
		numFormat4.setGroupingSize(3);
		
		numFormatCap.setDecimalFormatSymbols(dfs);
		numFormatCap.setGroupingUsed(true);
		numFormatCap.setGroupingSize(3);		
		
		

		//colonne (poid,taille en %)=> nombre de colonnes du tableau de datas
		PdfWidths pw = new PdfWidths();		
		pw.addColonneVariableMin(2,20);
		pw.addColonneVariableMin(4,40);
		pw.addColonneVariableMin(4,20);
		pw.addColonneVariableMin(4,20);
	
		
		nbCols = pw.getNbCols();
		widthsCols = pw.getWidthsCols();
	
	}


	
	/* implmentation des fonctions de MultiPagesDocument */
	
	public PdfPTable firstHeader() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();		
		tb.add(makeEntete());
		tb.add(makeSpace(5));
		tb.add(makeTitre());
		tb.add(makeSpace(5));
		tb.add(makeEntetesColonnes());		
		
		return tb.getTable();	
	}
	
	
	public PdfPTable middleHeader(int lastLine) throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(makeEntetesColonnes());	
		return tb.getTable();
	}
	
	
	public PdfPTable lastFooter() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
	
		return tb.getTable();
	}	
	
	
	public PdfPTable middleFooter(int nbLinesPage, int lastLine) throws DocumentException {
	
		
		
		return lastFooter();		
	}
	
	
	public PdfPTable bodyLine(int numLine) throws DocumentException {
	
		
		LignePlanDossier lp = lignesdata.lignes.get(numLine);

		return makeLinePlanD(lp, numLine %2==0);
	}
	
	
	public PdfPTable emptyLine(float emptyHeight) throws DocumentException {
		
		PdfPTable pptSpace = new PdfPTable(nbCols);
		pptSpace.setWidthPercentage(100);
		pptSpace.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPaddingBottom(0);
		cell.setPaddingTop(0);
		cell.setFixedHeight(emptyHeight);		
		cell.setPhrase(new Phrase(""));		
	
		for (int i=0; i<nbCols; i++) {
			pptSpace.addCell(cell);
		}
	
		return pptSpace;
	}
	
	
	public int nbLines() {
	
		return lignesdata.getNbLignes();
	}
	
	


	/* mthodes internes */

	private PdfPTable makeSpace(int space) {
	
		PdfPTable pptSpace = new PdfPTable(1);
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setFixedHeight(space);
		cell.setPhrase(new Phrase(""));
	
		pptSpace.addCell(cell);
	
		return pptSpace;
	}
	

	private PdfPTable makeEntete() throws DocumentException {
	
		PdfPTable pptEntete = new PdfPTable(2);
		pptEntete.setWidthPercentage(100);

		PdfPCell cellDossier = new PdfPCell(makeInfosDossier());
		cellDossier.setMinimumHeight(100);
		cellDossier.setBorder(PdfPCell.NO_BORDER);
		pptEntete.addCell(cellDossier);		
		PdfPCell cellDossier2 = new PdfPCell(makeInfosDossier2());
		cellDossier2.setBorder(PdfPCell.NO_BORDER);
		pptEntete.addCell(cellDossier2);
		
		return pptEntete;	
	}
	

	private PdfPTable makeInfosDossier() {
	
		PdfPTable pptInfos = new PdfPTable(1);
		
		PdfPCell cell;
			cell = new PdfPCell(pptInfos.getDefaultCell());
			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);

		
			cell.setPhrase(new Phrase("Dossier : " + lignesdata.raisonSociale, fontAdrSocieteGras));		
			pptInfos.addCell(cell);
			
		String dateCourante = formatDate.format(new Date(System.currentTimeMillis()));
		cell.setPhrase(new Phrase("Date : " + dateCourante, fontAdrSociete));
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		pptInfos.addCell(cell);
		
		cell.setPhrase(new Phrase("Page "+currentPage+" / "+totalPages, fontInfosFact));
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		pptInfos.addCell(cell);
		
		return pptInfos;	
	}
	
	private PdfPTable makeInfosDossier2() {
		
		PdfPTable pptInfos = new PdfPTable(1);
		
		PdfPCell cell;
			cell = new PdfPCell(pptInfos.getDefaultCell());
			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);

		
			cell.setPhrase(new Phrase(lignesdata.raisonSociale, fontAdrSocieteGras));		
			pptInfos.addCell(cell);
			cell.setPhrase(new Phrase(lignesdata.adresse, fontAdrSociete));		
			pptInfos.addCell(cell);				
			cell.setPhrase(new Phrase(lignesdata.codePostal+" "+lignesdata.ville, fontAdrSociete));		
			pptInfos.addCell(cell);

			String telfax = "";
			if (!lignesdata.tel.equals("")) {
			if (lignesdata.tel.length()>0) {
				telfax = "T\u00E9l: "+ lignesdata.tel;
				if (lignesdata.fax.length()>0) {
					telfax += " - Fax: "+ lignesdata.fax;
				}
			}
			else if (lignesdata.fax.length()>0) {
				telfax = "Fax: "+ lignesdata.fax;
			}		

			if (telfax.length()>0) {
				cell.setPhrase(new Phrase(telfax, fontAdrSociete));		
				pptInfos.addCell(cell);
			}
			}
			if (!lignesdata.email.equals("")) {
				cell.setPhrase(new Phrase("e-mail: "+ lignesdata.email, fontAdrSociete));		
				pptInfos.addCell(cell);
			}			
		
		cell.setPhrase(new Phrase(""));		
		pptInfos.addCell(cell);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		pptInfos.addCell(cell);
		
		return pptInfos;	
	}

	private PdfPTable makeTitre() {
	
		PdfPTable pptTitre = new PdfPTable(1);
		pptTitre.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptTitre.getDefaultCell());		
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setBackgroundColor(new Color(175,175,175));
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setFixedHeight(30);
		cell.setPaddingBottom(5);
		cell.setPhrase(new Phrase("PLAN COMPTABLE DOSSIER", fontTitre));
		
		pptTitre.addCell(cell);
		
		return pptTitre;	
	}
	

	
	
	private PdfPTable makeEntetesColonnes() throws DocumentException {
	
		PdfPTable pptCols = new PdfPTable(nbCols);
		pptCols.setWidthPercentage(100);
		pptCols.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptCols.getDefaultCell());	
		
		
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setPaddingBottom(4);
		cell.setMinimumHeight(40);
		
		cell.setPhrase(new Phrase("Compte", fontEnteteColonne));
		cell.setBorder(PdfPCell.BOX - PdfPCell.RIGHT );	
		pptCols.addCell(cell);
		
		cell.setPhrase(new Phrase("Intitul\u00E9", fontEnteteColonne));
		cell.setBorder(PdfPCell.BOTTOM +PdfPCell.TOP);	
		pptCols.addCell(cell);
		
		cell.setPhrase(new Phrase("Type", fontEnteteColonne));
		cell.setBorder(PdfPCell.BOTTOM +PdfPCell.TOP);	
		pptCols.addCell(cell);
		
		cell.setPhrase(new Phrase("Contrepartie", fontEnteteColonne));
		cell.setBorder(PdfPCell.BOX - PdfPCell.LEFT);	
		pptCols.addCell(cell);
		
		return pptCols;
	}	
	
	
	private PdfPTable makeLinePlanD(LignePlanDossier lp, boolean pair) throws DocumentException {
	
		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);

		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());
		cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.LEFT+PdfPCell.RIGHT);	
		if (pair){
			cell.setBackgroundColor(new Color(175,175,175));
		}
			
		cell.setPaddingLeft(5);
		cell.setPhrase(new Phrase(lp.numCompte, fontLine));
		cell.setBorder(PdfPCell.NO_BORDER);	
		pptLine.addCell(cell);		
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cell.setPhrase(new Phrase(lp.intitule, fontLine));
		pptLine.addCell(cell);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setPhrase(new Phrase(lp.type, fontLine));
		pptLine.addCell(cell);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		if (lp.contrepartie != null) {
		cell.setPhrase(new Phrase(lp.contrepartie, fontLine));	
		pptLine.addCell(cell);
		}
		else{
			cell.setPhrase(new Phrase("", fontLine));
			cell.setBorder(PdfPCell.NO_BORDER);	
			pptLine.addCell(cell);
		}
		return pptLine;	
	}



	@Override
	public Image imageFond() {
		// TODO Auto-generated method stub
		return null;
	}



	@Override
	public int nbEx() {
		// TODO Auto-generated method stub
		return 1;
	}
		

	
	

} 
