/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *   Import de donnes comptables Agave
 */

package org.opensi.compta.actions.transferts;


import java.io.*;

import org.opensi.util.tools.DateTime;
import org.opensi.util.tools.Arrondi;


public class ImportAgave extends ImportCompta {


	public ImportAgave(String s_, String base, String fichierImport, String ecraser, String journal) {
		
		super(s_, base, fichierImport, ecraser, journal);
	}


	public void importer() throws Exception {
	
		Arrondi rd = new Arrondi(2);

		EcritureOxf currentEcr = null;
		int currentSum = 0;
		int currentLine = 0;

		FileReader fr = new FileReader(fichier);

		BufferedReader br = new BufferedReader(fr);

		// 2 lignes d'entete
		if (br.ready()) {br.readLine();currentLine++;}
		if (br.ready()) {br.readLine();currentLine++;}

		while (br.ready()) {

			String line = br.readLine();
			cc.incLine();
			currentLine++;
			
			line+=";";
			String [] elem = splitmanuel(line,12);
			
			
			String numeroCompte = elem[7].trim();
			String intitule = numeroCompte;
			
			String type = (numeroCompte.startsWith("C")?"C":(numeroCompte.startsWith("F")?"F":"G"));
			
			if (!type.equals("G"))
				numeroCompte = "0"+ numeroCompte;
				
			numeroCompte = completeWith0(numeroCompte);

			cc.checkIntituleCompte(intitule);

			if (cc.checkTypeCompte(type)) {				
				cc.checkNumeroCompte(numeroCompte, type);
			}

			comptes.put(numeroCompte.toLowerCase(), new CompteOxf(currentLine, numeroCompte, intitule, type));

			if (currentSum==0) {

				String codeJournal = elem[0].trim();
				long dateEcriture = (new DateTime(1,Integer.parseInt(elem[4].trim()),Integer.parseInt(elem[3].trim()))).getDateInMillis();
				
				// souci des a nouveaux
				if (dateEcriture<debutExercice) {
					dateEcriture = periodeDebutExercice;
				}		

				cc.checkCodeJournal(codeJournal);
				cc.checkPeriode(dateEcriture);

				currentEcr = new EcritureOxf(codeJournal, dateEcriture);

				ecritures.addEcriture(currentEcr);
			}


			long dateOp = (new DateTime(Integer.parseInt(elem[5].trim()),Integer.parseInt(elem[4].trim()),Integer.parseInt(elem[3].trim()))).getDateInMillis();
			
			String libelle = elem[8].trim();
			String compteAna = elem[9].trim();

			int montantD = 0;
			int montantC = 0;
			String numPiece = elem[6].trim();
			
			long dateEcheance = 0;
					
			String lettre = "";

			
			montantC = (int) (rd.round(Double.parseDouble(elem[11].trim()) * 100));
			if (montantC>0)
				cc.checkMontantOp(montantC);

			
			montantD = (int) (rd.round(Double.parseDouble(elem[10].trim()) * 100));
			if (montantD>0)
				cc.checkMontantOp(montantD);

			currentSum += montantD - montantC;


			cc.checkDateOp(dateOp);
			cc.checkLibelleOp(libelle);
			cc.checkNumeroCompte(numeroCompte);			

			OperationOxf op = new OperationOxf(currentLine, dateOp, libelle, numeroCompte,
																				montantD, montantC,
																				numPiece, dateEcheance, lettre);
			op.setType(compteAna);

			currentEcr.addOp(op);																						

		}

		br.close();
	}
	

} // fin ImportAgave



