/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *   Import de donnes comptables
 */

package org.opensi.compta.actions.transferts;

import java.io.*;
import java.sql.*;

import org.experlog.openeas.api.Session;

import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;


public abstract class ImportCompta {


	protected EcrituresOxf ecritures = new EcrituresOxf();
	protected Hashtable<String, CompteOxf> comptes = new Hashtable<String, CompteOxf>();

	protected CheckCompta cc;

	private String base;
	private String baseDossier;
	protected String fichier;
	protected Session session;
	protected Connection con;

	protected long debutExercice;
	protected long periodeDebutExercice;

	private long finExercice;

	protected String journal;
	protected String dossierId;

	protected ErrorLog elog = new ErrorLog();



	public ImportCompta(String s_, String base, String fichierImport, String ecraser, String journal) {
		try {

			this.session = Session.findClient(s_);
			this.base = base;
			this.baseDossier = session.getCookie().get("BaseDossier");
			this.journal = journal;
			this.con = session.getConnection(null);
			this.fichier = session.getAppConfiguration().get("opensi.absdir") +"/iobuffer/"+ fichierImport;

			this.dossierId = session.getCookie().get("Dossier_Id");

			// dates d'exercice

			Statement stt = con.createStatement();

			ResultSet rset = stt.executeQuery("select Debut_Exercice, Fin_Exercice from EXERCICE where Nom_Base='"+ base +"'");

			rset.next();

			debutExercice = rset.getLong("Debut_Exercice");
			finExercice = rset.getLong("Fin_Exercice");


			Calendar c = Calendar.getInstance();

			c.setTimeInMillis(debutExercice);
			c.set(Calendar.DAY_OF_MONTH, 1);
			periodeDebutExercice = c.getTimeInMillis();

			rset.close();
			stt.close();

			cc = new CheckCompta(debutExercice, finExercice, periodeDebutExercice);
			cc.setErrorLog(elog);

			importer();

			checkComptesDefinis();
			checkNoMvtCF();

			session.trace("Erreurs : "+ elog.size());
			session.trace("Nb Ecritures : "+ ecritures.getSize());
			session.trace("Stabilit des critures : "+ ecritures.isStable());

			if (!ecritures.isStable()) {

				elog.addError("Ecritures non quilibres");
			}
			else if (elog.getNbError()==0) {

       	elog.addInfo(ecritures.getSize() +" critures importes");
        elog.addInfo(ecritures.getNbOp() +" lignes d'critures importes");
				session.trace("INSERTION");

				if (ecraser.equals("1")) {
					ecraserExercice();
				}

				checkLettrage();

				creerComptesInexistants();
				propagerComptes();

				creerJournauxInexistants();

				insertEcritures();

				refreshSoldes();

				refreshNbOp();
			}

		}
		catch(Exception e) {
			e.printStackTrace();
		}
		finally {

			//fermeture de la connexion SQL
			try {
				session.closeConnection(con, null);
			}
			catch (SQLException se) {
				session.trace("Impossible de fermer la connexion");
				se.printStackTrace();
			}

			// suppression du fichier d'import
			try {
				File file = new File(fichier);
				file.deleteOnExit();
			}
			catch(SecurityException se) {
				session.trace("Impossible d'enregistrer le fichier  supprimer");
				se.printStackTrace();
			}
		}

	} // fin ImportCompta (constructeur)


	public abstract void importer() throws Exception;


	public ErrorLog getErrorLog() {

		return elog;
	}


	private void ecraserExercice() throws SQLException {

		Statement stt = con.createStatement();

		stt.executeUpdate("delete from "+ base +".OPERATION");
		stt.executeUpdate("delete from "+ base +".ECRITURE");
		stt.executeUpdate("update "+ base +".SOLDE_COMPTE set Total_Credit=0, Total_Debit=0, Solde=0");

		stt.close();
	}



	protected String remplace(String elem)  {
		if (elem.startsWith("\"") && (elem.endsWith("\"")) && elem.length()>1) {
			elem=elem.substring(1,elem.length());
			elem=elem.substring(0,elem.length()-1);
		}
		return elem;
	}

	protected String nombre(String elem)  {
		return elem.replace(',','.');
	}

	protected String [] splitmanuel(String elem, int nb)
	{
		int etat=1 ;
		char EC=' ' ;
		int curseur=0;
 		int marqueur=0;
		char separateur=';';
		char delimiteur='"';
		String [] spl=new String[nb];
		int k=0;

		String messageErreur="";

		try {
			while (curseur< elem.length())
			{
	  		EC = elem.charAt(curseur);

				switch(etat)
				{
					case 1: {

						if (EC==delimiteur){
							etat=2;
							marqueur= curseur+1;
						}
						else{
							if(EC==separateur||EC=='\n') {
								etat=5;
							}
							else
							{
								etat=4;
								marqueur= curseur;
							}

						}
					} break ;

					case 2:	{

						if (EC==delimiteur){
							etat=3;
						}
						else{
							if(EC=='\n') {
								etat=6 ;
								messageErreur=("Vous n'avez pas bien dlimit votre texte");
							}
							else{
								etat=2;
							}
						}
					} break ;

					case 3:{
						if (EC==delimiteur){
							etat=2;
						}
						else{
							if(EC==separateur||EC=='\n') {
								etat=1;
								spl[k]=elem.substring(marqueur,curseur-1);
								k++;
							}
							else
							{
								etat=6;
								messageErreur="Vous n'avez peut tre pas bien dlimit votre texte\n" ;
							}
						}
					} break ;
					case 4:	{
						if(EC==separateur||EC=='\n') {
							etat=1;
							spl[k]=elem.substring(marqueur,curseur);
							k++;
						}
						else {
							etat=4;
						}
					} break ;
					case 5:{
						spl[k]="";
						k++;
						etat=1;
						curseur--;
					} break;
					default: curseur=elem.length()-1 ; break ;
				}
					curseur ++ ;
			}
		}
		catch (ArrayIndexOutOfBoundsException e) {
			return new String[1];
		}
		return ( k == nb ) ? spl : new String[1];
	}


	private Hashtable<String, Hashtable<String, ArrayList<OperationOxf>>> getLetSet() {

		Hashtable<String, Hashtable<String, ArrayList<OperationOxf>>> compteSet = new Hashtable<String, Hashtable<String, ArrayList<OperationOxf>>>();

		for (int i=0; i<ecritures.getSize(); i++) {

			EcritureOxf ecr = ecritures.getEcriture(i);

			for (int j=0; j<ecr.getSize(); j++) {

				OperationOxf op = ecr.getOp(j);

				String compte = op.getNumeroCompte().toLowerCase();
				String lettre = op.getLettre().toUpperCase();

				if (lettre.length() > 0) {

					if (!compteSet.containsKey(compte)) {
						compteSet.put(compte, new Hashtable<String, ArrayList<OperationOxf>>());
					}

					Hashtable<String, ArrayList<OperationOxf>> lettreSet = compteSet.get(compte);

					if (!lettreSet.containsKey(lettre)) {
						lettreSet.put(lettre, new ArrayList<OperationOxf>());
					}

					ArrayList<OperationOxf> opList = lettreSet.get(lettre);

					opList.add(op);
				}
			}
		}

		return compteSet;
	}


	private void checkLettrage() {

		Hashtable<String, Hashtable<String, ArrayList<OperationOxf>>> compteSet = getLetSet();

		for (Enumeration<Hashtable<String, ArrayList<OperationOxf>>> ec = compteSet.elements(); ec.hasMoreElements();) {

			Hashtable<String, ArrayList<OperationOxf>> lettreSet = ec.nextElement();

			for (Enumeration<ArrayList<OperationOxf>> el = lettreSet.elements() ; el.hasMoreElements();) {

				ArrayList<OperationOxf> opList = el.nextElement();

				int credit = 0;
				int debit = 0;

				for (int i=0; i<opList.size(); i++) {
					debit += opList.get(i).getMontantD();
					credit += opList.get(i).getMontantC();
				}

				if (debit!=credit) {
					for (int i=0; i<opList.size(); i++) {
						opList.get(i).setLettre("");
					}
				}
			}
    }
	}


	private void checkComptesDefinis() {

		if (comptes.isEmpty()) {
			elog.addError("Dfinitions des comptes manquantes !");
		}
		else {

			for (int i=0; i<ecritures.getSize(); i++) {

				EcritureOxf ecr = ecritures.getEcriture(i);

				for (int j=0; j<ecr.getSize(); j++) {

					String compte = ecr.getOp(j).getNumeroCompte();

					if (!compteDefini(compte)) {
						elog.addError(ecr.getOp(j).getLineFrom(), "Compte '"+ compte +"' non dfini");
					}
				}
			}
		}
	}


	private boolean compteDefini(String compte) {

		return comptes.containsKey(compte.toLowerCase());
	}


	private void checkNoMvtCF() {

		for (int i=0; i<ecritures.getSize(); i++) {

			EcritureOxf ecr = ecritures.getEcriture(i);

			for (int j=0; j<ecr.getSize(); j++) {

				String compte = ecr.getOp(j).getNumeroCompte();

				if (compte.equals("40100000")) { elog.addError(ecr.getOp(j).getLineFrom(), "Opration sur compte collectif fournisseurs (40100000)");	}
				if (compte.equals("41100000")) { elog.addError(ecr.getOp(j).getLineFrom(), "Opration sur compte collectif clients (41100000)");	}
			}
		}
	}


	private void insertEcritures() throws Exception {

		PreparedStatement psEcr = con.prepareStatement("insert into "+ base +".ECRITURE (Code_Journal, Date_Ecriture, Vide, Numero) values (?,?,?,?)", Statement.RETURN_GENERATED_KEYS);
		PreparedStatement psOp = con.prepareStatement("insert into "+ base +".OPERATION"
														+"(Num_Ecriture, Numero_Compte, Montant_D, Montant_C, Libelle, Num_Piece, Lettre, Date_Op, Date_Echeance, Num_Ligne, Type)"
														+"values (?,?,?,?,?,?,?,?,?,?,?)");

		PreparedStatement psNumero = con.prepareStatement("select coalesce(max(Numero),0)+1 from "+ base +".ECRITURE where Code_Journal=? and Date_Ecriture=?");


		for (int i=0; i<ecritures.getSize(); i++) {

			EcritureOxf ecriture = ecritures.getEcriture(i);

			psNumero.setString(1, ecriture.getCodeJournal());
			psNumero.setLong(2, ecriture.getDateEcriture());
			ResultSet rsN = psNumero.executeQuery();
			rsN.next();

			psEcr.setString(1, ecriture.getCodeJournal());
			psEcr.setLong(2, ecriture.getDateEcriture());
			psEcr.setInt(3,0);
			psEcr.setInt(4, rsN.getInt(1));

			psEcr.executeUpdate();

			ResultSet rset = psEcr.getGeneratedKeys();
			rset.next();
			int numEcriture = rset.getInt(1);


			for (int j=0; j<ecriture.getSize(); j++) {

				OperationOxf op = ecriture.getOp(j);

				psOp.setInt(1, numEcriture);
				psOp.setString(2, op.getNumeroCompte());
				psOp.setDouble(3, (double)op.getMontantD()/100);
				psOp.setDouble(4, (double)op.getMontantC()/100);
				psOp.setString(5, op.getLibelle());
				psOp.setString(6, op.getNumPiece());
				psOp.setString(7, op.getLettre());
				psOp.setLong(8, op.getDateOp());
				psOp.setLong(9, op.getDateEcheance());
				psOp.setInt(10, j);
				psOp.setString(11, op.getType());

				psOp.executeUpdate();
			}

		}

		psEcr.close();
		psOp.close();
		psNumero.close();
	}


	private void creerComptesInexistants() throws Exception {

		// Cration des comptes inexistants (mme ceux qui ne sont pas mouvements)

		PreparedStatement psCpte = con.prepareStatement("insert into "+ baseDossier +".COMPTE (Numero_Compte, Intitule, Type_Compte) values (?,?,?)");

		PreparedStatement psExCompte = con.prepareStatement("select Numero_Compte from "+ baseDossier +".COMPTE where Numero_Compte=?");

		int nbNewComptes = 0;

		Enumeration<CompteOxf> enumComptes = comptes.elements();

		while (enumComptes.hasMoreElements()) {

			CompteOxf compte = enumComptes.nextElement();

			psExCompte.setString(1, compte.getNumero());
			ResultSet rset = psExCompte.executeQuery();

			if (!rset.next()) {

				nbNewComptes++;

				psCpte.setString(1, compte.getNumero());
				psCpte.setString(2, compte.getIntitule());
				psCpte.setString(3, compte.getType());

				psCpte.executeUpdate();
			}

			rset.close();
		}

		psExCompte.close();
		psCpte.close();

		elog.addInfo(nbNewComptes +" nouveaux comptes crs");
	}


	private void propagerComptes() throws Exception {

		// propagation des comptes de l'exercice d'import dans les exercices o ils sont inexistants

		Statement stt = con.createStatement();
		Statement stt2 = con.createStatement();

		ResultSet rsNB = stt.executeQuery("select Nom_Base from EXERCICE where Dossier_Id='"+ dossierId +"'");

		while (rsNB.next()) {

			PreparedStatement psCpte = con.prepareStatement("insert into "+ rsNB.getString("Nom_Base") +".SOLDE_COMPTE (Numero_Compte) values (?)");

			String reqNewComptes = "select distinct cr.Numero_Compte from "+ baseDossier +".COMPTE cr left join "+ rsNB.getString("Nom_Base") +".SOLDE_COMPTE c on cr.Numero_Compte=c.Numero_Compte where c.Numero_Compte is null";

			ResultSet rsNC = stt2.executeQuery(reqNewComptes);

			while (rsNC.next()) {

				psCpte.setString(1, rsNC.getString("Numero_Compte"));
				psCpte.executeUpdate();
			}

			psCpte.close();
		}

		stt.close();
		stt2.close();
	}


	private void creerJournauxInexistants() throws Exception {

		// Cration des journaux prsents dans les critures importes et non prsents dans les journaux

		PreparedStatement psJournal = con.prepareStatement("insert into "+ baseDossier +".JOURNAL (Code_Journal, Intitule, Type_Journal) values (?,?, 'OD')");

		PreparedStatement psExCJ = con.prepareStatement("select Code_Journal from "+ baseDossier +".JOURNAL where Code_Journal=?");

		PreparedStatement psNbOp = con.prepareStatement("insert into "+ base +".NB_OP_JOURNAUX (Code_Journal, Periode) values (?,?)");

		ArrayList<Long> periodes = getPeriodes();

		for (int j=0; j<ecritures.getSize(); j++) {

			String codeJournal = ecritures.getEcriture(j).getCodeJournal();

			psExCJ.setString(1, codeJournal);
			ResultSet rsECJ = psExCJ.executeQuery();

			if (!rsECJ.next()) {
				psJournal.setString(1, codeJournal);
				psJournal.setString(2, codeJournal);
				psJournal.executeUpdate();

				// insertion des periodes dans NB_OP_JOURNAUX

				psNbOp.setString(1, codeJournal);

				for (int i=0; i<periodes.size(); i++) {

					psNbOp.setLong(2, periodes.get(i).longValue());

					psNbOp.executeUpdate();
				}
			}

			rsECJ.close();
		}

		psJournal.close();
		psNbOp.close();
		psExCJ.close();
	}



	private void refreshSoldes() throws Exception {

		PreparedStatement ps = con.prepareStatement("update "+ base +".SOLDE_COMPTE set Total_Credit=?, Total_Debit=?, Solde=? where Numero_Compte=?");

		String reqSoldes = "select Numero_Compte, sum(Montant_D) as Total_Debit, sum(Montant_C) as Total_Credit from "+ base +".OPERATION group by Numero_Compte";

		Statement stt = con.createStatement();

		ResultSet rset = stt.executeQuery(reqSoldes);

		while (rset.next()) {

			double credit = rset.getDouble("Total_Credit");
			double debit = rset.getDouble("Total_Debit");

			ps.setDouble(1, credit);
			ps.setDouble(2, debit);
			ps.setDouble(3, credit-debit);
			ps.setString(4, rset.getString("Numero_Compte"));

			ps.executeUpdate();
		}

		String reqClients = "select sum(Montant_D) as Total_Debit, sum(Montant_C) as Total_Credit"
													+" from "+ base +".OPERATION o, "+ baseDossier +".COMPTE c where c.Numero_Compte=o.Numero_Compte and c.Type_Compte='C'";
		rset = stt.executeQuery(reqClients);

		if (rset.next()) {

			double credit = rset.getDouble("Total_Credit");
			double debit = rset.getDouble("Total_Debit");

			ps.setDouble(1, credit);
			ps.setDouble(2, debit);
			ps.setDouble(3, credit-debit);
			ps.setString(4, "41100000");

			ps.executeUpdate();
		}


		String reqFournisseurs = "select sum(Montant_D) as Total_Debit, sum(Montant_C) as Total_Credit"
															+" from "+ base +".OPERATION o, "+ baseDossier +".COMPTE c where c.Numero_Compte=o.Numero_Compte and c.Type_Compte='F'";
		rset = stt.executeQuery(reqFournisseurs);

		if (rset.next()) {

			double credit = rset.getDouble("Total_Credit");
			double debit = rset.getDouble("Total_Debit");

			ps.setDouble(1, credit);
			ps.setDouble(2, debit);
			ps.setDouble(3, credit-debit);
			ps.setString(4, "40100000");

			ps.executeUpdate();
		}

	}



	private void refreshNbOp() throws Exception {

		Statement stt = con.createStatement();
    stt.executeUpdate("delete from "+ base +".NB_OP_JOURNAUX");

		PreparedStatement ps = con.prepareStatement("insert into "+ base +".NB_OP_JOURNAUX(Nb_Op,Code_Journal,Periode) values (?,?,?)");


		ResultSet rset = stt.executeQuery("select Code_Journal, Date_Ecriture, count(Op_Id) as NbOp from "+ base +".OPERATION o, "+ base +".ECRITURE e where o.Num_Ecriture=e.Num_Ecriture group by Date_Ecriture, Code_Journal");

		while (rset.next()) {

			ps.setInt(1, rset.getInt("NbOp"));
			ps.setString(2, rset.getString("Code_Journal"));
			ps.setLong(3, rset.getLong("Date_Ecriture"));

			ps.executeUpdate();
		}

		stt.close();
	}




	private ArrayList<Long> getPeriodes() {

		Calendar calendar = Calendar.getInstance();

		calendar.clear();
		calendar.setTimeInMillis(debutExercice);
		int moisCourant = calendar.get(Calendar.MONTH);
		int anneeCourante = calendar.get(Calendar.YEAR);

		calendar.clear();
		calendar.setTimeInMillis(finExercice);
		int moisFin = calendar.get(Calendar.MONTH);
		int anneeFin = calendar.get(Calendar.YEAR);

		ArrayList<Long> periodes = new ArrayList<Long>();

		while (anneeCourante<anneeFin || (anneeCourante==anneeFin && moisCourant<=moisFin)) {

			calendar.clear();
			calendar.set(anneeCourante,moisCourant,1,0,0,0);

			if (moisCourant==11) {
				moisCourant = 0;
				anneeCourante++;
			}
			else {
				moisCourant++;
			}

			periodes.add(new Long(calendar.getTimeInMillis()));
		}

		return periodes;
  }


	protected String completeWith0(String str) {

		String str2 = new String(str);

		int nb = 8 - str2.length();

		for (int i=0; i<nb; i++) {

			str2 += "0";
		}

		return str2;
	}


} // fin ImportCompta



