/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *   Import de donnes comptables CSV
 */

package org.opensi.compta.actions.transferts;


import java.io.*;
import java.sql.*;

import org.opensi.util.tools.DateTime;


public class ImportCsv extends ImportCompta {


	public ImportCsv(String s_, String base, String fichierImport, String ecraser, String journal) {
		
		super(s_, base, fichierImport, ecraser, journal);
	}


	public void importer() throws Exception {
	
	
		EcritureOxf currentEcr = null;
		int currentSum = 0;
		int currentLine = 0;
		int ht;
		int tva;
		int ttc;
		FileReader fr = new FileReader(fichier);

		BufferedReader br = new BufferedReader(fr);
		if (br.ready()) {
			String line = br.readLine();
			currentLine++;

			while (br.ready()) {

				line = br.readLine();
				cc.incLine();
				currentLine++;
				line+=";;";
				String [] elem = splitmanuel(line, 11);
				if (elem.length==11) {
					String nom=remplace(elem[0]);
					String num_client=remplace(elem[1]);
					String num_vente=remplace(elem[2]);
					String num_tva=remplace(elem[3]);
					String designation=remplace(elem[4]);
					String num_fact=remplace(elem[5]);
					String date_fact=remplace(elem[6]);
					String date_echeance=remplace(elem[7]);
					String montant_ht=nombre(remplace(elem[8]));
					String montant_tva=nombre(remplace(elem[9]));
					String montant_ttc=nombre(remplace(elem[10]));

					if (num_fact.length()>20) {elog.addError("Le numro de la facture est incorrect ligne : "+currentLine);}
					if (date_fact.length()!=10 && date_fact.length()!=8) {elog.addError("La date de facturation est incorrecte ligne : "+currentLine);}
					if (date_echeance.length()!=10 && date_echeance.length()!=8 && date_echeance.length()!=0) {elog.addError("La date d'chance est incorrecte ligne : "+currentLine);}
					
					cc.checkEmpty(num_client,"Le numro de client est vide");
					cc.checkEmpty(num_tva,"Le numro de TVA est vide");
					cc.checkEmpty(num_fact,"Le numro de facture est vide");
					cc.checkEmpty(date_fact,"La date de facturation est vide");
					
					cc.checkNumeroCompte(num_client,"C");
					cc.checkNumeroCompte(num_tva,"G");

					if (num_vente.length()!=0) {
						cc.checkNumeroCompte(num_vente,"G");
						comptes.put(num_vente.toLowerCase(), new CompteOxf(currentLine, num_vente, "VENTES", "G"));
					}
					else {
						//compte de vente par dfaut
						num_vente=getDefautCompteVente();
						if (!num_vente.equals("")) {
							cc.checkNumeroCompte(num_vente,"G");
							comptes.put(num_vente.toLowerCase(), new CompteOxf(currentLine, num_vente, "VENTES", "G"));
						}
					}
				
					comptes.put(num_tva.toLowerCase(), new CompteOxf(currentLine, num_tva, "TVA", "G"));
					comptes.put(num_client.toLowerCase(), new CompteOxf(currentLine, num_client, nom, "C"));
					
					DateTime dt;
					DateTime dtop;
					if (date_fact.length()==8) {
						dt = new DateTime(DateTime.parseDate(date_fact,"dd/MM/yy"));	
					}
					else {
						dt = new DateTime(DateTime.parseDate(date_fact,"dd/MM/yyyy"));						
					}
					long dateOp = dt.getDateInMillis();
					dt.setDay(1);
					
					long dateEcriture = dt.getDateInMillis();
						
					cc.checkCodeJournal(journal);
					cc.checkPeriode(dateEcriture);
					cc.checkDate(dateOp);

					long dateEcheance =0;
					if (date_echeance.length()==8) {
						dateEcheance = DateTime.parseDate(date_echeance,"dd/MM/yy");
					}
					else if (date_echeance.length()==10) {
						dateEcheance = DateTime.parseDate(date_echeance,"dd/MM/yyyy");
					}

					currentEcr = new EcritureOxf(journal, dateEcriture);
					
					int montant_vente=0;
					int montant_taxe=0;
					
					try {
						if (!montant_ht.equals("")) {
							double htdb = (new Double(montant_ht)).doubleValue();
							montant_vente=(new Double(htdb*100)).intValue();
							if (!montant_tva.equals("")) {
								double tvadb = (new Double(montant_tva)).doubleValue();
								montant_taxe = (new Double(tvadb*100)).intValue();
							}
						}
						else if (!montant_tva.equals("")) {
							double tvadb = (new Double(montant_tva)).doubleValue();
							montant_taxe= (new Double(tvadb*100)).intValue();
							if (!montant_ttc.equals("")) {
								double ttcdb = (new Double(montant_ttc)).doubleValue();
								ttc=(new Double(ttcdb*100)).intValue();
								montant_vente=ttc-montant_taxe;
							}
						}
						else if (!montant_ttc.equals("")) {
						
							double ttcdb = (new Double(montant_ttc)).doubleValue();
							ttc=(new Double(ttcdb*100)).intValue();
							montant_vente=ttc;
						}
					}
					catch (NumberFormatException e) {
						elog.addError("Les montants sont incorrects ligne : "+currentLine);
					}
					if (montant_vente>0) {
				
						ecritures.addEcriture(currentEcr);

						cc.checkLibelleOp(designation);

						currentEcr.addOp(new OperationOxf(currentLine, dateOp, designation, num_vente,
																							0, montant_vente,
																							num_fact, dateEcheance, ""));																						
						if (montant_taxe>0) {
							currentEcr.addOp(new OperationOxf(currentLine, dateOp, designation, num_tva,
																								0, montant_taxe,
																								num_fact, dateEcheance, ""));	
						}																					
						currentEcr.addOp(new OperationOxf(currentLine, dateOp, designation, num_client,
																							(montant_vente+montant_taxe), 0,
																							num_fact, dateEcheance, ""));
																																													
					}

				}
				else {
					elog.addError("Le nombre de colonnes est incorrect ligne : "+currentLine);
				}
			}
		}
		else {
			elog.addError("Fichier vide");
		}

		br.close();
	}
	
	
	
	private String getDefautCompteVente() throws Exception {
	
		String reqVente = "select Numero_Compte_Vente from PARAM_DOSSIER where Dossier_Id='"+dossierId+"'";
		
		PreparedStatement psCpte = con.prepareStatement(reqVente);
		ResultSet rset = psCpte.executeQuery();
		if (rset.next()) {		
			return rset.getString("Numero_Compte_Vente");
		}
		return "";
	}
	

} // fin ImportCsv



