/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *   Import de donnes comptables Quadra
 */

package org.opensi.compta.actions.transferts;


import java.io.*;

import org.opensi.util.tools.DateTime;


public class ImportQuadra extends ImportCompta {


	public ImportQuadra(String s_, String base, String fichierImport, String ecraser, String journal) {

		super(s_, base, fichierImport, ecraser, journal);
	}


	public void importer() throws Exception {


		EcritureOxf currentEcr = null;
		int currentSum = 0;
		int currentLine = 0;

		FileReader fr = new FileReader(fichier);

		BufferedReader br = new BufferedReader(fr);


		while (br.ready()) {

			String line = br.readLine();
			cc.incLine();
			currentLine++;

			if (line.startsWith("C")) {
				String type = line.substring(217, 218);
				String intitule = line.substring(9, 39);
				String numero = line.substring(1, 9);

				cc.checkIntituleCompte(intitule);

				if (cc.checkTypeCompte(type)) {
					cc.checkNumeroCompte(numero, type);
				}

				comptes.put(numero.toLowerCase(), new CompteOxf(currentLine, numero, intitule, type));
			}

			else if (line.startsWith("M")) {

				if (currentSum==0) {

					// String codeJournal = line.substring(9, 12).trim();
					String codeJournal = line.substring(110, 113).trim();
					long dateEcriture = DateTime.parseDate(line.substring(16, 20), "MMyy");
					// souci des a nouveaux
					if (dateEcriture<debutExercice) {
						dateEcriture = periodeDebutExercice;
					}

					cc.checkCodeJournal(codeJournal);
					cc.checkPeriode(dateEcriture);


					currentEcr = new EcritureOxf(codeJournal, dateEcriture);


					ecritures.addEcriture(currentEcr);
				}


				long dateOp = DateTime.parseDate(line.substring(14, 20),"ddMMyy");
				String libelle = line.substring(21, 41).trim();
				String numeroCompte = line.substring(1, 9);
				int montantD = 0;
				int montantC = 0;
				String numPiece = line.substring(148, 158).trim();
				long dateEcheance = 0;

				if (!line.substring(63, 69).equals("000000")) {
					dateEcheance = DateTime.parseDate(line.substring(63, 69),"ddMMyy");
				}
				String lettre = line.substring(69, 74).trim();

				if (line.substring(41, 43).equalsIgnoreCase("C+") || line.substring(41, 43).equalsIgnoreCase("D-")) {
					montantC = Integer.parseInt(line.substring(43, 55).trim());
					cc.checkMontantOp(montantC);
				}
				else {
					montantD = Integer.parseInt(line.substring(43, 55).trim());
					cc.checkMontantOp(montantD);
				}

				currentSum += montantD - montantC;


				cc.checkDateOp(dateOp);
				cc.checkLibelleOp(libelle);
				cc.checkNumeroCompte(numeroCompte);

				currentEcr.addOp(new OperationOxf(currentLine, dateOp, libelle, numeroCompte,
																					montantD, montantC,
																					numPiece, dateEcheance, lettre));

			}
		}

		br.close();
	}


} // fin ImportQuadra



