package org.opensi.compta.actions.transferts;


import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.DefaultHandler;

import java.util.Hashtable;


import org.opensi.util.tools.DateTime;



public class OxfHandler extends DefaultHandler {

	
	private Locator locator;
	private EcrituresOxf ecrituresOxf;
	private EcritureOxf currentEcr;
	private Hashtable<String, CompteOxf> comptes;
	private CheckCompta cc;


	public OxfHandler(CheckCompta cc, EcrituresOxf ecrituresOxf, Hashtable<String, CompteOxf> comptes) {
	
		super();
		this.cc = cc;
		this.ecrituresOxf = ecrituresOxf;
		this.comptes = comptes;
	}


	public void setDocumentLocator(Locator locator) {

		this.locator = locator;
	}
	

	public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws	SAXException {
		
		//System.out.println(localName +"/"+namespaceURI+"/"+ qName);
		
		int currentLine = locator.getLineNumber();
		
		cc.setLine(currentLine);
		
		
		if (localName.equals("compte")) {
			
			String type = atts.getValue("type");
			String intitule = atts.getValue("intitule");
			String numero = atts.getValue("numero");
			
			cc.checkIntituleCompte(intitule);
			
			if (cc.checkTypeCompte(type)) {
				cc.checkNumeroCompte(numero, type);
			}	
			
			comptes.put(numero.toLowerCase(), new CompteOxf(currentLine, numero, intitule, type));
					
		}
		else if (localName.equals("ecriture")) {
		
			String codeJournal = atts.getValue("journal");
			long dateEcriture = DateTime.parseDate(atts.getValue("periode"),"MMyyyy");			
			
			cc.checkCodeJournal(codeJournal);
			cc.checkPeriode(dateEcriture);			
			
			currentEcr = new EcritureOxf(codeJournal, dateEcriture);
			
			ecrituresOxf.addEcriture(currentEcr);			
						
		}
		else if (localName.equals("credit") || localName.equals("debit")) {
			
			long dateOp = DateTime.parseDate(atts.getValue("date"),"ddMMyyyy");			
			String libelle = atts.getValue("libelle");
			String numeroCompte = atts.getValue("compte");
			int montantD = 0;
			int montantC = 0;
			String numPiece = atts.getValue("numpiece");	
			long dateEcheance = atts.getValue("echeance").equals("")?0:DateTime.parseDate(atts.getValue("echeance"),"ddMMyyyy");
			String lettre = atts.getValue("lettre");
			
			if (localName.equals("credit")) {
				montantC = Integer.parseInt(atts.getValue("montant"));
				cc.checkMontantOp(montantC);
			}
			else {
				montantD = Integer.parseInt(atts.getValue("montant"));
				cc.checkMontantOp(montantD);
			}
			
			cc.checkDateOp(dateOp);
			cc.checkLibelleOp(libelle);
			cc.checkNumeroCompte(numeroCompte);
			
			currentEcr.addOp(new OperationOxf(currentLine, dateOp, libelle, numeroCompte,
																				montantD, montantC,
																				numPiece, dateEcheance, lettre));			
			
		}
		
		
	} // fin startElement
	
	
	public void endElement(String uri, String localName, String qname) throws SAXException {}
	
	
	/*public EcrituresOxf getEcritures() { return ecrituresOxf;	}
	
	public Hashtable getComptes() { return comptes; }*/
	
	
				
} // fin OxfHandler
