/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/
  
/**
 *  Gnration d'une criture d' nouveaux
 */

package org.opensi.compta.exercice;


import org.opensi.compta.Ecriture;
import org.opensi.compta.Operation;

import java.sql.*;


public class ANouveauxFactory {

	private Connection con;
	private String baseN1;
	private String baseN2;
	private String baseDossier;
	private long dateANouveau;
	private boolean temp;
	private Ecriture ean;


	public ANouveauxFactory(Connection con, String baseN1, String baseN2, String baseDossier, long dateANouveau, boolean temp) {
	
		this.con = con;
		this.baseN1 = baseN1;
		this.baseN2 = baseN2;
		this.baseDossier = baseDossier;
		this.dateANouveau = dateANouveau;
		this.temp = temp;
		
		makeEcriture();	
	}
	
	
	public Ecriture getEcritureAN() {
	
		return ean;
	}
	
	
	public void insertEcriture() throws SQLException {
	
		if (ean.isStable()) {
	
			deleteOldEcrAN();
	
			ean.insertDB(con, baseN2, baseDossier);
		}
		else {
			System.out.println("Ecriture A Nouveau dsquilibre !\n");
			System.out.println("Total Debit :"+ ean.totalDebit());
			System.out.println("Total Credit :"+ ean.totalCredit());
			System.out.println("Nb Op :"+ ean.getSize() +"\n");
		}
	}
	
	
	private void deleteOldEcrAN() throws SQLException {
	
		Statement stt = con.createStatement();
		
		String reqOldEcrAN = "select e.Num_Ecriture from "+ baseN2 +".ECRITURE e,"+ baseDossier +".JOURNAL j where Vide=0 and j.Type_Journal='AN' and e.Code_Journal=j.Code_Journal";
		
		ResultSet rset = stt.executeQuery(reqOldEcrAN);
	
		while (rset.next()) {
	
			Ecriture oldean = Ecriture.loadEcriture(rset.getInt("Num_Ecriture"), con, baseN2);
			oldean.supprimer(con, baseN2, baseDossier);
		}
	}
	
	
	private void makeEcriture() {
		try {
	
			String codeJournal = "AN";

			String libelleSAN = (temp?"S.A.N. Temporaire":"S.A.N.");


			this.ean = new Ecriture(codeJournal, dateANouveau);


			// oprations d' nouveaux pour les comptes de classes 1  5

			Statement stt = con.createStatement();

			String reqComptes15 = "select s.Numero_Compte, s.Total_Credit, s.Total_Debit from "+ baseN1 +".SOLDE_COMPTE s, "+ baseDossier +".COMPTE c "
													+ "where c.Numero_Compte=s.Numero_Compte and s.Solde<>0 and c.Numero_Compte<>'41100000' and c.Numero_Compte<>'40100000' and c.Type_Compte='G' "
													+ "and (c.Numero_Compte like '1%' or c.Numero_Compte like '2%' or c.Numero_Compte like '3%' or c.Numero_Compte like '4%' or c.Numero_Compte like '5%')";

			ResultSet rset = stt.executeQuery(reqComptes15);

			while (rset.next()) {

				double totalCredit = rset.getDouble("Total_Credit");
				double totalDebit = rset.getDouble("Total_Debit");

				Operation op = new Operation();
				op.setNumeroCompte(rset.getString("Numero_Compte"));
				op.setDateOp(dateANouveau);
				op.setLibelle(libelleSAN);

				if (totalCredit>totalDebit) {
					op.setMontantC(totalCredit-totalDebit);
					op.setMontantD(0);
				}
				else {
					op.setMontantC(0);
					op.setMontantD(totalDebit-totalCredit);
				}

				ean.addOp(op);
			}


			// oprations d' nouveaux pour les oprations non lettres de l'exerciee prcdent pour les comptes de tiers

			String reqOpTiers = "select o.* from "+ baseDossier +".COMPTE c,"+ baseN1 +".OPERATION o "
												+	"where o.Lettre='' and o.Numero_Compte=c.Numero_Compte and (c.Type_Compte='C' or c.Type_Compte='F')";

			rset = stt.executeQuery(reqOpTiers);

			while (rset.next()) {

				Operation op = new Operation();
				op.setNumeroCompte(rset.getString("Numero_Compte"));
				op.setDateOp(rset.getLong("Date_Op"));
				op.setLibelle(rset.getString("Libelle"));
				op.setMontantC(rset.getDouble("Montant_C"));
				op.setMontantD(rset.getDouble("Montant_D"));
				op.setDateEcheance(rset.getLong("Date_Echeance"));
				op.setRaccourci(rset.getString("Raccourci"));
				op.setType(rset.getString("Type"));	
				op.setNumPiece(rset.getString("Num_Piece"));
				op.setContrepartie(rset.getString("Contrepartie"));

				ean.addOp(op);
			}


			// actualisation du compte de rsultat (12000000)

			double solde6 = 0;
			double solde7 = 0;

			String reqSolde6 = "select (sum(Total_Debit)-sum(Total_Credit)) from "+ baseN1 +".SOLDE_COMPTE where Numero_Compte like '6%'";

			rset = stt.executeQuery(reqSolde6);

			if (rset.next()) {		
				solde6 = rset.getDouble(1);
			}

			String reqSolde7 = "select (sum(Total_Credit)-sum(Total_Debit)) from "+ baseN1 +".SOLDE_COMPTE where Numero_Compte like '7%'";

			rset = stt.executeQuery(reqSolde7);

			if (rset.next()) {
				solde7 = rset.getDouble(1);
			}

			double resultat = solde7 - solde6;

			Operation op = new Operation();
			op.setNumeroCompte("12000000");
			op.setDateOp(dateANouveau);
			op.setLibelle(libelleSAN);		

			if (resultat>0) {
				op.setMontantC(resultat);
			}
			else if (resultat<0) {
				op.setMontantD(-resultat);
			}	

			if (resultat!=0) {
				ean.addOp(op);
			}
		
		}
		catch(SQLException sqle) {
			sqle.printStackTrace();
		}		
	}
  
	
} // fin EcritureANouveaux











