/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/


package org.opensi.compta.persist;

import org.experlog.openeas.api.Session;

import org.opensi.compta.Ecriture;
import org.opensi.compta.Operation;

import java.util.*;
import java.sql.*;


public class EcrituresSet {


 	private Hashtable<Integer, Ecriture> ecritures = new Hashtable<Integer, Ecriture>();
	private Hashtable<String, Ecriture> ecrencours = new Hashtable<String, Ecriture>();


	private String base;
	private String baseDossier;

	private Session s;


	protected EcrituresSet(String session) throws SQLException {

		this.s = Session.findClient(session);
		this.base = s.getCookie().get("BaseExo");
		this.baseDossier = s.getCookie().get("BaseDossier");
	}


	public void valider(int index) {

		Ecriture ecr = getEcritureTemp(index);
    insertDB(ecr);
  }


  public void insertDB(Ecriture ecr) {
    try {

      if (ecr.isStable()) {

        Connection con = s.getConnection(null);

				ecr.insertDB(con, base, baseDossier);

				s.closeConnection(con, null);
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
  }


	public Ecriture getEcritureTemp(int index) {

		Ecriture retour = null;

		Integer id = new Integer(index);

    if (ecritures.containsKey(id)) {
      retour = ecritures.get(id);
    }

    return retour;
  }



	public List getEcritures(int page, String codeJournal, long dateEcriture, int writeable) {

		List<Integer> resultat = new ArrayList<Integer>();

		try {

      String requeteSql = "select Num_Ecriture, Type_Journal from "+ base +".ECRITURE e,"+ baseDossier +".JOURNAL j ";
      requeteSql += "where Vide=0 and e.Code_Journal=j.Code_Journal and j.Code_Journal='"+ codeJournal +"' ";
      requeteSql += "and Date_Ecriture="+ dateEcriture +" order by Num_Ecriture";

			int lastRecord = page * 10;
      int premRecord = lastRecord - 10;

			requeteSql += " limit 10 offset "+ premRecord;

      boolean isWriteable = (writeable == 1);

      Connection con = s.getConnection(null);

      Statement stt = con.createStatement();
      ResultSet rset = stt.executeQuery(requeteSql);

      while (rset.next()) {

        int numecr = rset.getInt("Num_Ecriture");
        resultat.add(new Integer(numecr));
        String typeJournal = rset.getString("Type_Journal");
        isWriteable = isWriteable && !typeJournal.equals("AN");
      }

      int nbecr = getNbEcritures(con, codeJournal, dateEcriture);

      if (isWriteable && (nbecr < lastRecord)) {
        String cle = codeJournal + new Long(dateEcriture).toString();
        Ecriture encours = getEcrEnCours(cle);
        if (encours == null) {
          int numecrvide = creerEcritureVide(codeJournal,dateEcriture);
          encours = getEcriture(numecrvide);
          encours.setEnCours();
          ecrencours.put(cle, encours);
          resultat.add(new Integer(numecrvide));
        } else {
          resultat.add(new Integer(encours.getNumEcriture()));
        }
      }

			s.closeConnection(con, null);

    } catch (Exception e) {
      e.printStackTrace();
    }
    return resultat;
  }
	private Ecriture getEcrEnCours(String cle) {

		Ecriture retour = null;

    if (ecrencours.containsKey(cle)) {
      Ecriture aux = ecrencours.get(cle);
      if (aux.getEnCours()) {
        retour = aux;
      }
    }

    return retour;
  }


	private int getNbEcritures(Connection con, String codeJournal, long dateEcriture) throws SQLException {

		int nbEcr = 0;

    String reqNbEcr = "select count(*) from "+ base +".ECRITURE where Vide=0 "
    								+ "and Code_Journal='"+ codeJournal +"' and Date_Ecriture="+ dateEcriture;

    Statement stt = con.createStatement();
    ResultSet rset = stt.executeQuery(reqNbEcr);
    if (rset.next()) {
      nbEcr = rset.getInt(1);
    }

    return nbEcr;
  }


	private int creerEcritureVide(String codeJournal, long dateEcriture) {

		int retour = 0;

		try {

      Connection con = s.getConnection(null);
      String newEcriture = "insert into "+ base +".ECRITURE (Code_Journal,Date_Ecriture,Vide) "
      								 + "values ('"+ codeJournal +"',"+ dateEcriture +", 1)";

      Statement stt = con.createStatement();
      stt.executeUpdate(newEcriture, Statement.RETURN_GENERATED_KEYS);

			ResultSet rset = stt.getGeneratedKeys();

			if (rset.next()) {
        retour = rset.getInt(1);
      }

			stt.close();

			s.closeConnection(con, null);

    } catch (Exception e) {
      e.printStackTrace();
    }

    return retour;
  }


	public Ecriture getEcriture(int index) throws Exception {

		Ecriture ecriture = null;

    Integer id = new Integer(index);

		boolean stable = true;

    if (ecritures.containsKey(id)) {
      ecriture = ecritures.get(id);
      stable = ecriture.isStable();
    }

    if (stable) {

      Connection con = s.getConnection(null);

      String reqEcriture = "select * from "+ base +".ECRITURE where Num_Ecriture="+ index;

			Statement stt = con.createStatement();

			ResultSet rset = stt.executeQuery(reqEcriture);

			if (rset.next()) {

        long dateEcriture = rset.getLong("Date_Ecriture");
        String codeJournal = rset.getString("Code_Journal");
				int numero = rset.getInt("Numero");

				ecriture = new Ecriture(codeJournal, dateEcriture, index, numero);

				if (rset.getInt("Vide") == 1) {
          Operation op = new Operation();
          ecriture.addOp(op);
        }

				ecritures.put(id, ecriture);
      }

			remplissage(con, ecriture);

			s.closeConnection(con, null);
    }

    return ecriture;
	}


	private void remplissage(Connection con, Ecriture ecr) throws SQLException {

    String reqOps = "select * from "+ base +".OPERATION where Num_Ecriture="+ ecr.getNumEcriture() +" order by Num_Ligne";

		PreparedStatement psMajOp = con.prepareStatement("update "+ base +".OPERATION set Num_Ligne=? where Op_Id=?");

		Statement stt = con.createStatement();

		ResultSet rset = stt.executeQuery(reqOps);

		int numLigne = 0;

		while (rset.next()) {

      int opId = rset.getInt("Op_Id");

			psMajOp.setInt(1, numLigne);
			psMajOp.setInt(2, opId);
			psMajOp.executeUpdate();

      Operation op = new Operation(opId,
                                   numLigne,
                                   rset.getString("Numero_Compte"),
                                   rset.getDouble("Montant_D"),
                                   rset.getDouble("Montant_C"),
                                   rset.getString("Libelle"),
																	 rset.getString("Num_Piece"),
                                   rset.getString("Contrepartie"),
                                   rset.getString("Type"),
																	 rset.getString("Raccourci"),
                                   rset.getString("Lettre"),
																	 rset.getLong("Date_Op"),
																	 rset.getLong("Date_Echeance"),
																	 rset.getInt("Pointage"));
      ecr.addOp(op);
      numLigne++;
    }
  }


	public Ecriture creerEcriture(String codeJournal, long dateEcriture) throws Exception {
    int numecr = creerEcritureVide(codeJournal, dateEcriture);
    return getEcriture(numecr);
	}


	public void supprEcriture(int index) {
    try {

      Connection con = s.getConnection(null);
      Integer id = new Integer(index);

      if (ecritures.containsKey(id)) {
        Ecriture ecr = ecritures.remove(id);
        ecr.supprimer(con, base, baseDossier);
      }

      s.closeConnection(con, null);

    } catch (Exception e) {
      e.printStackTrace();
    }
  }


	public void resetEcriture(int index) {
    try {

      Connection con = s.getConnection(null);
      Integer id = new Integer(index);

      if (ecritures.containsKey(id)) {
        Ecriture ecr = ecritures.get(id);
        ecr.supprimerOperations(con, base, baseDossier);
        Operation op = new Operation();
        ecr.addOp(op);
      }
      s.closeConnection(con, null);

    } catch (Exception e) {
      e.printStackTrace();
    }
  }


} // fin EcrituresSet
