/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

package org.opensi.facturation;


import org.opensi.util.tools.DateTime;

import java.sql.*;
import java.text.DecimalFormat;

import org.experlog.openeas.api.Session;


public class GenDocNumber {

	private long debutPeriode;
	private long finPeriode;
	private long dateDoc;
	
	private int numeroPeriode = -1;
	
	private Connection con;
	private String base;
	
	private Session s;
	
	private String prefixeFacture;
	private String prefixeAvoir;
	
	
	public GenDocNumber(long dateDoc, String s_) throws Exception {
	
		DateTime dt = new DateTime(dateDoc);
		dt.setDay(1);
		debutPeriode = dt.getDateInMillis();
		dt.setMonth(dt.getMonth()+1);
		finPeriode = dt.getDateInMillis();
		
		s = Session.findClient(s_);
		
		con = s.getConnection(null);
		base = s.getCookie().get("BaseDossier");
		this.dateDoc = dateDoc;
		
		Statement stt = con.createStatement();		
		ResultSet rset = stt.executeQuery("select Prefixe_Facture, Prefixe_Avoir from PARAM_DOSSIER where Dossier_Id='"+ s.getCookie().get("Dossier_Id") +"'");
		
		rset.next();
		
		this.prefixeFacture = rset.getString("Prefixe_Facture");
		this.prefixeAvoir = rset.getString("Prefixe_Avoir");
	}
	
	public GenDocNumber(long dateDoc, Connection con, String base, String dossierId) throws Exception {
		
		DateTime dt = new DateTime(dateDoc);
		dt.setDay(1);
		debutPeriode = dt.getDateInMillis();
		dt.setMonth(dt.getMonth()+1);
		finPeriode = dt.getDateInMillis();
		
		this.con = con;
		this.base = base;
		this.dateDoc = dateDoc;
		
		Statement stt = con.createStatement();		
		ResultSet rset = stt.executeQuery("select Prefixe_Facture, Prefixe_Avoir from PARAM_DOSSIER where Dossier_Id='"+ dossierId +"'");
		
		rset.next();
		
		this.prefixeFacture = rset.getString("Prefixe_Facture");
		this.prefixeAvoir = rset.getString("Prefixe_Avoir");
	}

	
	public int getNumeroPeriode() {
		
		return numeroPeriode;
	}
	
	
	public int getNumUnique() throws SQLException {
		
		Statement stt = con.createStatement();
		
		ResultSet rsNU = stt.executeQuery("select coalesce(max(nu), 0)+1 from (select max(Num_Unique) as nu from "+ base +".FACTURE union select max(Num_Unique) as nu from "+ base +".AVOIR) as tnu");
		
		rsNU.next();
		
		return rsNU.getInt(1);
	}
	
	
	private void getMaxNumeroAF() throws SQLException {
		
		PreparedStatement psDernierAvoir = con.prepareStatement("select count(Num_Entier) from "+ base +".AVOIR where Date_Avoir>=? and Date_Avoir<? and Numero>0");
		
		psDernierAvoir.setLong(1, debutPeriode);
		psDernierAvoir.setLong(2, finPeriode);
		
		ResultSet rsDA = psDernierAvoir.executeQuery();
		rsDA.next();
		
		PreparedStatement psDernierFacture = con.prepareStatement("select count(Num_Entier) from "+ base +".FACTURE where Date_Facture>=? and Date_Facture<? and Numero>0"); 
		
		psDernierFacture.setLong(1, debutPeriode);
		psDernierFacture.setLong(2, finPeriode);
		
		ResultSet rsDF = psDernierFacture.executeQuery();
		rsDF.next();
		
		numeroPeriode = rsDA.getInt(1) + rsDF.getInt(1) + 1;
	}
		
	
	public String getNumeroFacture() throws SQLException {	
		
		getMaxNumeroAF();
		
		DecimalFormat nf = new DecimalFormat("0000");
		
		return prefixeFacture + DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);		
	}
	
	
	public String getNumeroAvoir() throws SQLException {
		
		getMaxNumeroAF();
		
		DecimalFormat nf = new DecimalFormat("0000");
	
		return prefixeAvoir + DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);
	}
	
	
	public String getNumeroProforma() throws SQLException {	
		
		PreparedStatement psDernierProforma = con.prepareStatement("select count(Num_Entier) from "+ base +".PROFORMA where Date_Proforma>=? and Date_Proforma<? and Numero>0"); 
		
		psDernierProforma.setLong(1, debutPeriode);
		psDernierProforma.setLong(2, finPeriode);
		
		ResultSet rsDP = psDernierProforma.executeQuery();
		rsDP.next();
		
		numeroPeriode = rsDP.getInt(1) + 1;
		
		DecimalFormat nf = new DecimalFormat("0000");
		
		return "P"+ DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);		
	}
	
	
	public String getNumeroAffaire() throws SQLException {
		
		// on prend le max car une affaire ayant un numro peut tre supprime
		PreparedStatement psDerniereAffaire = con.prepareStatement("select max(Numero) from "+ base +".AFFAIRE where Date_Creation>=? and Date_Creation<? and Numero>0");
		
		psDerniereAffaire.setLong(1, debutPeriode);
		psDerniereAffaire.setLong(2, finPeriode);
		
		ResultSet rsAff = psDerniereAffaire.executeQuery();
		rsAff.next();
		
		numeroPeriode = rsAff.getInt(1) + 1;
		
		DecimalFormat nf = new DecimalFormat("0000");
		
		return "AC"+ DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);
	}
	
	
	public String getNumeroCommande() throws SQLException {
		
		// on prend le max car une commande fournisseur ayant un numro peut tre supprime
		PreparedStatement psDerniereCommande = con.prepareStatement("select max(Numero) from "+ base +".COMMANDE where Date_Commande>=? and Date_Commande<? and Numero>0");
		
		psDerniereCommande.setLong(1, debutPeriode);
		psDerniereCommande.setLong(2, finPeriode);
		
		ResultSet rsCom = psDerniereCommande.executeQuery();
		rsCom.next();
		
		numeroPeriode = rsCom.getInt(1) + 1;
		
		DecimalFormat nf = new DecimalFormat("0000");
		
		return "C"+ DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);
	}
	
	
	public String getNumeroDevis() throws SQLException {
		
		// on prend le max car un devis ayant un numro peut tre supprim
		PreparedStatement psDernierDevis = con.prepareStatement("select max(Numero) from "+ base +".DEVIS where Date_Devis>=? and Date_Devis<? and Numero>0");
		
		psDernierDevis.setLong(1, debutPeriode);
		psDernierDevis.setLong(2, finPeriode);
		
		ResultSet rsDevis = psDernierDevis.executeQuery();
		rsDevis.next();
		
		numeroPeriode = rsDevis.getInt(1) + 1;
		
		DecimalFormat nf = new DecimalFormat("0000");
		
		return "D"+ DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);
	}
	
	
	public String getNumeroBL() throws SQLException {
		
		PreparedStatement psDernierBL = con.prepareStatement("select count(Num_Entier) from "+ base +".BON_LIVRAISON where Date_Liv>=? and Date_Liv<? and Numero>0");
		
		psDernierBL.setLong(1, debutPeriode);
		psDernierBL.setLong(2, finPeriode);
		
		ResultSet rsBL = psDernierBL.executeQuery();
		rsBL.next();
		
		numeroPeriode = rsBL.getInt(1) + 1;
		
		DecimalFormat nf = new DecimalFormat("0000");
			
		return "BL"+ DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);
	}
	
	
	public String getNumeroBR() throws SQLException {
		
		PreparedStatement psDernierBR = con.prepareStatement("select count(Num_Entier) from "+ base +".BON_RECEPTION where Date_BR>=? and Date_BR<? and Numero>0");
		
		psDernierBR.setLong(1, debutPeriode);
		psDernierBR.setLong(2, finPeriode);
		
		ResultSet rsBR = psDernierBR.executeQuery();
		rsBR.next();
		
		numeroPeriode = rsBR.getInt(1) + 1;
		
		DecimalFormat nf = new DecimalFormat("0000");
		
		return "BR"+ DateTime.formatTime(dateDoc, "yyMM") + nf.format(numeroPeriode);
	}
	
	
	protected void finalize() throws SQLException {
	
		s.closeConnection(con, null);
	}	
	

} // fin GenDocNumber

