/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Donnes ncessaires  l'dition d'un bon de prparation standard
 */

package org.opensi.facturation.actions.documents.modeles;


import java.sql.*;
import java.util.ArrayList;

import org.experlog.openeas.api.Session;
import org.opensi.util.tools.StringUtils;


public class DataBonPreparation {


	// les lignes du bon de prparation
	protected ArrayList<LigneBonPreparation> lignes = new ArrayList<LigneBonPreparation>(10);	


	// coordonnes socit mettrice

	protected String raisonSociale;
	protected String adresse;
	protected String compAdresse;
	protected String codePostal;
	protected String ville;
	protected String tel;
	protected String fax;
	protected String email;
	protected String web;
	
	protected double capital;
	protected String typeSociete;
	protected String tvaIntra;
	protected String rcs;
	protected String siren;
	
	// coordonnes du destinataire
	
	protected String clientId;
	protected String raisonSocialeDest;
	protected String adresseDest;
	protected String compAdresseDest;
	protected String adresseDest3;
	protected String codePostalDest;
	protected String villeDest;
	protected String paysDest;
	
	protected String raisonSocialeFact;
	protected String adresseFact;
	protected String compAdresseFact;
	protected String adresseFact3;
	protected String codePostalFact;
	protected String villeFact;
	protected String paysFact;
	protected String interlocuteurFact;
	
	// contact de livraison
	protected String interlocuteurLiv = "";
	protected String telInterLiv;
	protected String faxInterLiv;
	protected String emailInterLiv;
	
	// infos bon de prparation
	
	protected String numeroBon;
	protected String comWebId;
	protected String refCommande;
	protected String refAffaire;
	protected String modeExpedition;
	protected String urlLogo = "";
	protected String urlAnnule = "";	
	protected String comFin = "";
	protected String mentions = "";
	
	protected long dateBon;	
	
	protected boolean coul = false;
	protected boolean tail = false;
	protected boolean unite = false;
	protected boolean nbPieces = false;
	protected boolean numLot = false;
	protected boolean datePeremption = false;
	protected boolean comWeb;
	protected boolean annule;
	protected boolean logoAdr;
		
	
	public DataBonPreparation(Session s, String bonId) {	
		try {
		
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
      
			
			String base = s.getCookie().get("BaseDossier");
			String dossierId = s.getCookie().get("Dossier_Id");
			
			this.urlLogo = s.getAppConfiguration().get("opensi.absdir") +"/logos/"+ dossierId +".jpg";
			this.urlAnnule = s.getAppConfiguration().get("opensi.absdir") +"/images/annule.jpg";
      
			ResultSet rset;
			
      
			// coordonnes de l'entreprise facturante
      
			String reqFacturante = "select Ville_RCS, Num_TVA_Intra, Type_Societe, Montant_Capital, Num_SIRET, Raison_Sociale, Adresse, Comp_Adresse, Code_Postal, Ville, Telephone, Fax, Email, Site_Web from DOSSIER where Dossier_Id='"+ dossierId +"'";
			
			rset = stt.executeQuery(reqFacturante);
			
			rset.next();
			
			this.raisonSociale = rset.getString("Raison_Sociale");
			this.adresse = rset.getString("Adresse");
			this.compAdresse = rset.getString("Comp_Adresse");
			this.codePostal = rset.getString("Code_Postal");
			this.ville = rset.getString("Ville");
			this.tel = rset.getString("Telephone");
			this.fax = rset.getString("Fax");
			this.email = rset.getString("Email");
			this.web = rset.getString("Site_Web");
			
			this.siren = formatSiren(rset.getString("Num_SIRET"));
			this.typeSociete = rset.getString("Type_Societe");
			this.capital = rset.getDouble("Montant_Capital");
			this.tvaIntra = rset.getString("Num_TVA_Intra");
			this.rcs = rset.getString("Ville_RCS");
			
			rset.close();
			
			
			// Elments du bon de livraison
			
			String reqBL = "select Num_Entier, Imp_Com, Commentaires, Mentions, Commentaires_2, Com_Web_Id, Etat, Nom_Inter_Liv, Prenom_Inter_Liv, c.Civ_Courte, Tel_Inter_Liv, Fax_Inter_Liv, Email_Inter_Liv from "+ base +".BON_LIVRAISON, CIVILITE c where c.Civ_Id=Civ_Inter_Liv and Bon_Id="+ bonId;
			
			rset = stt.executeQuery(reqBL);
			
			rset.next();
			
			this.numeroBon = rset.getString("Num_Entier");
			this.dateBon = System.currentTimeMillis();			
			this.comWebId = rset.getString("Com_Web_Id");
			this.comWeb = (rset.getInt("Com_Web_Id")!=0);
			this.annule = rset.getString("Etat").equals("A");
			
			this.comFin = rset.getString("Commentaires");
			
			if (rset.getString("Commentaires_2")!=null) {
				this.comFin += "\n\n"+ rset.getString("Commentaires_2");
			}
			this.mentions = "\n" + rset.getString("Mentions");
			String civilite = rset.getString("Civ_Courte");
			String nom = rset.getString("Nom_Inter_Liv");
			this.interlocuteurLiv = (!civilite.equals("")?civilite+" ":"")+ (!nom.equals("")?nom+" "+rset.getString("Prenom_Inter_Liv"):"");
			this.telInterLiv = rset.getString("Tel_Inter_Liv");
			this.faxInterLiv = rset.getString("Fax_Inter_Liv");
			this.emailInterLiv = rset.getString("Email_Inter_Liv");
			
			rset.close();
			
			
			if (comWeb) {
				// commande web
				String reqCommandeWeb = "select b.Nom_Liv, b.Adresse_Liv, b.Comp_Adresse_Liv, b.Adresse_3_Liv, b.CP_Liv, b.Ville_Liv, p1.Nom_FR as Pays_Liv,";
				reqCommandeWeb += " c.Nom_Inter, c.Prenom_Inter, c.Societe_Client, c.Adresse_Client_1, c.Adresse_Client_2, c.Adresse_Client_3, c.CP_Client, c.Ville_Client, p2.Nom_FR as Pays_Client,";
				reqCommandeWeb += " s.Nom_site, c.Com_Site_Id, c.Mode_Expedition, civ.Civ_Courte, if(c.Site_Id=0,c.Client_Web_Id,'') as Client_Id";
				reqCommandeWeb += " from "+ base +".BON_LIVRAISON b, "+ base +".COMMANDE_WEB c left join "+ base +".SITE_WEB s on s.Site_Id=c.Site_Id, PAYS p1, PAYS p2, CIVILITE civ";
				reqCommandeWeb += " where c.Civ_Inter=civ.Civ_Id and b.Code_Pays_Liv=p1.Code_Pays and c.Code_Pays=p2.Code_Pays and b.Bon_Id="+ bonId +" and b.Com_Web_Id=c.Com_Web_Id";

				rset = stt.executeQuery(reqCommandeWeb);

				rset.next();
			
				this.raisonSocialeDest = rset.getString("Nom_Liv");
				this.adresseDest = rset.getString("Adresse_Liv");
				this.compAdresseDest = rset.getString("Comp_Adresse_Liv");
				this.adresseDest3 = rset.getString("Adresse_3_Liv");
				this.codePostalDest = rset.getString("CP_Liv");
				this.villeDest = rset.getString("Ville_Liv");
				this.paysDest = rset.getString("Pays_Liv");				
				this.clientId = rset.getString("Client_Id");
				this.refCommande = rset.getString("Com_Site_Id");
				this.modeExpedition = rset.getString("Mode_Expedition");

				civilite=rset.getString("Civ_Courte");
				
				this.interlocuteurFact = civilite+" "+rset.getString("Nom_Inter")+" "+rset.getString("Prenom_Inter");
				this.raisonSocialeFact = (!rset.getString("Societe_Client").equals("") && !rset.getString("Societe_Client").equals(" "))?rset.getString("Societe_Client"):"";
				this.adresseFact = rset.getString("Adresse_Client_1");
				this.compAdresseFact = rset.getString("Adresse_Client_2");
				this.adresseFact3 = rset.getString("Adresse_Client_3");
				this.codePostalFact = rset.getString("CP_Client");
				this.villeFact = rset.getString("Ville_Client");
				this.paysFact = rset.getString("Pays_Client");				

				rset.close();				
			}
			else {
				// affaire
				String reqAffaire = "select a.Raison_Sociale, a.Affaire_Id, a.Client_Id, a.Num_Entier, b.Nom_Liv, b.Adresse_Liv, b.Comp_Adresse_Liv, b.Adresse_3_Liv, b.CP_Liv, b.Ville_Liv, p.Nom_FR as Pays_Liv ";
				reqAffaire += "from "+ base +".AFFAIRE a, "+ base +".BON_LIVRAISON b, PAYS p where b.Code_Pays_Liv=p.Code_Pays and a.Affaire_Id=b.Affaire_Id and b.Bon_Id="+ bonId;
				
				rset = stt.executeQuery(reqAffaire);
				
				rset.next();
				
				// coordonnes de l'entreprise facture
				this.raisonSocialeFact = rset.getString("Raison_Sociale");
				this.raisonSocialeDest = rset.getString("Nom_Liv");
				this.adresseDest = rset.getString("Adresse_Liv");
				this.compAdresseDest = rset.getString("Comp_Adresse_Liv");
				this.adresseDest3 = rset.getString("Adresse_3_Liv");
				this.codePostalDest = rset.getString("CP_Liv");
				this.villeDest = rset.getString("Ville_Liv");
				this.paysDest = rset.getString("Pays_Liv");				
				this.clientId = rset.getString("Client_Id");				
				
				this.refAffaire = rset.getString("Num_Entier");
				
				int affaireId = rset.getInt("Affaire_Id");
				
				rset.close();
				
				
				// rfrence commande
	
				String reqCommande = "select Ref_Com from "+ base +".COM_CLIENT where Etat='V' and Affaire_Id="+ affaireId;
	
				rset = stt.executeQuery(reqCommande);
	
				rset.next();
	
				this.refCommande = rset.getString("Ref_Com");
	
				rset.close();
			}
			      
			
			// Paramtres dossier
      
			String reqCom = "select Logo_Adr from PARAM_DOSSIER where Dossier_Id='"+ dossierId +"'";
			
			rset = stt.executeQuery(reqCom);
			
			rset.next();
			
			logoAdr = rset.getInt("Logo_Adr")==1;
			
			rset.close();	
      
      
      // selection des articles appartenant au bon de livraison
			
			String reqArticles = "";
			
			if (comWeb) {		  			
				reqArticles = "select * from ((select d.Libelle, d.Article_Id as Reference, d.Designation, b.Quantite, d.Type_Ligne, d.Commentaire, d.Commentaire_Avant, f.Localisation, d.Unite, b.Num_Lot, b.Nb_Pieces, b.Date_Peremption "
													 + "from "+ base +".ARTICLE_BON_LIVRAISON b,"+ base +".ARTICLE_COMMANDE_WEB d,"+ base +".FICHE_ARTICLE f "
										    	 + "where d.Type_Ligne='S' and d.Article_Id=f.Article_Id and b.Ref_Ligne_Web=d.Detail_Id and b.Bon_Id="+ bonId +" order by f.Localisation, b.Rank)"

													 + " union all "

													 + "(select d.Libelle, d.Article_Id as Reference, d.Designation, b.Quantite, d.Type_Ligne, d.Commentaire, d.Commentaire_Avant, '-' as Localisation, d.Unite, b.Num_Lot, b.Nb_Pieces, b.Date_Peremption "
													 + "from "+ base +".ARTICLE_BON_LIVRAISON b,"+ base +".ARTICLE_COMMANDE_WEB d "
										    	 + "where d.Type_Ligne<>'S' and b.Ref_Ligne_Web=d.Detail_Id and b.Bon_Id="+ bonId +" order by b.Rank)) as articles order by Localisation";
	  	}
	  	else {
				reqArticles = "select * from ((select d.Libelle, d.Reference, d.Designation, b.Quantite, d.Type_Ligne, d.Commentaire, d.Commentaire_Avant, f.Localisation, d.Unite, b.Num_Lot, b.Nb_Pieces, b.Date_Peremption "
					 + "from "+ base +".ARTICLE_BON_LIVRAISON b,"+ base +".ARTICLE_COM_CLIENT d,"+ base +".FICHE_ARTICLE f "
		    	 + "where d.Type_Ligne='S' and d.Reference=f.Article_Id and b.Ref_Ligne=d.Ligne_Id and b.Bon_Id="+ bonId +" order by f.Localisation, b.Rank)"

					 + " union all "

					 + "(select d.Libelle, d.Reference, d.Designation, b.Quantite, d.Type_Ligne, d.Commentaire, d.Commentaire_Avant, '-' as Localisation, d.Unite, b.Num_Lot, b.Nb_Pieces, b.Date_Peremption "
					 + "from "+ base +".ARTICLE_BON_LIVRAISON b,"+ base +".ARTICLE_COM_CLIENT d "
		    	 + "where d.Type_Ligne<>'S' and b.Ref_Ligne=d.Ligne_Id and b.Bon_Id="+ bonId +" order by b.Rank)) as articles order by Localisation";
	  	}
			
			
			  
			//String reqArticles = "select Libelle, Reference, Prix, Designation, Quantite, Ristourne, Type_Ligne, Commentaire from "+ base +".ARTICLE_DEVIS where Devis_Id="+ devisId;
			rset = stt.executeQuery(reqArticles);

			
			String reqInfosSup = "select Couleur, Taille, Descrip_1, Descrip_2, Imp_Nom_BP, Imp_Desc1_BP, Imp_Desc2_BP, Ref_Fabricant "
												 + "from "+ base +".FICHE_ARTICLE where Article_Id=?";
			
			PreparedStatement psInfosSup = con.prepareStatement(reqInfosSup);
			
			
			String reqNomenclature = "select n.ArticleComp_Id, n.Quantite, f.Designation "
														 + "from "+ base +".FICHE_ARTICLE f,"+ base +".COMPOSANT_ARTICLE n where f.Article_Id=n.ArticleComp_Id and n.Article_Id=?";
			
			PreparedStatement psNomenclature = con.prepareStatement(reqNomenclature);
					
			String reqStock = "select Stock_Init+Entrees-Sorties as Stock_Reel,Stock_Init+Entrees-Sorties-Com_Clients as Stock_Dispo "
				 + "from "+ base +".STOCKS_ARTICLE where Article_Id=?";

			PreparedStatement psStock = con.prepareStatement(reqStock);

			
			while (rset.next()) {
				
				LigneBonPreparation lb = new LigneBonPreparation();
        
				String libelle = rset.getString("Libelle");
				
				if (libelle.length()>0) {
					libelle = "\n"+libelle;
				}			
				
				lb.setReference(rset.getString("Reference"));
				lb.setDesignation(rset.getString("Designation") + libelle);
				lb.setQuantite(rset.getDouble("Quantite"));
				lb.setCommentaire(rset.getString("Commentaire"));
				lb.setCommentaireAvant(rset.getString("Commentaire_Avant"));
				lb.setLocalisation(rset.getString("Localisation"));
				lb.setType('A');
				
				lb.setUnite(rset.getString("Unite"));
				lb.setNumLot(rset.getString("Num_Lot"));
				lb.setNbPieces(rset.getInt("Nb_Pieces"));
				lb.setDatePeremption(rset.getLong("Date_Peremption"));
				
				lignes.add(lb);
				
				if (!lb.unite.equals("U")) unite = true;
				if (!lb.num_lot.equals("")) numLot = true;
				if (lb.nb_pieces != 0) nbPieces = true;
				if (lb.date_peremption != 0) datePeremption = true;
				
				
				if (rset.getString("Type_Ligne").equalsIgnoreCase("S")) {					
				
					// rcupration infos supplmentaires si existent
					psInfosSup.setString(1, lb.reference);
					ResultSet rsIS = psInfosSup.executeQuery();

					if (rsIS.next()) {
						
						psStock.setString(1, lb.reference);
						ResultSet rsSt = psStock.executeQuery();
						if (rsSt.next()) {
							lb.setStockReel(rsSt.getDouble("Stock_Reel"));
							lb.setStockDispo(rsSt.getDouble("Stock_Dispo"));
						}

						lb.setCouleur(rsIS.getString("Couleur"));
						lb.setTaille(rsIS.getString("Taille"));

						if (rsIS.getString("Couleur").length()>0) {
							coul = true;
						}

						if (rsIS.getString("Taille").length()>0) {
							tail = true;
						}
						
						if (rsIS.getInt("Imp_Desc1_BP")==1) {
							lb.setDetail_1(StringUtils.removeHtmlTags(rsIS.getString("Descrip_1")));
						}
						if (rsIS.getInt("Imp_Desc2_BP")==1) {
							lb.setDetail_2(StringUtils.removeHtmlTags(rsIS.getString("Descrip_2")));
						}

						lb.setRefFabricant(rsIS.getString("Ref_Fabricant"));

						// gestion de la nomenclature
						if (rsIS.getInt("Imp_Nom_BP")==1)	{

							psNomenclature.setString(1, lb.reference);
							ResultSet rsN = psNomenclature.executeQuery();

							while (rsN.next()) {
								LigneBonPreparation ln = new LigneBonPreparation();

								ln.setReference(rsN.getString("ArticleComp_Id"));
								ln.setDesignation(rsN.getString("Designation"));
								ln.setQuantite(rsN.getDouble("Quantite"));
								ln.setType('N');			

								lignes.add(ln);
							}

							rsN.close();
						}
						
					}
					rsIS.close();								
				}				
			}
			
      rset.close();
			stt.close();			
			s.closeConnection(con, null);
			
		} catch(Exception e) {
			e.printStackTrace();
		}	
	}
	
	
	
	public int getNbLignes() {
		return lignes.size();
	}
	
	
	public String formatSiren(String siret) {
		if (siret.length()>=9) {
			return siret.substring(0,3) +" "+ siret.substring(3,6) +" "+ siret.substring(6,9);
		}
		else {
			return siret;
		}
	}
	
	

} // fin DataBonPreparation
