/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Informations utiles pour l'edition de la facture
 */

package org.opensi.facturation.actions.documents.modeles;

import java.sql.*;

import java.util.ArrayList;

import org.experlog.openeas.api.Session;
import org.opensi.util.tools.Arrondi;
import org.opensi.util.tools.StringUtils;

import java.text.DecimalFormatSymbols;
import java.text.DecimalFormat;

import java.text.SimpleDateFormat;


public class DataFactureFournisseur {

	private DecimalFormatSymbols dfs2 = new DecimalFormatSymbols();
	private DecimalFormat numFormat2 = new DecimalFormat("0.00");
	private DecimalFormat numFormatQte = new DecimalFormat("0.##");
	private SimpleDateFormat formatDate = new SimpleDateFormat("dd.MM.yy");

	// liste des articles de la facture

	protected ArrayList<LigneFactureFournisseur> lignes = new ArrayList<LigneFactureFournisseur>(10);
	protected ArrayList<VentilTVA> vtvas = new ArrayList<VentilTVA>(2);

	// infos socit facturante

	protected String raisonSociale;
	protected String adresseSociete1;
	protected String adresseSociete2;
	protected String codePostal;
	protected String ville;
	protected String tel;
	protected String fax;
	protected String email;
	protected String web;

	// infos socit facture

	protected String fournisseurId;
	protected String societeFact;
	protected String adresseFact1;
	protected String adresseFact2;
	protected String adresseFact3;
	protected String codePostalFact;
	protected String villeFact;
	protected String paysFact;
	protected String numTvaFournisseur = "";

	// Infos facture

	protected double capital;
	protected String typeSociete;
	protected String tvaIntra;
	protected String rcs;
	protected String siren;
	protected String numFact;
	protected long dateFact;
	//protected String responsable;
	protected String interlocuteur;
	protected String modeReglement;
	protected long echeance;

	protected String comFin = "";
	protected String mentions = "";
	protected String typeVente;

	protected double tRemise;
	protected double tEscompte = 0;
	protected double mRemise = 0;
	protected double mEscompte = 0;
	protected double fraisPort;
	protected double acompte;
	protected double tauxTvaPort;
	protected double montantBase = 0;
	protected double totalBase = 0;
	protected double totalTVA = 0;
	protected double netAPayer = 0;
	protected double montantTTC = 0;

	protected boolean rist = false;
	protected boolean coul = false;
	protected boolean tail = false;
	protected boolean unite = false;
	protected boolean nbPieces = false;
	protected boolean numLot = false;
	protected boolean datePeremption = false;
	protected boolean logoAdr;
	protected String urlLogo = "";
	protected int nbex;

	protected String refCommande;
	protected String refAffaire;
	protected String refBL;
	
	private Arrondi r = new Arrondi(2);
	

	public DataFactureFournisseur(Session s, String Facture_Id) {

		try {

			dfs2.setDecimalSeparator('.');

			numFormat2.setDecimalFormatSymbols(dfs2);
			numFormatQte.setDecimalFormatSymbols(dfs2);

			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
			Statement stt2 = con.createStatement();
			Statement stt3 = con.createStatement();

			String dossier = s.getCookie().get("Dossier_Id");
			String base = s.getCookie().get("BaseDossier");
			
			this.urlLogo = s.getAppConfiguration().get("opensi.absdir") +"/logos/"+ dossier +".jpg";
			
			ResultSet rset;


			// Elments de la facture

			String reqFacture = "select f.*, c.Civ_Courte, p.Nom_FR as Pays from "+ base +".FACTURE_FOURNISSEUR f, CIVILITE c, PAYS p";
			reqFacture += " where c.Civ_Id=f.Civ_Inter and f.Code_Pays=p.Code_Pays and f.Facture_Id="+ Facture_Id;
			
			rset = stt.executeQuery(reqFacture);

			rset.next();

			this.numFact = rset.getString("Num_Entier");
			this.dateFact = rset.getLong("Date_Facture");
			this.tEscompte = rset.getDouble("Escompte");
			this.tRemise = rset.getDouble("Remise");
			this.fraisPort = rset.getDouble("Frais_Port");
			this.echeance = rset.getLong("Echeance");
			this.acompte = rset.getDouble("Acompte");
			this.modeReglement = rset.getString("Mode_Reg");
			this.fournisseurId = rset.getString("Fournisseur_Id");
			this.typeVente = rset.getString("Type_Achat");
			this.tauxTvaPort = rset.getDouble("Taux_TVA_Port");

			if (rset.getInt("Imp_Com")==1 && rset.getString("Commentaires").length()>0) {
				this.comFin = "\n"+ rset.getString("Commentaires");
			}
			this.mentions = "\n" + rset.getString("Mentions");

			this.societeFact = rset.getString("Raison_Sociale");
			this.adresseFact1 = rset.getString("Adresse_Fact");
			this.adresseFact2 = rset.getString("Comp_Adresse_Fact");
			this.adresseFact3 = rset.getString("Adresse_3_Fact");
			this.codePostalFact = rset.getString("CP_Fact");
			this.villeFact = rset.getString("Ville_Fact");
			this.paysFact = rset.getString("Pays");

			if (rset.getString("Nom_Inter").equals("")) {
				this.interlocuteur = "";
			}
			else {
				this.interlocuteur = rset.getString("Civ_Courte") +" "+ rset.getString("Nom_Inter").toUpperCase() + " " + rset.getString("Prenom_Inter");
			}

			rset.close();


			// coordonnes de l'entreprise facturante

	      String reqFacturante = "select Ville_RCS, Num_TVA_Intra, Type_Societe, Montant_Capital, Num_SIRET, Raison_Sociale,";
	      reqFacturante += " Adresse, Comp_Adresse, Code_Postal, Ville, Telephone, Fax, Email, Site_Web ";
	      reqFacturante += "from DOSSIER where Dossier_Id='"+ dossier +"'";

			rset = stt.executeQuery(reqFacturante);

			rset.next();

			this.raisonSociale = rset.getString("Raison_Sociale");
			this.adresseSociete1 = rset.getString("Adresse");
			this.adresseSociete2 = rset.getString("Comp_Adresse");
			this.codePostal = rset.getString("Code_Postal");
			this.ville = rset.getString("Ville");
			this.tel = rset.getString("Telephone");
			this.fax = rset.getString("Fax");
			this.email = rset.getString("Email");
			this.web = rset.getString("Site_Web");
			this.siren = formatSiren(rset.getString("Num_SIRET"));
			this.typeSociete = rset.getString("Type_Societe");
			this.capital = rset.getDouble("Montant_Capital");
			this.tvaIntra = rset.getString("Num_TVA_Intra");
			this.rcs = rset.getString("Ville_RCS");

			rset.close();

			// rfrences affaire, BL et commandes

			this.refAffaire = "";
			this.refCommande = "";
			this.refBL = "";

			int nbBons = 0;

			// nombre d'exemplaires  sortir

	      if (fournisseurId.equals("")) {
	        this.comFin += "\n\n";
	        this.nbex = 1;
	      }
	      else {

	    	  String reqEx = "select Nb_Ex, Num_TVA_Intra from "+ base +".FICHE_FOURNISSEUR where Fournisseur_Id='"+ fournisseurId +"'";
			  rset = stt.executeQuery(reqEx);

			  if (rset.next()) {
			    this.nbex = rset.getInt("Nb_Ex") + 1;
					this.numTvaFournisseur = rset.getString("Num_TVA_Intra");
			  } else {
				  this.comFin += "\n\n";
				  this.nbex = 1;
				  this.numTvaFournisseur = "";
			  }
			  rset.close();
	      }

			// commentaires sur facture (dossier)

	      String reqCom = "select Com_Fact, Logo_Adr from PARAM_DOSSIER where Dossier_Id='"+ dossier +"'";

			rset = stt.executeQuery(reqCom);

			rset.next();

			logoAdr = rset.getInt("Logo_Adr")==1;

			if (rset.getString("Com_Fact").length()>0) {
				this.comFin += "\n\n"+rset.getString("Com_Fact");
			}

			rset.close();
			
			
			// ventilation des TVA		
			
			String reqVTvas = "select a.Taux_TVA, sum(a.Prix * a.Quantite * (1-a.Ristourne/100)) as Base_Montant "
										 + "from "+ base +".ARTICLE_FACTURE_FOURNISSEUR a, "+ base +".TAUX_TVA t where Facture_Id="+ Facture_Id +" and a.Code_TVA=t.Code_TVA group by a.Taux_TVA order by a.Taux_TVA";
			
			rset = stt.executeQuery(reqVTvas);
			
			while (rset.next()) {
			
				double tauxTva = rset.getDouble("Taux_TVA");
			
				vtvas.add(new VentilTVA(rset.getDouble("Base_Montant") * (1-tRemise/100), tauxTva));				
				totalTVA += ((tauxTva/100) * rset.getDouble("Base_Montant") * (1-tRemise/100));
			}
			
			rset.close();


			// selection des articles appartenant  la facture

			String reqArticles = "select * from "+ base +".ARTICLE_FACTURE_FOURNISSEUR where Facture_Id="+ Facture_Id;
			rset = stt.executeQuery(reqArticles);

			PreparedStatement psInfosSup = con.prepareStatement("select Couleur, Taille from "+ base +".FICHE_ARTICLE where Article_Id=?");

			while (rset.next()) {
				
				String commentaireAvant = rset.getString("Commentaire_Avant");				
				
				if (commentaireAvant.length()>0) {
				
					String tabCom[] = commentaireAvant.split("\n");
					
					for (int i=0; i<tabCom.length; i++) {
					
						LigneFactureFournisseur la = new LigneFactureFournisseur();
						la.setType('C');
						la.setDesignation(tabCom[i]);
						la.setSousTotal(montantBase);
						lignes.add(la);
					}
				}

				LigneFactureFournisseur lf = new LigneFactureFournisseur();

				lf.setPrice(rset.getDouble("Prix"));
				lf.setReference(rset.getString("Reference"));
				lf.setRef_Fournisseur(rset.getString("Ref_Fournisseur"));
				lf.setDesignation(rset.getString("Designation"));
				lf.setQte(rset.getDouble("Quantite"));
				lf.setRistourne(rset.getDouble("Ristourne"));
				lf.setTypeLigne(rset.getString("Type_Ligne"));
				lf.setTva(rset.getDouble("Taux_TVA"));
				lf.setMontant(lf.price*lf.qte*(1-lf.ristourne/100));
				lf.setDetail_1("");
				lf.setDetail_2("");
				lf.setRefBL("");
				lf.setUnite(rset.getString("Unite"));
				lf.setNumLot(rset.getString("Num_Lot"));
				lf.setNbPieces(rset.getInt("Nb_Pieces"));
				lf.setDatePeremption(rset.getLong("Date_Peremption"));
				lf.setType('A');
				
				montantBase += lf.montant;
				
				lf.setSousTotal(montantBase);
				
				if (lf.ristourne!=0) { rist = true; }
				if (!lf.unite.equals("U")) unite = true;
				if (!lf.num_lot.equals("")) numLot = true;
				if (lf.nb_pieces != 0) nbPieces = true;
				if (lf.date_peremption != 0) datePeremption = true;

				ResultSet rset2;

				if (nbBons>1) {
					String reqRefBL = "select b.Num_Entier, ab.Quantite, b.Date_Liv from "+ base +".BON_RECEPTION b, "+ base +".ARTICLE_BON_RECEPTION ab ";
					reqRefBL += "where b.BR_Id=ab.BR_Id and b.Facture_Id="+ Facture_Id +" and ab.Ref_Ligne="+ rset.getString("Ref_Ligne");

					rset2 = stt2.executeQuery(reqRefBL);

					while (rset2.next()) {
						lf.setRefBL(lf.refBL + formatDate.format(new Date(rset2.getLong("Date_Liv"))) +" BL N "+ rset2.getString("Num_Entier") +" : Qt = "+ numFormatQte.format(rset2.getDouble("Quantite")) +"\n");
					}

					rset2.close();
				}
				
				lignes.add(lf);
				
				
				String commentaireApres = rset.getString("Commentaire");				
				
				if (commentaireApres.length()>0) {
				
					String tabCom[] = commentaireApres.split("\n");
					
					for (int i=0; i<tabCom.length; i++) {
					
						LigneFactureFournisseur lc = new LigneFactureFournisseur();
						lc.setType('C');
						lc.setDesignation(tabCom[i]);
						lc.setSousTotal(montantBase);
						lignes.add(lc);
					}
				}


				if (rset.getString("Type_Ligne").equalsIgnoreCase("S")) {


					psInfosSup.setString(1, lf.reference);
					ResultSet rsIS = psInfosSup.executeQuery();

					if (rsIS.next()) {

						lf.setCouleur(rsIS.getString("Couleur"));
						lf.setTaille(rsIS.getString("Taille"));

						if (rsIS.getString("Couleur").length()>0) {
							coul = true;
						}

						if (rsIS.getString("Taille").length()>0) {
							tail = true;
						}
					}


					rset2 = stt2.executeQuery("select Descrip_1, Descrip_2, Imp_Desc1_Facture, Imp_Desc2_Facture, Imp_Nom_Facture from "+ base +".FICHE_ARTICLE where Article_Id='"+ rset.getString("Reference") +"'");
					rset2.next();

					if (rset2.getInt("Imp_Desc1_Facture")==1) {
						lf.setDetail_1(StringUtils.removeHtmlTags(rset2.getString("Descrip_1")));
					}
					if (rset2.getInt("Imp_Desc2_Facture")==1) {
						lf.setDetail_2(StringUtils.removeHtmlTags(rset2.getString("Descrip_2")));
					}

					// gestion de la nomenclature

					if (rset2.getInt("Imp_Nom_Facture")==1)	{

						ResultSet rset3 = stt3.executeQuery("select n.ArticleComp_Id, n.Quantite, f.Designation from "+ base +".FICHE_ARTICLE f,"+ base +".COMPOSANT_ARTICLE n where f.Article_Id=n.ArticleComp_Id and n.Article_Id='"+ rset.getString("Reference") +"'");

						while (rset3.next()) {
							LigneFactureFournisseur ln = new LigneFactureFournisseur();

							ln.setReference(rset3.getString("ArticleComp_Id"));
							ln.setDesignation(rset3.getString("Designation"));
							ln.setQte(rset3.getDouble("Quantite"));
							ln.setType('N');
							ln.setSousTotal(montantBase);	

							lignes.add(ln);
						}

						rset3.close();
					}

					rset2.close();
				}
				
			}
			
			
			mRemise = montantBase * tRemise/100;
			totalBase = montantBase + fraisPort - mRemise;
			
			totalTVA = totalTVA + fraisPort * tauxTvaPort/100;
			
			montantTTC = totalBase + totalTVA;
			mEscompte = r.round(montantTTC * tEscompte/100);
			netAPayer = montantTTC - mEscompte - acompte;

			rset.close();
			stt.close();
			stt2.close();
			stt3.close();
			s.closeConnection(con, null);

		} catch(Exception e) { e.printStackTrace();	}

	}
	
	public double calcSousTotal(int derniereLigne) {		
		if (derniereLigne<0) 
			return 0;
		return lignes.get(derniereLigne).sousTotal;		
	}	
	

	public int getNbLignes() {
		return lignes.size();
	}
	

	public String formatSiren(String siret) {
		if (siret.length()>=9) {
			return siret.substring(0,3) +" "+ siret.substring(3,6) +" "+ siret.substring(6,9);
		}
		else {
			return siret;
		}
	}


} // fin DataFactureFournisseur
