/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Gnration d'un devis au format standard
 */

package org.opensi.facturation.actions.documents.modeles;


import java.awt.Color;
import java.io.IOException;
import java.sql.Date;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;

import org.opensi.util.pdf.MultiPagesDocument;
import org.opensi.util.pdf.PdfWidths;
import org.opensi.util.pdf.TableBuilder;

import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;

public class DevisStd extends MultiPagesDocument {


private static final Font fontAdrFacture = new Font(Font.HELVETICA, 11, Font.BOLD);
private static final Font fontAdrSociete = new Font(Font.HELVETICA, 10, Font.NORMAL);
private static final Font fontAdrSocieteGras = new Font(Font.HELVETICA, 11, Font.BOLD);
private static final Font fontAdrLiv = new Font(Font.HELVETICA, 8, Font.NORMAL);
private static final Font fontAdrContact = new Font(Font.HELVETICA, 7, Font.NORMAL);
private static final Font fontTitre = new Font(Font.HELVETICA , 16, Font.BOLD, Color.WHITE);
private static final Font fontWeb = new Font(Font.HELVETICA, 10, Font.ITALIC);
private static final Font fontCadre1 = new Font(Font.HELVETICA, 10, Font.BOLD);
private static final Font fontEnteteColonne = new Font(Font.HELVETICA, 9, Font.BOLD);
private static final Font fontInfosFact = new Font(Font.HELVETICA , 10, Font.NORMAL);
private static final Font fontLegale = new Font(Font.HELVETICA , 7, Font.NORMAL);
private static final Font fontLine = new Font(Font.HELVETICA , 8, Font.NORMAL);
private static final Font fontModalites = new Font(Font.HELVETICA , 10, Font.BOLD);
private static final Font fontTotaux = new Font(Font.HELVETICA , 10, Font.NORMAL);
private static final Font fontTTC = new Font(Font.HELVETICA , 10, Font.BOLD);
private static final Font fontArtNom = new Font(Font.HELVETICA, 9, Font.ITALIC);
private static final Font fontSmallTitre = new Font(Font.HELVETICA , 8, Font.BOLD+Font.ITALIC, Color.WHITE);


private DecimalFormat numFormat = new DecimalFormat("0.00");
private DecimalFormat numFormat4 = new DecimalFormat("0.00##");
private DecimalFormat numFormatCap = new DecimalFormat("0.##");
private SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");


private DataDevis id;	   // les donnes dynamiques du devis

private float[] widthsCols;
private int nbCols;

private boolean affTotaux = true;

private boolean rist;
private boolean coul;
private boolean tail;
private boolean unite;
private boolean nbPieces;
private boolean numLot;
private boolean datePeremption;

private double sousTotalCourant = 0;

private Image logo;
private boolean marq = false;


  public DevisStd(String tempdir, DataDevis id) throws Exception {

		super(tempdir);
		
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		dfs.setGroupingSeparator(' ');
		
		numFormat.setDecimalFormatSymbols(dfs);
		numFormat.setNegativeSuffix("");
		numFormat.setNegativePrefix("-");
		numFormat.setPositiveSuffix(" ");
		numFormat.setGroupingUsed(true);
		numFormat.setGroupingSize(3);		
		
		numFormat4.setDecimalFormatSymbols(dfs);
		numFormat4.setGroupingUsed(true);
		numFormat4.setGroupingSize(3);
		
		numFormatCap.setDecimalFormatSymbols(dfs);
		numFormatCap.setGroupingUsed(true);
		numFormatCap.setGroupingSize(3);
		
		
		this.id = id;
		
		try {
			logo = Image.getInstance(id.urlLogo);
			if (logo.getDpiX()!=0 && logo.getDpiY()!=0) {
				logo.scaleAbsolute(logo.plainWidth() * 72 / logo.getDpiX(), logo.plainHeight() * 72 / logo.getDpiY());
			}
		}
		catch (IOException e) {
			logo = null;
		}
		

		rist = id.rist;
		tail = id.tail;
		coul = id.coul;
		unite = id.unite;
		nbPieces = id.nbPieces;
		numLot = id.numLot;
		datePeremption = id.datePeremption;
		
		calculColonnes();	
	
	}
  
	private void calculColonnes() {
		PdfWidths pw = new PdfWidths();
		
		pw.addColonneVariableMinMax(2, 12, 20);
		pw.addColonneVariableMin(4,20);
		if (marq) pw.addColonneVariableMin(1, 6);
		if (coul) pw.addColonneVariableMin(1, 6);
		if (tail) pw.addColonneVariableMin(1, 6);
		if (numLot) pw.addColonneVariableMinMax(2, 6, 10);
		if (nbPieces) pw.addColonneFixe(5);
		if (datePeremption) pw.addColonneFixe(8);
		pw.addColonneVariableMin(1, 8);
		if (unite) pw.addColonneFixe(5);
		pw.addColonneVariableMin(1, 6);
		if (rist) pw.addColonneFixe(6);
		pw.addColonneVariableMin(1, 9);
		
		nbCols = pw.getNbCols();
		widthsCols = pw.getWidthsCols();
	}



	
	/* implmentation des fonctions de MultiPagesDocument */
	
	public PdfPTable firstHeader() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(makeEntete());
		tb.add(makeSpace(5));
		tb.add(makeTitre());
		tb.add(makeSpace(5));
		tb.add(makeResponsable());
		tb.add(makeSpace(5));
		tb.add(makeEntetesColonnes());		
		
		return tb.getTable();	
	}
	
	
	public PdfPTable middleHeader(int lastLine) throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(firstHeader());
		
		if (affTotaux) {
			tb.add(makeSousTotal(id.calcSousTotal(lastLine)));
		}
		
		return tb.getTable();
	}
	
	
	public PdfPTable lastFooter() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		if (id.comFin.length()>0 || id.mentions.length()>0) {
			tb.add(makeZoneCommentaire());
			tb.add(emptyLine(10));
		}
		
		tb.add(makeTotauxLast());		
		tb.add(makeSpace(3));
		tb.add(makeInfosLegales());
		
		return tb.getTable();
	}
	
	
	public PdfPTable middleFooter(int nbLinesPage, int lastLine) throws DocumentException {
	
		sousTotalCourant = id.calcSousTotal(lastLine);
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(emptyLine(10));
		tb.add(makeTotauxMiddle());
		tb.add(makeSpace(3));
		tb.add(makeInfosLegales());		
		
		return tb.getTable();		
	}
	
	
	public PdfPTable bodyLine(int numLine) throws DocumentException {
	
		LigneDevis ld = id.lignes.get(numLine);
	
		switch (ld.type) {			
			case 'A':	return makeLineArticle(ld);
			case 'N': return makeLineNomenclature(ld);
			case 'C':
			default: return makeLineCommentaire(ld);
		}		
	}
	
	
	public PdfPTable emptyLine(float emptyHeight) throws DocumentException {
		
		PdfPTable pptSpace = new PdfPTable(nbCols);
		pptSpace.setWidthPercentage(100);
		pptSpace.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.LEFT+PdfPCell.RIGHT);
		cell.setPaddingBottom(0);
		cell.setPaddingTop(0);
		cell.setFixedHeight(emptyHeight);		
		cell.setPhrase(new Phrase(""));		
	
		for (int i=0; i<nbCols; i++) {
			pptSpace.addCell(cell);
		}
	
		return pptSpace;
	}
	
	
	public int nbLines() {
	
		return id.getNbLignes();
	}
	
	
	public int nbEx() {
	
		return 1;
	}
	
	
	public Image imageFond() {
		return null;
	}



	/* mthodes internes */

	private PdfPTable makeSpace(int space) {
	
		PdfPTable pptSpace = new PdfPTable(1);
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setFixedHeight(space);
		cell.setPhrase(new Phrase(""));
	
		pptSpace.addCell(cell);
	
		return pptSpace;
	}
	

	private PdfPTable makeEntete() throws DocumentException {
	
		PdfPTable pptEntete = new PdfPTable(2);
		pptEntete.setWidthPercentage(100);
		int[] widths = {50,50};
		pptEntete.setWidths(widths);
		
		PdfPTable pptEnteteGauche = new PdfPTable(1);
		pptEnteteGauche.setWidthPercentage(100);
		PdfPTable pptEnteteDroite = new PdfPTable(1);
		pptEnteteDroite.setWidthPercentage(100);
		
		PdfPCell cellSociete = new PdfPCell(makeInfosSociete());
		cellSociete.setMinimumHeight(100);
		cellSociete.setBorder(PdfPCell.NO_BORDER);
		pptEnteteGauche.addCell(cellSociete);		
		PdfPCell cellAdrLiv;
		if (id.imprimerAdrLiv) {
			cellAdrLiv = new PdfPCell(makeAdrLiv());
		} else {
			cellAdrLiv = new PdfPCell(pptEnteteGauche.getDefaultCell());
			cellAdrLiv.setPhrase(new Phrase(""));
		}
		cellAdrLiv.setBorder(PdfPCell.NO_BORDER);
		pptEnteteGauche.addCell(cellAdrLiv);
		PdfPCell cellVide = new PdfPCell(pptEnteteGauche.getDefaultCell());
		cellVide.setBorder(PdfPCell.NO_BORDER);
		cellVide.setPhrase(new Phrase(""));
		pptEnteteGauche.addCell(cellVide);
		
		PdfPCell cellFacture = new PdfPCell(makeInfosFacture());
		cellFacture.setFixedHeight(100);
		cellFacture.setBorder(PdfPCell.NO_BORDER);
		pptEnteteDroite.addCell(cellFacture);		
		PdfPCell cellAdrFact = new PdfPCell(makeAdrFact());
		cellAdrFact.setMinimumHeight(80);
		cellAdrFact.setBorder(PdfPCell.NO_BORDER);
		pptEnteteDroite.addCell(cellAdrFact);
		pptEnteteDroite.addCell(cellVide);
		
		PdfPCell cellPartieGauche = new PdfPCell(pptEnteteGauche);
		cellPartieGauche.setBorder(PdfPCell.NO_BORDER);
		PdfPCell cellPartieDroite = new PdfPCell(pptEnteteDroite);		
		cellPartieDroite.setBorder(PdfPCell.NO_BORDER);
		pptEntete.addCell(cellPartieGauche);
		pptEntete.addCell(cellPartieDroite);
		
		return pptEntete;		
	}


	private PdfPTable makeInfosSociete() {
	
		PdfPTable pptInfos = new PdfPTable(1);
		
		PdfPCell cell;
				
		if (logo != null) {
			cell = new PdfPCell(logo, false);

			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);

			pptInfos.addCell(cell);

			cell.setImage(null);

			cell.setMinimumHeight(5);
			cell.setPhrase(new Phrase("", fontAdrSociete));
			pptInfos.addCell(cell);
			cell.setMinimumHeight(0);
		}
		else {
			cell = new PdfPCell(pptInfos.getDefaultCell());
			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		}	
		
		if (!id.logoAdr) {
			cell.setPhrase(new Phrase(id.raisonSociale, fontAdrSocieteGras));		
			pptInfos.addCell(cell);
			cell.setPhrase(new Phrase(id.adresse, fontAdrSociete));		
			pptInfos.addCell(cell);
			if (!id.compAdresse.equals("")) {
				cell.setPhrase(new Phrase(id.compAdresse, fontAdrSociete));
				pptInfos.addCell(cell);
			}		
			cell.setPhrase(new Phrase(id.codePostal+" "+id.ville, fontAdrSociete));		
			pptInfos.addCell(cell);

			String telfax = "";

			if (id.tel.length()>0) {
				telfax = "T\u00E9l: "+ id.tel;
				if (id.fax.length()>0) {
					telfax += " - Fax: "+ id.fax;
				}
			}
			else if (id.fax.length()>0) {
				telfax = "Fax: "+ id.fax;
			}		

			if (telfax.length()>0) {
				cell.setPhrase(new Phrase(telfax, fontAdrSociete));		
				pptInfos.addCell(cell);
			}

			if (!id.email.equals("")) {
				cell.setPhrase(new Phrase("e-mail: "+ id.email, fontAdrSociete));		
				pptInfos.addCell(cell);
			}
			if (!id.web.equals("")) {
				cell.setPhrase(new Phrase(id.web, fontWeb));		
				pptInfos.addCell(cell);
			}
		}
		
		cell.setPhrase(new Phrase(""));		
		pptInfos.addCell(cell);
		
		return pptInfos;	
	}
	
	
	private PdfPTable makeAdrLiv() throws DocumentException {

		PdfPTable pptInfosFacture = new PdfPTable(2);
		int[] widthsFact = {50,50};
		pptInfosFacture.setWidths(widthsFact);

		PdfPCell cell = new PdfPCell(pptInfosFacture.getDefaultCell());

		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setColspan(2);

		cell.setBackgroundColor(new Color(175,175,175));

		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setPhrase(new Phrase("LIVRAISON", fontSmallTitre));
		pptInfosFacture.addCell(cell);

		cell.setBackgroundColor(Color.WHITE);

		cell.setPaddingBottom(0);
		cell.setPaddingLeft(5);

		cell.setBorder(PdfPCell.BOX-PdfPCell.BOTTOM);

		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cell.setPhrase(new Phrase(id.denominationLiv, fontAdrLiv));
		pptInfosFacture.addCell(cell);

		cell.setBorder(PdfPCell.LEFT+PdfPCell.RIGHT);

		cell.setPhrase(new Phrase(id.adresse1Liv, fontAdrLiv));
		pptInfosFacture.addCell(cell);

		if (!id.adresse2Liv.equals("")) {
			cell.setPhrase(new Phrase(id.adresse2Liv, fontAdrLiv));
			pptInfosFacture.addCell(cell);
		}

		if (!id.adresse3Liv.equals("")) {
			cell.setPhrase(new Phrase(id.adresse3Liv, fontAdrLiv));
			pptInfosFacture.addCell(cell);
		}

		cell.setPhrase(new Phrase(id.codePostalLiv+" "+id.villeLiv, fontAdrLiv));
		pptInfosFacture.addCell(cell);

		if (!id.paysLiv.equals("")) {
			cell.setPhrase(new Phrase(id.paysLiv, fontAdrLiv));
			pptInfosFacture.addCell(cell);
		}

		if (!id.interlocuteurLiv.equals("")) {
			cell.setPhrase(new Phrase("Contact : "+id.interlocuteurLiv, fontAdrContact));
			pptInfosFacture.addCell(cell);
		}
		if (!id.interMailLiv.equals("")) {
			cell.setPhrase(new Phrase("E-Mail : "+id.interMailLiv, fontAdrContact));
			pptInfosFacture.addCell(cell);
		}

		if (!id.interTelLiv.equals("") && !id.interFaxLiv.equals("")) {
			cell.setColspan(1);
			cell.setBorder(PdfPCell.LEFT);
			cell.setPhrase(new Phrase("T\u00E9l : "+id.interTelLiv, fontAdrContact));
			pptInfosFacture.addCell(cell);
			cell.setBorder(PdfPCell.RIGHT);
			cell.setPhrase(new Phrase("Fax : "+id.interFaxLiv, fontAdrContact));
			pptInfosFacture.addCell(cell);
		}
		else if (!id.interTelLiv.equals("")) {
			cell.setPhrase(new Phrase("T\u00E9l : "+id.interTelLiv, fontAdrContact));
			pptInfosFacture.addCell(cell);
		}
		else if (!id.interFaxLiv.equals("")) {
			cell.setPhrase(new Phrase("Fax : "+id.interFaxLiv, fontAdrContact));
			pptInfosFacture.addCell(cell);
		}
		
		cell.setBorder(PdfPCell.BOX-PdfPCell.TOP);
		cell.setMinimumHeight(5);
		cell.setPhrase(new Phrase("", fontAdrLiv));
		cell.setColspan(2);
		pptInfosFacture.addCell(cell);

		PdfPTable pptGlobale = new PdfPTable(2);
		int[] widths = {60,40};
		pptGlobale.setWidths(widths);

		PdfPCell cellGlob = new PdfPCell(pptInfosFacture);
		cellGlob.setBorder(PdfPCell.NO_BORDER);

		pptGlobale.addCell(cellGlob);

		cellGlob = new PdfPCell(pptGlobale.getDefaultCell());
		cellGlob.setPhrase(new Phrase(""));
		cellGlob.setBorder(PdfPCell.NO_BORDER);

		pptGlobale.addCell(cellGlob);

		return pptGlobale;
	}
	
	
	private PdfPTable makeAdrFact() throws DocumentException {

		PdfPTable pptInfosFacture = new PdfPTable(2);

		PdfPCell cell = new PdfPCell(pptInfosFacture.getDefaultCell());

		cell.setColspan(2);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);

		cell.setMinimumHeight(0);
		cell.setPaddingLeft(0);
		cell.setPaddingBottom(0);
		
		cell.setBorder(PdfPCell.NO_BORDER);
		
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cell.setPhrase(new Phrase(id.raisonSocialeDest, fontAdrFacture));
		pptInfosFacture.addCell(cell);
		
		cell.setPhrase(new Phrase(id.adresseDest, fontAdrFacture));
		pptInfosFacture.addCell(cell);
		
		if (!id.compAdresseDest.equals("")) {
			cell.setPhrase(new Phrase(id.compAdresseDest, fontAdrFacture));
			pptInfosFacture.addCell(cell);
		}
		
		if (!id.adresseDest3.equals("")) {
			cell.setPhrase(new Phrase(id.adresseDest3, fontAdrFacture));
			pptInfosFacture.addCell(cell);
		}
		
		cell.setPhrase(new Phrase(id.codePostalDest+" "+id.villeDest, fontAdrFacture));
		pptInfosFacture.addCell(cell);
		
		if (!id.paysDest.equals("")) {
			cell.setPhrase(new Phrase(id.paysDest, fontAdrFacture));
			pptInfosFacture.addCell(cell);
		}
		
		cell.setPhrase(new Phrase("", fontLine));
		pptInfosFacture.addCell(cell);

		PdfPTable pptGlobale = new PdfPTable(2);
		int[] widths = {85,15};
		pptGlobale.setWidths(widths);

		PdfPCell cellGlob = new PdfPCell(pptInfosFacture);
		cellGlob.setBorder(PdfPCell.NO_BORDER);

		pptGlobale.addCell(cellGlob);

		cellGlob = new PdfPCell(pptGlobale.getDefaultCell());
		cellGlob.setPhrase(new Phrase(""));
		cellGlob.setBorder(PdfPCell.NO_BORDER);

		pptGlobale.addCell(cellGlob);

		return pptGlobale;
	}


	private PdfPTable makeInfosFacture() throws DocumentException {
		
		PdfPTable pptInfosFacture = new PdfPTable(2);
		
		int[] widths = {60,40};
		pptInfosFacture.setWidths(widths);
		
		PdfPCell cell = new PdfPCell(pptInfosFacture.getDefaultCell());
		
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setPaddingLeft(4);
		cell.setPaddingBottom(4);
		
		cell.setPhrase(new Phrase("Devis n "+ id.numeroDevis, fontInfosFact));
		pptInfosFacture.addCell(cell);
		
		cell.setPhrase(new Phrase("Date : "+ formatDate.format(new Date(id.dateDevis)), fontInfosFact));
		pptInfosFacture.addCell(cell);
		
		cell.setColspan(2);
		if (id.clientId.length()>0) {
			cell.setPhrase(new Phrase("Client : "+ id.clientId +" - "+ id.raisonSocialeDest, fontInfosFact));
		}
		else {
			cell.setPhrase(new Phrase("Client : "+ id.raisonSocialeDest, fontInfosFact));
		}
		pptInfosFacture.addCell(cell);
		
		cell.setBorder(PdfPCell.NO_BORDER);
		
		cell.setPhrase(new Phrase("", fontLine));
		cell.setFixedHeight(5);
		pptInfosFacture.addCell(cell);
		cell.setMinimumHeight(0);
		
		cell.setPhrase(new Phrase("Page "+currentPage+" / "+totalPages, fontInfosFact));
		cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		pptInfosFacture.addCell(cell);
		
		cell.setPhrase(new Phrase("", fontLine));
		cell.setFixedHeight(55);
		pptInfosFacture.addCell(cell);
		
		return pptInfosFacture;		
	}


	private PdfPTable makeTitre() {
	
		PdfPTable pptTitre = new PdfPTable(1);
		pptTitre.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptTitre.getDefaultCell());		
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setBackgroundColor(new Color(175,175,175));
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setFixedHeight(30);
		cell.setPaddingBottom(5);
		cell.setPhrase(new Phrase("D     E     V     I     S", fontTitre));
		
		pptTitre.addCell(cell);
		
		return pptTitre;	
	}


	private PdfPTable makeResponsable() {
	
		boolean inter = !id.interlocuteur.equals("");

		PdfPTable pptResponsable = new PdfPTable(inter?3:1);
		pptResponsable.setWidthPercentage(100);

		PdfPCell cell = new PdfPCell(pptResponsable.getDefaultCell());

		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setBorder(PdfPCell.BOX-PdfPCell.BOTTOM);
		cell.setFixedHeight(20);
		cell.setPaddingLeft(5);
		cell.setPhrase(new Phrase(id.responsable.equals("")?"":"Responsable : "+ id.responsable, fontCadre1));
		pptResponsable.addCell(cell);

		if (inter) {
			cell.setBorder(PdfPCell.TOP + PdfPCell.LEFT);
			cell.setPhrase(new Phrase("Contact : "+ id.interlocuteur, fontLegale));
			pptResponsable.addCell(cell);

			cell.setBorder(PdfPCell.TOP + PdfPCell.RIGHT);
			cell.setPhrase(new Phrase("T\u00E9l : "+ id.interTel, fontLegale));
			pptResponsable.addCell(cell);
		}
		
		cell.setMinimumHeight(0);
		cell.setBorder(PdfPCell.BOX-PdfPCell.TOP);
		cell.setPhrase(new Phrase("", fontLegale));
		pptResponsable.addCell(cell);
			
		if (inter) {
			cell.setBorder(PdfPCell.BOTTOM + PdfPCell.LEFT);
			cell.setPhrase(new Phrase("E-Mail : "+ id.interMail, fontLegale));
			pptResponsable.addCell(cell);

			cell.setBorder(PdfPCell.BOTTOM + PdfPCell.RIGHT);
			cell.setPhrase(new Phrase("Fax : "+ id.interFax, fontLegale));
			pptResponsable.addCell(cell);
		}
		
		return pptResponsable;
	}


	private PdfPTable makeEntetesColonnes() throws DocumentException {
	
		PdfPTable pptCols = new PdfPTable(nbCols);
		pptCols.setWidthPercentage(100);
		pptCols.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptCols.getDefaultCell());		
		
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setPaddingBottom(4);
		cell.setMinimumHeight(40);
		
		cell.setPhrase(new Phrase("R\u00E9f\u00E9rence", fontEnteteColonne));
		pptCols.addCell(cell);
		
		cell.setPhrase(new Phrase("D\u00E9signation", fontEnteteColonne));
		pptCols.addCell(cell);
		
		if (marq) {
			cell.setPhrase(new Phrase("Marque.", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		
		if (coul) {
			cell.setPhrase(new Phrase("Coul.", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		
		if (tail) {
			cell.setPhrase(new Phrase("Taille", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		
		if (numLot) {
			cell.setPhrase(new Phrase("N Lot", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		if (nbPieces) {
			cell.setPhrase(new Phrase("Nb Pc", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		if (datePeremption) {
			cell.setPhrase(new Phrase("DLC", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		
		
		cell.setPhrase(new Phrase("Quantit\u00E9", fontEnteteColonne));
		pptCols.addCell(cell);
		
		if (unite) {
			cell.setPhrase(new Phrase("Unit\u00E9", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		
		cell.setPhrase(new Phrase((id.editionTTC?"PU TTC":"PU HT"), fontEnteteColonne));
		pptCols.addCell(cell);
		if (rist) {
			cell.setPhrase(new Phrase("Rist. %", fontEnteteColonne));
			pptCols.addCell(cell);
		}
		cell.setPhrase(new Phrase((id.editionTTC?"Montant TTC":"Montant HT"), fontEnteteColonne));		
		pptCols.addCell(cell);
		
		return pptCols;
	}
	
	
	private PdfPTable makeLineArticle(LigneDevis ld) throws DocumentException {
	
		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);

		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());

		cell.setBorder(PdfPCell.RIGHT+PdfPCell.LEFT);

		cell.setPaddingLeft(5);
		cell.setPhrase(new Phrase(ld.reference, fontLine));
		pptLine.addCell(cell);		
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cell.setPhrase(new Phrase(ld.designation, fontLine));
		pptLine.addCell(cell);

		if (marq) {
			cell.setPhrase(new Phrase(ld.marque, fontLine));
			pptLine.addCell(cell);
		}
		if (coul) {
			cell.setPhrase(new Phrase(ld.couleur, fontLine));
			pptLine.addCell(cell);
		}
		if (tail) {
			cell.setPhrase(new Phrase(ld.taille, fontLine));
			pptLine.addCell(cell);
		}
		
		cell.setPaddingLeft(0);
		cell.setPaddingRight(0);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		
		if (numLot) {
			cell.setPhrase(new Phrase((ld.num_lot), fontLine));
			pptLine.addCell(cell);
		}
		if (nbPieces) {
			if (ld.nb_pieces != 0)
				cell.setPhrase(new Phrase(""+numFormatCap.format(ld.nb_pieces), fontLine));
			else
				cell.setPhrase(new Phrase("", fontLine));			
			pptLine.addCell(cell);
		}
		if (datePeremption) {
			if (ld.date_peremption != 0)
				cell.setPhrase(new Phrase(""+ formatDate.format(new Date(ld.date_peremption)), fontLine));
			else
				cell.setPhrase(new Phrase("", fontLine));
			pptLine.addCell(cell);
		}
		
		cell.setPhrase(new Phrase(""+numFormatCap.format(ld.quantite), fontLine));
		pptLine.addCell(cell);
		if (unite) {
			cell.setPhrase(new Phrase(ld.unite, fontLine));
			pptLine.addCell(cell);
		}

		cell.setPaddingRight(3);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		cell.setPhrase(new Phrase(numFormat4.format(ld.prixUnitaire), fontLine));
		pptLine.addCell(cell);
		if (rist) {
			if (ld.ristourne!=0) {
				cell.setPhrase(new Phrase(numFormat.format(ld.ristourne), fontLine));
			}
			else {
				cell.setPhrase(new Phrase("", fontLine));
			}
			pptLine.addCell(cell);
		}

		cell.setPhrase(new Phrase(numFormat.format(ld.montant), fontLine));		
		pptLine.addCell(cell);

		if (ld.detail_1.length()>0) {
			cell.setPaddingLeft(5);
			cell.setPaddingRight(0);
			cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
			cell.setPhrase(new Phrase("", fontLine));
			pptLine.addCell(cell);
			cell.setPhrase(new Phrase(ld.detail_1, fontLine));
			pptLine.addCell(cell);
			cell.setPhrase(new Phrase("", fontLine));
			pptLine.addCell(cell);
			pptLine.addCell(cell);
			pptLine.addCell(cell);
			if (rist) pptLine.addCell(cell);
			if (marq) pptLine.addCell(cell);		
			if (coul) pptLine.addCell(cell);		
			if (tail) pptLine.addCell(cell);
			if (unite) pptLine.addCell(cell);
			if (numLot) pptLine.addCell(cell);
			if (nbPieces) pptLine.addCell(cell);
			if (datePeremption) pptLine.addCell(cell);
		}

		if (ld.detail_2.length()>0) {
			cell.setPaddingLeft(5);
			cell.setPaddingRight(0);
			cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
			cell.setPhrase(new Phrase("", fontLine));
			pptLine.addCell(cell);
			cell.setPhrase(new Phrase(ld.detail_2, fontLine));
			pptLine.addCell(cell);
			cell.setPhrase(new Phrase("", fontLine));
			pptLine.addCell(cell);
			pptLine.addCell(cell);
			pptLine.addCell(cell);
			if (rist) pptLine.addCell(cell);
			if (marq) pptLine.addCell(cell);		
			if (coul) pptLine.addCell(cell);		
			if (tail) pptLine.addCell(cell);
			if (unite) pptLine.addCell(cell);
			if (numLot) pptLine.addCell(cell);
			if (nbPieces) pptLine.addCell(cell);
			if (datePeremption) pptLine.addCell(cell);
		}

		return pptLine;	
	}
	
	
	private PdfPTable makeLineNomenclature(LigneDevis ld) throws DocumentException {
	
		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());		
		
		cell.setBorder(PdfPCell.RIGHT+PdfPCell.LEFT);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		
		cell.setPhrase(new Phrase("", fontLine));
		pptLine.addCell(cell);
		
		
		PdfPTable pptArtNom = new PdfPTable(2);
		pptArtNom.setWidthPercentage(100);
		int[] widths = {10,90};
		pptArtNom.setWidths(widths);
		
		PdfPCell cellArtNom = new PdfPCell(pptArtNom.getDefaultCell());
		cellArtNom.setBorder(PdfPCell.NO_BORDER);
		
		cellArtNom.setPhrase(new Phrase(numFormatCap.format(ld.quantite), fontArtNom));
		pptArtNom.addCell(cellArtNom);
		
		if (unite) {
			cellArtNom.setPhrase(new Phrase(ld.unite, fontLine));
			pptLine.addCell(cellArtNom);
		}
		
		cellArtNom.setPhrase(new Phrase(ld.designation, fontArtNom));
		pptArtNom.addCell(cellArtNom);
		
		PdfPCell cellDesc = new PdfPCell(pptArtNom);
		cellDesc.setBorder(PdfPCell.NO_BORDER);
		cellDesc.setPaddingLeft(10);
		pptLine.addCell(cellDesc);
		
		cell.setPhrase(new Phrase("", fontLine));
		pptLine.addCell(cell);
		pptLine.addCell(cell);
		pptLine.addCell(cell);
		
		if (rist) pptLine.addCell(cell);		
		if (marq) pptLine.addCell(cell);		
		if (coul) pptLine.addCell(cell);		
		if (tail) pptLine.addCell(cell);
		
		if (numLot) pptLine.addCell(cell);
		if (nbPieces) pptLine.addCell(cell);
		if (datePeremption) pptLine.addCell(cell);
		
		return pptLine;	
	}
	
	
	private PdfPTable makeLineCommentaire(LigneDevis ld) throws DocumentException {
	
		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());		
		
		cell.setBorder(PdfPCell.RIGHT+PdfPCell.LEFT);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);		
		
		cell.setPaddingLeft(5);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cell.setPhrase(new Phrase("", fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(ld.designation, fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase("", fontLine));
		pptLine.addCell(cell);
		pptLine.addCell(cell);
		pptLine.addCell(cell);
		
		if (rist) pptLine.addCell(cell);
		if (marq) pptLine.addCell(cell);		
		if (coul) pptLine.addCell(cell);		
		if (tail) pptLine.addCell(cell);
		if (unite) pptLine.addCell(cell);
		
		if (numLot) pptLine.addCell(cell);
		if (nbPieces) pptLine.addCell(cell);
		if (datePeremption) pptLine.addCell(cell);
		
		return pptLine;	
	}
	
	
	private PdfPTable makeSousTotal(double repriseSousTotal) throws DocumentException {
	
		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());		
		
		cell.setBorder(PdfPCell.RIGHT+PdfPCell.LEFT+PdfPCell.BOTTOM);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setPaddingBottom(6);
		cell.setPaddingTop(3);
		
		cell.setPaddingLeft(5);
		cell.setPhrase(new Phrase("", fontLine));
		pptLine.addCell(cell);
		
		cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		cell.setPaddingRight(3);
		cell.setPhrase(new Phrase((id.editionTTC?"Sous-total TTC":"Sous-total HT"), fontTotaux));
		pptLine.addCell(cell);
		cell.setPaddingLeft(0);	
		
		cell.setPhrase(new Phrase("", fontLine));
		pptLine.addCell(cell);	
		
		cell.setPhrase(new Phrase("", fontTotaux));
		pptLine.addCell(cell);
		if (rist) pptLine.addCell(cell);
		if (marq) pptLine.addCell(cell);
		if (coul) pptLine.addCell(cell);
		if (tail) pptLine.addCell(cell);
		if (unite) pptLine.addCell(cell);		
		if (numLot) pptLine.addCell(cell);
		if (nbPieces) pptLine.addCell(cell);
		if (datePeremption) pptLine.addCell(cell);
		
		cell.setPhrase(new Phrase(numFormat.format(repriseSousTotal), fontTotaux));		
		pptLine.addCell(cell);
		
		return pptLine;	
	}
	
	
	private PdfPTable makeTotauxLast() throws DocumentException {
		
		PdfPTable pptTotaux = new PdfPTable(3);
		pptTotaux.setWidthPercentage(100);
		int[] widths = {50,15,35};
		pptTotaux.setWidths(widths);
		
		if (!affTotaux) {
		
			PdfPCell cell = new PdfPCell(makeCadreDivers());
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.LEFT);	
			cell.setColspan(2);
			cell.setPadding(5);
			pptTotaux.addCell(cell);
			
			cell = new PdfPCell(pptTotaux.getDefaultCell());
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.RIGHT);
			cell.setPhrase(new Phrase(""));	
			pptTotaux.addCell(cell);
		}
		else {
			
			PdfPCell cell = new PdfPCell(makeCadreTvaDivers());	
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.LEFT);		
			pptTotaux.addCell(cell);

			cell = new PdfPCell(pptTotaux.getDefaultCell());	
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP);		
			cell.setPhrase(new Phrase(""));
			pptTotaux.addCell(cell);


			cell = new PdfPCell(makeCadreTotaux());
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.RIGHT);		
			pptTotaux.addCell(cell);		
		}
				
		return pptTotaux;	
	}	
	
	
	private PdfPTable makeTotauxMiddle() throws DocumentException {
		
		PdfPTable pptTotaux = new PdfPTable(3);
		pptTotaux.setWidthPercentage(100);
		int[] widths = {50,15,35};
		pptTotaux.setWidths(widths);
		
		if (!affTotaux) {
		
			PdfPCell cell = new PdfPCell(makeCadreDivers());
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.LEFT);	
			cell.setColspan(2);
			cell.setPadding(5);
			pptTotaux.addCell(cell);
			
			cell = new PdfPCell(pptTotaux.getDefaultCell());
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.RIGHT);
			cell.setPhrase(new Phrase(""));	
			pptTotaux.addCell(cell);
		}
		else {
			
			PdfPCell cell = new PdfPCell(makeCadreDivers());	
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.LEFT);
			cell.setPaddingBottom(3);	
			pptTotaux.addCell(cell);

			cell = new PdfPCell(pptTotaux.getDefaultCell());	
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP);		
			cell.setPhrase(new Phrase(""));
			pptTotaux.addCell(cell);

			cell = new PdfPCell(makeCadreSousTotal());
			cell.setVerticalAlignment(PdfPCell.ALIGN_TOP);	
			cell.setBorder(PdfPCell.BOTTOM+PdfPCell.TOP+PdfPCell.RIGHT);		
			pptTotaux.addCell(cell);			
		}
				
		return pptTotaux;	
	}	
	
	
	private PdfPTable makeCadreTVA() {
	
		PdfPTable pptTva = new PdfPTable(3);
		pptTva.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptTva.getDefaultCell());
	
		cell.setBorder(PdfPCell.BOX);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setPhrase(new Phrase("Base HT", fontEnteteColonne));
		pptTva.addCell(cell);
		cell.setPhrase(new Phrase("Taux TVA", fontEnteteColonne));
		pptTva.addCell(cell);
		cell.setPhrase(new Phrase("Montant TVA", fontEnteteColonne));	
		pptTva.addCell(cell);
		
		for (int i=0; i<id.vtvas.size(); i++) {
		
			VentilTVA vtva = id.vtvas.get(i);
		
			cell.setPaddingRight(3);
			cell.setBorder(PdfPCell.BOX-PdfPCell.TOP-PdfPCell.BOTTOM);
			cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
			cell.setPhrase(new Phrase(numFormat.format(vtva.baseHT), fontTotaux));		
			pptTva.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(vtva.tauxTVA) +" %"+ (vtva.npr?" NPR":""), fontTotaux));
			pptTva.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(vtva.montantTVA), fontTotaux));
			pptTva.addCell(cell);
		}		
		
		cell.setBorder(PdfPCell.TOP);
		cell.setColspan(3);
		cell.setPhrase(new Phrase(""));
		cell.setFixedHeight(0);
		pptTva.addCell(cell);
		
		return pptTva;
	}
	
	
	private PdfPTable makeCadreDivers() throws DocumentException {
		
		PdfPTable pptDivers = new PdfPTable(2);
		pptDivers.setWidthPercentage(100);
		int[] widths = {15,85};
		pptDivers.setWidths(widths);
		
		PdfPCell cell = new PdfPCell(pptDivers.getDefaultCell());
		
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setColspan(2);
		
		if (id.dateExp!=0) {
			cell.setPhrase(new Phrase("Date d'expiration de l'offre : "+ formatDate.format(new Date(id.dateExp)), fontModalites));
			pptDivers.addCell(cell);
		}
		
		if (!id.numTvaClient.equals("")) {
			cell.setPhrase(new Phrase("", fontModalites));
			pptDivers.addCell(cell);
			cell.setPhrase(new Phrase("N TVA Client : "+ id.numTvaClient, fontModalites));
			pptDivers.addCell(cell);
		}
		
		cell.setMinimumHeight(10);
		cell.setPhrase(new Phrase("", fontModalites));
		pptDivers.addCell(cell);
		
		cell.setMinimumHeight(90);
		cell.setColspan(1);
		cell.setPhrase(new Phrase(""));
		pptDivers.addCell(cell);
		
		cell.setBorder(PdfPCell.BOX);
		cell.setVerticalAlignment(PdfPCell.ALIGN_TOP);
		cell.setPhrase(new Phrase("Bon pour accord,\nnom :                            fonction :\ndate :\nsignature et cachet :", fontModalites));
		pptDivers.addCell(cell);
		
		return pptDivers;
	}
	
	
	private PdfPTable makeCadreTvaDivers() throws DocumentException {
		PdfPTable pptTvaDivers = new PdfPTable(1);
		pptTvaDivers.setWidthPercentage(100);
		//int[] widths = {55,15};
		//pptTotaux.setWidths(widths);
		PdfPCell cell = new PdfPCell(makeCadreTVA());	
		cell.setBorder(Rectangle.NO_BORDER);
		cell.setPaddingLeft(3);
		cell.setPaddingTop(3);
		pptTvaDivers.addCell(cell);		
		
		cell = new PdfPCell(makeCadreDivers());
		cell.setPaddingLeft(3);
		cell.setPaddingBottom(3);	
		cell.setBorder(Rectangle.NO_BORDER);		
		pptTvaDivers.addCell(cell);
		
		return pptTvaDivers;
	}
	
	private PdfPTable makeCadreSousTotal() throws DocumentException {
	
		PdfPTable pptCadreTotaux = new PdfPTable(2);
		pptCadreTotaux.setWidthPercentage(100);
		int[] widths = {55,45};
		pptCadreTotaux.setWidths(widths);
		
		PdfPCell cell = new PdfPCell(pptCadreTotaux.getDefaultCell());
		cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
		
		cell.setBorder(Rectangle.NO_BORDER);
		
		cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
		cell.setPhrase(new Phrase((id.editionTTC?"Sous-total TTC :":"Sous-total HT :"), fontTotaux));
		pptCadreTotaux.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(sousTotalCourant), fontTotaux));
		pptCadreTotaux.addCell(cell);
		
		cell.setColspan(2);
		cell.setPhrase(new Phrase(""));
		pptCadreTotaux.addCell(cell);
		
		return pptCadreTotaux;	
	}
	
	
	private PdfPTable makeCadreTotaux() throws DocumentException {
	
		PdfPTable pptCadreTotaux = new PdfPTable(2);
		pptCadreTotaux.setWidthPercentage(100);
		int[] widths = {55,45};
		pptCadreTotaux.setWidths(widths);
		
		PdfPCell cell = new PdfPCell(pptCadreTotaux.getDefaultCell());
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);		
		cell.setBorder(PdfPCell.NO_BORDER);		
		cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		
		cell.setPhrase(new Phrase((id.editionTTC?"Montant TTC :":"Montant HT :"), fontTotaux));
		pptCadreTotaux.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(id.montantBase) +" \u20AC", fontTotaux));
		pptCadreTotaux.addCell(cell);
				
		if (id.tRemise>0) {		
			cell.setPhrase(new Phrase("Remise ("+ numFormatCap.format(id.tRemise) +" %) :", fontTotaux));
			pptCadreTotaux.addCell(cell);
			cell.setPhrase(new Phrase(""+ numFormat.format(id.mRemise) +" \u20AC", fontTotaux));
			pptCadreTotaux.addCell(cell);
		}
		
		cell.setPhrase(new Phrase("Frais de livraison :", fontTotaux));
		pptCadreTotaux.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(id.fraisPort) +" \u20AC", fontTotaux));
		pptCadreTotaux.addCell(cell);		
		
		cell.setPhrase(new Phrase((id.editionTTC?"Total TTC :":id.exonerationTVA?"Montant Total :":"Total HT :"),id.editionTTC || id.exonerationTVA?fontTTC:fontTotaux));
		pptCadreTotaux.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(id.totalBase) +" \u20AC", id.editionTTC || id.exonerationTVA?fontTTC:fontTotaux));
		pptCadreTotaux.addCell(cell);		

		if (!id.exonerationTVA) {
			cell.setPhrase(new Phrase((id.editionTTC?"Dont TVA :":"Total TVA :"), fontTotaux));
			pptCadreTotaux.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(id.totalTVA) +" \u20AC", fontTotaux));
			pptCadreTotaux.addCell(cell);	
		}
		if (!id.editionTTC && !id.exonerationTVA) {
			cell.setPhrase(new Phrase("Montant TTC :", fontTTC));
			pptCadreTotaux.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(id.montantTTC) +" \u20AC", fontTTC));
			pptCadreTotaux.addCell(cell);		
		}


		if (id.tEscompte>0) {
			cell.setPhrase(new Phrase("Escompte ("+ numFormatCap.format(id.tEscompte) +" %) :", fontTotaux));
			pptCadreTotaux.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(id.mEscompte) +" \u20AC", fontTotaux));
			pptCadreTotaux.addCell(cell);
		}
		if (id.acompte!=0) {
			cell.setPhrase(new Phrase("Acompte :", fontTotaux));
			pptCadreTotaux.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(id.acompte) +" \u20AC", fontTotaux));
			pptCadreTotaux.addCell(cell);	

			cell.setPhrase(new Phrase("Net \u00E0 payer :", fontTTC));
			pptCadreTotaux.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(id.netAPayer) +" \u20AC", fontTTC));
			pptCadreTotaux.addCell(cell);	
		}
		
		cell.setColspan(2);
		cell.setPhrase(new Phrase(""));
		pptCadreTotaux.addCell(cell);
		
		return pptCadreTotaux;
	}
	
	
	private PdfPTable makeZoneCommentaire() throws DocumentException {		
		
		PdfPTable pptCom = new PdfPTable(nbCols);
		pptCom.setWidthPercentage(100);
		pptCom.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptCom.getDefaultCell());
		cell.setBorder(PdfPCell.LEFT + PdfPCell.RIGHT);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);		
		cell.setPaddingLeft(5);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		cell.setPhrase(new Phrase("", fontLine));
		pptCom.addCell(cell);
		cell.setPhrase(new Phrase(id.comFin + id.mentions, fontLine));
		pptCom.addCell(cell);
		cell.setPhrase(new Phrase("", fontLine));
		pptCom.addCell(cell);
		pptCom.addCell(cell);
		pptCom.addCell(cell);
		
		if (rist) pptCom.addCell(cell);
		if (marq) pptCom.addCell(cell);		
		if (coul) pptCom.addCell(cell);		
		if (tail) pptCom.addCell(cell);
		if (unite) pptCom.addCell(cell);	
		
		if (numLot) pptCom.addCell(cell);
		if (nbPieces) pptCom.addCell(cell);
		if (datePeremption) pptCom.addCell(cell);
	
		return pptCom;		
	}
	
	
	private PdfPTable makeInfosLegales() {
	
		PdfPTable pptLegales = new PdfPTable(1);
		pptLegales.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptLegales.getDefaultCell());		
		
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPhrase(new Phrase(id.raisonSociale +" - "+ id.typeSociete + (id.capital>0?" au capital de "+ numFormatCap.format(id.capital) +" Euros":"") +" - immatricul\u00E9e au RCS "+ id.rcs +" "+ id.siren +" - N TVA : "+ id.tvaIntra, fontLegale));
		
		pptLegales.addCell(cell);
		
		return pptLegales;	
	}
	
	// mthodes de paramtrage
	
	public void setAffTotaux(boolean affTotaux) {
		
		this.affTotaux = affTotaux;
	}
	public void setAffMarque(boolean marq) {
		
		this.marq = marq;
		calculColonnes();	
	}


} // fin DevisStd
