/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Donnes ncessaires  l'dition d'un tat des stocks
 */

package org.opensi.facturation.actions.editions;

import org.experlog.openeas.api.Session;

import java.sql.*;
import java.util.ArrayList;

import org.opensi.util.tools.DateTime;


public class DataEtatStock {

	
	// les lignes de l'tat des stocks
	protected ArrayList<LigneEtatStock> lignes = new ArrayList<LigneEtatStock>(100);

	protected String dossierId;
	protected String raisonSociale;
	protected String adresse;
	protected String compAdresse;
	protected String codePostal;
	protected String ville;
	protected String monnaie;

	protected String finPeriodeStr;
	protected String debutPeriodeStr;
	protected String dateCourante;
	protected String familleArticle;
	protected String sousFamilleArticle;
	protected String marqueArticle;
	
	protected double valorisationTotale = 0;
	protected double reelTotal = 0;
	
	protected String criteres = "";
	protected boolean stockEntier;


	public DataEtatStock(Session s, String valorisation, long debutPeriode, long finPeriode, String familleArticle, String sousFamilleArticle, String marqueArticle, boolean stockEntier) {
		try {
			
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
			
			String base = s.getCookie().get("BaseDossier");
			this.dossierId = s.getCookie().get("Dossier_Id");			
			
			this.dateCourante = DateTime.formatTime(System.currentTimeMillis(), "dd/MM/yyyy");
			this.finPeriodeStr = DateTime.formatTime(finPeriode, "dd/MM/yyyy");
			this.debutPeriodeStr = DateTime.formatTime(debutPeriode, "dd/MM/yyyy");
			
			this.familleArticle = familleArticle;
			this.sousFamilleArticle = sousFamilleArticle;
			this.marqueArticle = marqueArticle;
			this.stockEntier = stockEntier;
			
			DateTime dtPeriode = new DateTime();
			dtPeriode.setDay(1);
			dtPeriode.setMonth(dtPeriode.getMonth() + 1);			

			String reqLeDossier = "select * from DOSSIER where Dossier_Id='"+ dossierId+ "'";		

			ResultSet rset = stt.executeQuery(reqLeDossier);
			if (rset.next()) {
				this.raisonSociale = rset.getString("Raison_Sociale");
				this.adresse = rset.getString("Adresse");
				this.compAdresse = rset.getString("Comp_Adresse");
				this.codePostal = rset.getString("Code_Postal");
				this.ville = rset.getString("Ville");
				this.monnaie = rset.getString("Monnaie_Tenue");
			}
			String reqStock ="";
			if (valorisation.equals("cump")){
			reqStock = "select a.Marque,a.Article_Id, coalesce(stock.Valorisation, 0) as Valorisation, a.Designation, "
											+ "s.Stock_Init + s.Entrees - s.Sorties as Stock_Reel, s.Com_Clients, s.Com_Fournisseurs "
											+ "from "+ base +".STOCKS_ARTICLE s,"+ base +".FICHE_ARTICLE a ";
			
			reqStock += "left join (select san.Article_Id,san.Valorisation from "+ base +".STOCK_ANOUVEAU san join (select Article_Id,max(Periode) as Periode from  "+ base +".STOCK_ANOUVEAU san group by Article_Id ) as max"
					+ " on max.Article_Id=san.Article_Id and max.Periode=san.Periode) as stock on stock.Article_Id=a.Fiche_Article_Id";
			reqStock += " where ";

			if (!familleArticle.equals("")) {
				reqStock += "a.Famille='"+ s.sqlEncode(familleArticle) +"' and ";
				criteres += "Famille : "+ familleArticle;

				if (!sousFamilleArticle.equals("")) {
					reqStock += "a.Sous_Famille='"+ s.sqlEncode(sousFamilleArticle) +"' and ";
					if (criteres.length()>0) {
						criteres += " - ";
					}
					criteres += "Sous-Famille : "+ sousFamilleArticle;
				}
			}

			if (!marqueArticle.equals("")) {
				reqStock += "a.Marque='"+ s.sqlEncode(marqueArticle) +"' and ";
				if (criteres.length()>0) {
					criteres += " - ";
				}
				criteres += "Marque : "+ marqueArticle;
			}
			
			if (!stockEntier) {
				reqStock += " s.Stock_Init + s.Entrees - s.Sorties>0 and ";
			}

			reqStock += "a.Article_Id=s.Article_Id and a.Supprime=0 order by a.Article_Id";
		  
			}
			else if (valorisation.equals("dpa")){
				reqStock ="select a.Marque,a.Article_Id, coalesce(a.Prix_Achat_Der, 0) as Valorisation, a.Designation, "
					+ "s.Stock_Init + s.Entrees - s.Sorties as Stock_Reel, s.Com_Clients, s.Com_Fournisseurs "
					+ "from "+ base +".STOCKS_ARTICLE s,"+ base +".FICHE_ARTICLE a  where ";
				
				if (!familleArticle.equals("")) {
					reqStock += "a.Famille='"+ s.sqlEncode(familleArticle) +"' and ";
					criteres += "Famille : "+ familleArticle;

					if (!sousFamilleArticle.equals("")) {
						reqStock += "a.Sous_Famille='"+ s.sqlEncode(sousFamilleArticle) +"' and ";
						if (criteres.length()>0) {
							criteres += " - ";
						}
						criteres += "Sous-Famille : "+ sousFamilleArticle;
					}
				}

				if (!marqueArticle.equals("")) {
					reqStock += "a.Marque='"+ s.sqlEncode(marqueArticle) +"' and ";
					if (criteres.length()>0) {
						criteres += " - ";
					}
					criteres += "Marque : "+ marqueArticle;
				}
				
				if (!stockEntier) {
					reqStock += " s.Stock_Init + s.Entrees - s.Sorties>0 and ";
				}

				reqStock += "a.Article_Id=s.Article_Id and a.Supprime=0 order by a.Article_Id";
			}
			
			rset = stt.executeQuery(reqStock);
			
			while (rset.next()) {
				
				LigneEtatStock les = new LigneEtatStock();

				les.setNumero(rset.getString("Article_Id"));
				les.setMarque(rset.getString("Marque"));
				les.setDesignation(rset.getString("Designation"));
				les.setReel(rset.getDouble("Stock_Reel"));
				les.setComClient(rset.getDouble("Com_Clients"));
				les.setComFourn(rset.getDouble("Com_Fournisseurs"));
				les.setDispo(les.reel - les.comClient);
				les.setVirtuel(les.dispo + les.comFourn);
				les.setValorisation(rset.getDouble("Valorisation"));
				
				valorisationTotale += les.valorisation * les.reel;
				reelTotal += les.reel;
				
				lignes.add(les);
			}

			rset.close();
			stt.close();
			s.closeConnection(con, null);
			
		} catch(Exception e) {
			e.printStackTrace();
		}			
	}
	
	
	public int getNbLignes() {
		return lignes.size();
	}



} // fin DataEtatStock
