/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Donnes ncessaires  l'dition d'un palmars article
 */

package org.opensi.facturation.actions.editions;


import org.experlog.openeas.api.Session;
import java.text.SimpleDateFormat;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.Date;
import java.util.ArrayList;


public class DataPalmaresArticle {
	
	// les lignes du palmars
	protected ArrayList<LignePalmaresArticle> lignes = new ArrayList<LignePalmaresArticle>(10);

	protected String dossierId;
	protected String raisonSociale;
	protected String adresse;
	protected String compAdresse;
	protected String codePostal;
	protected String ville;
	protected String monnaie;

	protected String debutPeriodeStr;
	protected String finPeriodeStr;
	protected String dateCourante;
	protected String familleArticle;
	protected String sousFamilleArticle;
	protected String marqueArticle;
	
	protected String criteres = "";
	
	
	public DataPalmaresArticle(Session s, long debutPeriode, long finPeriode, String familleArticle, String sousFamilleArticle, String marqueArticle) {
		try {
			
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
	  
			String base = s.getCookie().get("BaseDossier");
			this.dossierId = s.getCookie().get("Dossier_Id");
			
			SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");
			this.dateCourante = formatDate.format(new Date(System.currentTimeMillis()));
			this.finPeriodeStr = formatDate.format(new Date(finPeriode));
			this.debutPeriodeStr = formatDate.format(new Date(debutPeriode));
			
			this.familleArticle = familleArticle;
			this.sousFamilleArticle = sousFamilleArticle;
			this.marqueArticle = marqueArticle;

			String reqLeDossier = "select * from DOSSIER where Dossier_Id='"+ dossierId+ "'";

			stt = con.createStatement();
			ResultSet rset = stt.executeQuery(reqLeDossier);

			rset.next();

			this.raisonSociale = rset.getString("Raison_Sociale");
			this.adresse = rset.getString("Adresse");
			this.compAdresse = rset.getString("Comp_Adresse");
			this.codePostal = rset.getString("Code_Postal");
			this.ville = rset.getString("Ville");
			this.monnaie = rset.getString("Monnaie_Tenue");

			rset.close();

			String ajoutReqCritere = "";

			if (!familleArticle.equals("")) {
				ajoutReqCritere += " and fa.Famille='"+ s.sqlEncode(familleArticle) +"'";
				criteres += "Famille : "+ familleArticle;

				if (!sousFamilleArticle.equals("")) {
					ajoutReqCritere += " and fa.Sous_Famille='"+ s.sqlEncode(sousFamilleArticle) +"'";
					if (criteres.length()>0) criteres += " - ";
					criteres += "Sous-Famille : "+ sousFamilleArticle;
				}
			}

			if (!marqueArticle.equals("")) {
				ajoutReqCritere += " and fa.Marque='"+ s.sqlEncode(marqueArticle) +"'";
				if (criteres.length()>0) {
					criteres += " - ";
				}
				criteres += "Marque : "+ marqueArticle;
			}

			String reqArticleCA = " select TG.Article_Id, TG.Designation, sum(TG.Quantite) as Qte, sum(TG.CA_HT) as CA_HT from"
					+" ((select fa.Article_Id, fa.Designation, coalesce(sum(af.Quantite),0) as Quantite, coalesce(if (f.Edition_TTC=0, sum(af.Prix * af.Quantite * (1-af.Ristourne/100) * (1-f.Remise/100)), sum(af.Prix * af.Quantite * (1-af.Ristourne/100) * (1-f.Remise/100)) / (1+af.Taux_TVA/100)),0) as CA_HT"
					+" from "+ base +".FACTURE f join "+ base +".ARTICLE_FACTURE af on f.Facture_Id=af.Facture_Id, "+ base +".FICHE_ARTICLE fa"
					+" where f.Numero<>0 and f.Date_Facture<="+ finPeriode +" and f.Date_Facture>="+ debutPeriode +" and fa.Article_Id=af.Reference "+ ajoutReqCritere +" group by fa.Article_Id, fa.Designation)"
					+" union "
					+" (select fa.Article_Id, fa.Designation, -coalesce(sum(aa.Quantite),0) as Quantite, -coalesce(if (a.Edition_TTC=0, sum(aa.Prix * aa.Quantite * (1-aa.Ristourne/100) * (1-a.Remise/100)), sum(aa.Prix * aa.Quantite * (1-aa.Ristourne/100) * (1-a.Remise/100)) / (1+aa.Taux_TVA/100)), 0) as CA_HT"
					+" from "+ base +".AVOIR a join "+ base +".ARTICLE_AVOIR aa on a.Avoir_Id=aa.Avoir_Id, "+ base +".FICHE_ARTICLE fa"
					+" where a.Numero<>0 and a.Date_Avoir<="+ finPeriode +" and a.Date_Avoir>="+ debutPeriode +" and fa.Article_Id=aa.Reference "+ ajoutReqCritere +" group by fa.Article_Id, fa.Designation)) as TG"
					+" group by TG.Article_Id, TG.Designation order by TG.CA_HT desc";


			rset = stt.executeQuery(reqArticleCA);

			while (rset.next()) {
				
				LignePalmaresArticle lp = new LignePalmaresArticle();

				lp.setNumero(rset.getString("Article_Id"));
				lp.setDesignation(rset.getString("Designation"));
				lp.setCaHT(rset.getDouble("CA_HT"));
				lp.setQte(rset.getDouble("Qte"));

				lignes.add(lp);
			}

			rset.close();
			
			stt.close();
			s.closeConnection(con, null);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}


	public int getNbLignes() {
		return lignes.size();
	}


} // fin DataPalmaresArticle
