/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Donnes ncessaires  l'dition d'un palmars fournisseur
 */

package org.opensi.facturation.actions.editions;

import org.experlog.openeas.api.Session;
import java.text.SimpleDateFormat;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Date;
import java.util.ArrayList;


public class DataPalmaresFournisseur {
	
	// les lignes du palmars fournisseur
	protected ArrayList<LignePalmaresFournisseur> lignes = new ArrayList<LignePalmaresFournisseur>(10);
	
	protected String dossierId;
	protected String raisonSociale;
	protected String adresse;
	protected String compAdresse;
	protected String codePostal;
	protected String ville;
	protected String monnaie;

	protected String finPeriodeStr;
	protected String debutPeriodeStr;
	protected String dateCourante;
	
	protected String criteres = "";
	
	protected String familleFournisseur;
	protected String typeDetail;
	protected boolean detail;
	protected String titreDetail;
	
	
	public DataPalmaresFournisseur(Session s, long debutPeriode, long finPeriode, String familleFournisseur, String typeDetail) {
		
		try {

			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
			
			this.dossierId = s.getCookie().get("Dossier_Id");;
			String base = s.getCookie().get("BaseDossier");
			
			SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");
			this.dateCourante = formatDate.format(new Date(System.currentTimeMillis()));
			this.finPeriodeStr = formatDate.format(new Date(finPeriode));
			this.debutPeriodeStr = formatDate.format(new Date(debutPeriode));
			
			this.familleFournisseur = familleFournisseur;
			this.typeDetail = typeDetail;
			
			this.detail = !typeDetail.equals("N");
			
			String reqLeDossier = "select * from DOSSIER where Dossier_Id='"+ dossierId+ "'";
			
			ResultSet rset = stt.executeQuery(reqLeDossier);
			rset.next();

			this.raisonSociale = rset.getString("Raison_Sociale");
			this.adresse = rset.getString("Adresse");
			this.compAdresse = rset.getString("Comp_Adresse");
			this.codePostal = rset.getString("Code_Postal");
			this.ville = rset.getString("Ville");
			this.monnaie = rset.getString("Monnaie_Tenue");

			rset.close();
		
			String reqFournisseurCA = "select f.Fournisseur_Id, f.Raison_Sociale, sum(c.Total_HT) as CA_HT, sum(Montant_TTC) as CA_TTC from "+ base +".FICHE_FOURNISSEUR f, "+ base +".COMMANDE c where ";

			if (!familleFournisseur.equals("")) {
				reqFournisseurCA += "f.Famille='"+ s.sqlEncode(familleFournisseur) +"' and ";
				criteres += "Famille : "+ familleFournisseur;
			}

			reqFournisseurCA += "f.Fournisseur_Id=c.Fournisseur_Id and c.Numero<>0 and c.Date_Commande<="+ finPeriode +" and c.Date_Commande>="+ debutPeriode
				+" group by f.Fournisseur_Id, f.Raison_Sociale order by CA_HT desc";

			String reqDetail;

			if (this.typeDetail.equals("D")) {
				titreDetail = " date";
				reqDetail = "select Date_Commande, sum(Total_HT) as CA_HT from "+ base +".COMMANDE where Fournisseur_Id=? and Date_Commande<="+ finPeriode +" and Date_Commande>="+ debutPeriode +" and Numero<>0 group by Date_Commande order by Date_Commande";
			}
			else {
				String param;

				if (this.typeDetail.equals("F")) {
					param = "a.Famille";
					titreDetail = " famille d'article";
				}
				else if (this.typeDetail.equals("A")) {
					param = "a.Article_Id";
					titreDetail = " article";
				}
				else {
					param = "a.Famille, a.Sous_Famille";
					titreDetail = "\nfamille/sous-famille";
				}

				reqDetail = "select "+ param +" as Groupe, sum(Prix * Quantite * (1-Ristourne/100) * (1-Remise/100)) as CA_HT from "+ base +".COMMANDE c, "+ base +".ARTICLE_COMMANDE ac, "+ base +".FICHE_ARTICLE a "
					+"where c.Fournisseur_Id=? and c.Numero<>0 and c.Date_Commande<="+ finPeriode +" and c.Date_Commande>="+ debutPeriode +" and c.Commande_Id=ac.Commande_Id and a.Article_Id=ac.Reference group by "+ param +" order by "+ param;
			}
			
			PreparedStatement psDetail = con.prepareStatement(reqDetail);
			rset = stt.executeQuery(reqFournisseurCA);
			while (rset.next()) {
				if (detail) {
					psDetail.setString(1, rset.getString("Fournisseur_Id"));

					ResultSet rset2 = psDetail.executeQuery();

					String gp;

					if (rset2.next()) {
						LignePalmaresFournisseur lp = new LignePalmaresFournisseur();
						
						lp.setNumero(rset.getString("Fournisseur_Id"));
						lp.setFournisseur(rset.getString("Raison_Sociale"));
						lp.setCaHT(rset.getDouble("CA_HT"));
						lp.setCaTTC(rset.getDouble("CA_TTC"));
						lp.setDetailSeulement(false); // on affichera une ligne complte
						
						if (this.typeDetail.equals("D")) {
							gp = formatDate.format(new Date(rset2.getLong("Date_Commande")));
						}
						else {
							gp = rset2.getString("Groupe");
						}
						lp.setGroupe(gp);
						lp.setHtGp(rset2.getDouble("CA_HT"));
						
						lignes.add(lp);
					}
					
					while (rset2.next()) {
						LignePalmaresFournisseur lp = new LignePalmaresFournisseur();
						
						lp.setNumero(rset.getString("Fournisseur_Id"));
						lp.setFournisseur(rset.getString("Raison_Sociale"));
						lp.setCaHT(rset.getDouble("CA_HT"));
						lp.setCaTTC(rset.getDouble("CA_TTC"));
						lp.setDetailSeulement(true); // on n'affichera que les dtails
						
						if (this.typeDetail.equals("D")) {
							gp = formatDate.format(new Date(rset2.getLong("Date_Commande")));
						}
						else {
							gp = rset2.getString("Groupe");
						}
						lp.setGroupe(gp);
						lp.setHtGp(rset2.getDouble("CA_HT"));
						
						lignes.add(lp);
					}
				}
				else {
					LignePalmaresFournisseur lp = new LignePalmaresFournisseur();
					
					lp.setNumero(rset.getString("Fournisseur_Id"));
					lp.setFournisseur(rset.getString("Raison_Sociale"));
					lp.setCaHT(rset.getDouble("CA_HT"));
					lp.setCaTTC(rset.getDouble("CA_TTC"));
					lp.setDetailSeulement(false); // on affichera une ligne complte
					lp.setGroupe(""); // initialisation
					lp.setHtGp(0); // initialisation
					
					lignes.add(lp);
				}
			}
			
			
			rset.close();
			stt.close();
			s.closeConnection(con, null);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	
	public int getNbLignes() {
		return lignes.size();
	}


} // fin DataPalmaresFournisseur
