/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Gnration d'un palmars fournisseur en pdf
 */

package org.opensi.facturation.actions.editions;

import java.awt.Color;

import org.opensi.util.pdf.MultiPagesDocument;
import org.opensi.util.pdf.TableBuilder;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;


public class PdfPalmaresFournisseur extends MultiPagesDocument  {
	
	
	private static final Font font9N = new Font(Font.HELVETICA, 9, Font.NORMAL);
	private static final Font font9G = new Font(Font.HELVETICA, 9, Font.BOLD);
	
	private DataPalmaresFournisseur dpf;	   // les donnes dynamiques du palmars fournisseur

	private DecimalFormat numFormat = new DecimalFormat("0.00");
	private SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");

	private int[] widthsCols;
	private int nbCols;

	
	public PdfPalmaresFournisseur(String tempdir, DataPalmaresFournisseur dpf) {
		
		super(tempdir);
		this.dpf = dpf;
		
		nbCols = dpf.detail?6:4;

		if (dpf.detail) {
			int widths5[] = {14,28,14,14,20,10};
			widthsCols = widths5;
		}
		else {
			int widths4[] = {20,40,20,20};
			widthsCols = widths4;
		}
		
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		dfs.setGroupingSeparator(' ');

		numFormat.setDecimalFormatSymbols(dfs);
		numFormat.setNegativeSuffix("");
		numFormat.setNegativePrefix("-");
		numFormat.setPositiveSuffix(" ");
		numFormat.setGroupingUsed(true);
		numFormat.setGroupingSize(3);
	}
	
	
	
	/* implmentation des fonctions de MultiPagesDocument */
	
	public PdfPTable firstHeader() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(makeEntete());
		tb.add(makeSpace(10));
		tb.add(makeEntetesColonnes());
		tb.add(makeSpace(5));
		
		return tb.getTable();	
	}
	
	
	public PdfPTable middleHeader(int lastLine) throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(firstHeader());
		
		return tb.getTable();
	}
	
	
	public PdfPTable lastFooter() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		if (this.nbLines() > 0)
			tb.add(makeFinPage());
		
		return tb.getTable();
	}
	
	
	public PdfPTable middleFooter(int nbLinesPage, int lastLine) throws DocumentException {
			
		TableBuilder tb = new TableBuilder();
		
		tb.add(makeFinPage());
		
		return tb.getTable();		
	}
	
	
	public PdfPTable bodyLine(int numLine) throws DocumentException {
	
		LignePalmaresFournisseur lp = dpf.lignes.get(numLine);

		return makeLine(lp);
	}
	
	
	public PdfPTable emptyLine(float emptyHeight) throws DocumentException {
		
		PdfPTable pptSpace = new PdfPTable(nbCols);	
		return pptSpace;
	}
	
	
	public int nbLines() {
		
		return dpf.getNbLignes();
	}
	
	
	public int nbEx() {
	
		return 1;
	}
	
	
	public Image imageFond() {
	
		return null;
	}
	
	
	/* mthodes internes */
	
	private PdfPTable makeSpace(int space) {
	
		PdfPTable pptSpace = new PdfPTable(1);
		pptSpace.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setFixedHeight(space);
		cell.setPhrase(new Phrase(""));
	
		pptSpace.addCell(cell);
	
		return pptSpace;
	}	
	
	
	public PdfPTable makeEntete() throws DocumentException {


		PdfPTable bandeau = new PdfPTable(1);

		bandeau.setWidthPercentage(100);

		PdfPTable entete = new PdfPTable(3);

		PdfPCell cellEntete = new PdfPCell(entete.getDefaultCell());

		cellEntete.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellEntete.setBorder(PdfPCell.NO_BORDER);
		cellEntete.setMinimumHeight(10);

		cellEntete.setPhrase(new Phrase("Dossier : "+ dpf.dossierId, font9N));
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase(dpf.raisonSociale, font9N));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);
		cellEntete.setColspan(1);

		cellEntete.setPhrase(new Phrase("Date : " + dpf.dateCourante, font9N));
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase(dpf.adresse, font9N));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase("", font9N));

		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase(dpf.compAdresse, font9N));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);
		cellEntete.setColspan(1);

		cellEntete.setPhrase(new Phrase("Page : "+ currentPage +" / "+ totalPages, font9N));
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase(dpf.codePostal +" "+ dpf.ville, font9N));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase("Palmar\u00E8s Fournisseur en "+ dpf.monnaie, new Font(Font.HELVETICA, 16, Font.BOLD)));
		cellEntete.setColspan(3);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase(""));
		cellEntete.setFixedHeight(10);
		entete.addCell(cellEntete);

		cellEntete.setPhrase(new Phrase("P\u00E9riode du "+ dpf.debutPeriodeStr +" au "+ dpf.finPeriodeStr, font9N));
		cellEntete.setMinimumHeight(0);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		entete.addCell(cellEntete);

		if (dpf.criteres.length()>0) {
			cellEntete.setPhrase(new Phrase(dpf.criteres, font9N));
			entete.addCell(cellEntete);
		}

		PdfPCell cellBandeau = new PdfPCell(entete);
		cellBandeau.setPadding(10);

		bandeau.addCell(cellBandeau);

		return bandeau;

	}
	
	private PdfPTable makeEntetesColonnes() throws DocumentException {

		PdfPTable pptCols = new PdfPTable(nbCols);
		pptCols.setWidthPercentage(100);
		pptCols.setWidths(widthsCols);

		PdfPCell cell = new PdfPCell(pptCols.getDefaultCell());
		cell.setMinimumHeight(30);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);

		cell.setPhrase(new Phrase("Num\u00E9ro", font9G));
		pptCols.addCell(cell);

		cell.setPhrase(new Phrase("Fournisseur", font9G));
		pptCols.addCell(cell);

		cell.setPhrase(new Phrase("CA TTC", font9G));
		pptCols.addCell(cell);

		cell.setPhrase(new Phrase("CA HT", font9G));
		pptCols.addCell(cell);

		if (dpf.detail) {
			cell.setColspan(2);
			cell.setPhrase(new Phrase("D\u00E9tail par"+ dpf.titreDetail, font9G));
			pptCols.addCell(cell);
		}

		return pptCols;
	}
	
	
	private PdfPTable makeFinPage() {
		
		PdfPTable pptFP = new PdfPTable(1);
		pptFP.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptFP.getDefaultCell());
		cell.setBorder(PdfPCell.TOP);
		cell.setPhrase(new Phrase(""));
		cell.setPaddingBottom(0);
		cell.setPaddingTop(0);
		cell.setFixedHeight(1);
				
		pptFP.addCell(cell);
	
		return pptFP;
	}



	private PdfPTable makeLine(LignePalmaresFournisseur lp) throws DocumentException {

		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);

		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		
		if (!lp.detailSeulement) { // on veut toute la ligne du palmars
			cell.setPhrase(new Phrase(lp.numero, font9N));
			pptLine.addCell(cell);

			cell.setPhrase(new Phrase(lp.fournisseur, font9N));
			pptLine.addCell(cell);
	
			cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
			cell.setPaddingRight(4);
	
			cell.setPhrase(new Phrase(numFormat.format(lp.caTTC), font9N));
			pptLine.addCell(cell);
	
			cell.setPhrase(new Phrase(numFormat.format(lp.caHT), font9N));
			pptLine.addCell(cell);
			
		} else { // on ne veut que le dtail dans la ligne du palmars
			cell.setPhrase(new Phrase("", font9N));
			pptLine.addCell(cell);
			pptLine.addCell(cell);
			pptLine.addCell(cell);
			pptLine.addCell(cell);
		}
		
		if (dpf.detail) { // dans tous les cas, on affiche les dtails si le palmars le demande
			cell.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
			cell.setBorder(PdfPCell.BOX - PdfPCell.RIGHT);
			cell.setPhrase(new Phrase(lp.groupe, font9N));
			pptLine.addCell(cell);

			cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
			cell.setPaddingRight(4);
			cell.setBorder(PdfPCell.BOX - PdfPCell.LEFT);
			cell.setPhrase(new Phrase(numFormat.format(lp.htGp), font9N));
			pptLine.addCell(cell);
		}

		return pptLine;
	}
	


} // fin PdfPalmaresFournisseur
