/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Donnes ncessaires  l'export pour IDEP
 */

package org.opensi.facturation.actions.idep;

import org.experlog.openeas.api.Session;
import org.opensi.util.tools.DateTime;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.ResultSet;
import java.util.ArrayList;

public class DataExportIdep {
	
	// les lignes de l'export
	protected ArrayList<LigneExportIdep> lignes = new ArrayList<LigneExportIdep>(10);
	
	protected int niveauObligation=0;
	protected boolean declarationIntro = false;
	protected boolean declarationSimplifiee = false;
	protected int departement=0;
	protected int codeRegime=0;
	protected int codeTransactionA=0;
	protected int codeTransactionB=0;
	protected String conditionsLiv="";
	protected int situationLieu=1;
	protected int codeTransport=0;

	public DataExportIdep(Session s, long debutPeriode, long finPeriode, long dateReedition, String typeExport) {

		try {
			
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
			
			String base = s.getCookie().get("BaseDossier");
			String dossier_id = s.getCookie().get("Dossier_Id");
			this.declarationIntro = typeExport.equals("I");			
			this.situationLieu = (this.declarationIntro?1:2);
			
			String req = "select Code_Postal from DOSSIER where Dossier_Id='" + dossier_id + "'";
			ResultSet rset = stt.executeQuery(req);
			if (rset.next()) {
				this.departement=Integer.parseInt(rset.getString("Code_Postal").substring(0,2));
			}
			rset.close();
			
			boolean reedition = (dateReedition!=0);
			long dateCourante = new DateTime().getTimeInMillis();
			
			// Si on dite pour la premire fois l'export IDEP, on va tagger les documents concerns
			// REMARQUE : les lignes dont le montant total arrondi est gal  0 ne sont pas dclares
			if (!reedition) {
				if (this.declarationIntro) {
					req = "select br.BR_Id as Doc_Id, 'BR' as Type_Doc, (sum(ac.Prix*(1-ac.Ristourne/100)*abr.Quantite) * (1-c.Remise/100)) as Valeur";
					req += " from PAYS p, "+ base +".BON_RECEPTION br, "+ base +".ARTICLE_BON_RECEPTION abr, ";
					req += base +".FACTURE_FOURNISSEUR ff, "+ base +".ARTICLE_FACTURE_FOURNISSEUR aff, ";
					req += base +".COMMANDE c, "+ base +".ARTICLE_COMMANDE ac";
					req += " where br.Date_DEB=0 and abr.BR_Id=br.BR_Id and abr.Facture=1 and br.Etat='V'";
					req += " and abr.Ref_Ligne=aff.Ref_Ligne and aff.Facture_Id=ff.Facture_Id ";
					req += " and abr.Ref_Ligne=ac.Ligne_Id and ac.Commande_Id=c.Commande_Id";
					req += " and c.Code_Pays<>'FR' and c.Code_Pays=p.Code_Pays and p.Zone_UE=1";
					req += " and ff.Numero>0 and ff.Date_Facture>="+ debutPeriode +" and ff.Date_Facture<="+ finPeriode;
					req += " group by br.BR_Id having round(Valeur)>0";
				} else {
					req = "select bl.Bon_Id as Doc_Id, 'BL' as Type_Doc, (sum(if(cc.Edition_TTC=0,ac.Prix,ac.Prix/(1+ac.Taux_TVA/100))*(1-ac.Ristourne/100)*abl.Quantite) * (1-cc.Remise/100)) as Valeur";
					req += " from PAYS p, "+ base +".BON_LIVRAISON bl, "+ base +".ARTICLE_BON_LIVRAISON abl, ";
					req += base +".COM_CLIENT cc, "+ base +".ARTICLE_COM_CLIENT ac,";
					req += base +".FACTURE fact, "+ base +".OPTION_TAXATION tx";
					req += " where bl.Date_DEB=0 and abl.Bon_Id=bl.Bon_Id and abl.Ref_Ligne=ac.Ligne_Id";
					req += " and ac.Com_Id=cc.Com_Id and bl.Facture_Id=fact.Facture_Id";
					req += " and bl.Facture_Id<>0 and bl.Etat='V' and (cc.Assujetti_TVA=1 or tx.Taxe_Arrivee=1)";
					req += " and bl.Code_Pays_Liv<>'FR' and bl.Code_Pays_Liv=p.Code_Pays and p.Zone_UE=1 and p.Code_Pays=tx.Code_Pays";
					req += " and fact.Numero>0 and fact.Date_Facture>="+ debutPeriode +" and fact.Date_Facture<="+ finPeriode;
					req += " group by bl.Bon_Id having round(Valeur)>0";
					req += " union";
					req += " select fact.Facture_Id as Doc_Id, 'FACT' as Type_Doc, (sum(if(fact.Edition_TTC=0,af.Prix,af.Prix/(1+af.Taux_TVA/100))*(1-af.Ristourne/100)*af.Quantite) * (1-fact.Remise/100)) as Valeur";
					req += " from PAYS p, "+ base +".FACTURE fact, "+ base +".ARTICLE_FACTURE af, "+ base +".OPTION_TAXATION tx";
					req += " where fact.Date_DEB=0 and af.Facture_Id=fact.Facture_Id and fact.Numero>0 and fact.Directe=1 and (fact.Assujetti_TVA=1 or tx.Taxe_Arrivee=1)";
					req += " and fact.Code_Pays_Liv<>'FR' and fact.Code_Pays_Liv=p.Code_Pays and p.Zone_UE=1 and p.Code_Pays=tx.Code_Pays";
					req += " and fact.Date_Facture>="+ debutPeriode +" and fact.Date_Facture<="+ finPeriode;
					req += " group by fact.Facture_Id having round(Valeur)>0";
					req += " union";
					req += " select bl.Bon_Id as Doc_Id, 'BL' as Type_Doc, (sum(if(cw.Edition_TTC=0,ac.Prix,ac.Prix/(1+ac.Taux_TVA/100))*(1-ac.Ristourne/100)*abl.Quantite) * (1-cw.Remise/100)) as Valeur";
					req += " from PAYS p, "+ base +".BON_LIVRAISON bl, "+ base +".ARTICLE_BON_LIVRAISON abl, ";
					req += base +".COMMANDE_WEB cw, "+ base +".ARTICLE_COMMANDE_WEB ac, ";
					req += base +".FACTURE fact, "+ base +".OPTION_TAXATION tx";
					req += " where bl.Date_DEB=0 and abl.Bon_Id=bl.Bon_Id and abl.Ref_Ligne_Web=ac.Detail_Id";
					req += " and ac.Com_Web_Id=cw.Com_Web_Id and bl.Facture_Id=fact.Facture_Id and bl.Facture_Id<>0";
					req += " and bl.Etat='V' and (cw.Assujetti_TVA=1 or tx.Taxe_Arrivee=1)";
					req += " and bl.Code_Pays_Liv<>'FR' and bl.Code_Pays_Liv=p.Code_Pays and p.Zone_UE=1 and p.Code_Pays=tx.Code_Pays";
					req += " and fact.Numero>0 and fact.Date_Facture>="+ debutPeriode +" and fact.Date_Facture<="+ finPeriode;
					req += " group by bl.Bon_Id having round(Valeur)>0";
				}
				
				String reqMajTagBR = "update "+ base +".BON_RECEPTION set Date_DEB="+dateCourante+" where BR_Id=?";
				String reqMajTagBL = "update "+ base +".BON_LIVRAISON set Date_DEB="+dateCourante+" where Bon_Id=?";
				String reqMajTagFact = "update "+ base +".FACTURE set Date_DEB="+dateCourante+" where Facture_Id=?";
				PreparedStatement psTagBR = con.prepareStatement(reqMajTagBR);
				PreparedStatement psTagBL = con.prepareStatement(reqMajTagBL);
				PreparedStatement psTagFact = con.prepareStatement(reqMajTagFact);
				
				rset = stt.executeQuery(req);
				while (rset.next()) {
					if (rset.getString("Type_Doc").equals("BR")) {
						psTagBR.setInt(1, rset.getInt("Doc_Id"));
						psTagBR.executeUpdate();
					} else if (rset.getString("Type_Doc").equals("BL")) {
						psTagBL.setInt(1, rset.getInt("Doc_Id"));
						psTagBL.executeUpdate();
					} else if (rset.getString("Type_Doc").equals("FACT")) {
						psTagFact.setInt(1, rset.getInt("Doc_Id"));
						psTagFact.executeUpdate();
					}
				}
				rset.close();
				
				psTagBR.close();
				psTagBL.close();
				psTagFact.close();
			}
			
			
			String reqParam = "";
			
			if (this.declarationIntro) {
				reqParam = "select Niveau_Obligation_Intro as Niveau_Obligation, Code_Regime_Intro as Code_Regime, Nature_Transaction_Intro as Nature_Transaction, Conditions_Liv_Intro as Conditions_Liv, Mode_Transport_Intro as Mode_Transport";
				reqParam += " from PARAM_DOSSIER where Dossier_Id='" + dossier_id + "'";
				
				req = "select (sum(ac.Prix*(1-ac.Ristourne/100)*abr.Quantite) * (1-c.Remise/100)) as Valeur,";
				req += " coalesce(sum(fa.Poids_Net*abr.Quantite),0) as Masse_Nette, coalesce(fa.Mesure_Poids,'K') as Mesure_Poids,";
				req += " coalesce(fa.Code_NC8,'') as Code_NC8, if(n.Libelle is null,'',n.Libelle) as Libelle_Produit, c.Code_Pays,";
				req += " ff.Num_Entier as Ref_Facture";
				req += " from PAYS p, "+ base +".BON_RECEPTION br, "+ base +".ARTICLE_BON_RECEPTION abr, ";
				req += base +".FACTURE_FOURNISSEUR ff, "+ base +".ARTICLE_FACTURE_FOURNISSEUR aff, ";
				req += base +".COMMANDE c, "+ base +".ARTICLE_COMMANDE ac";
				req += " left join "+ base +".FICHE_ARTICLE fa on fa.Article_Id=ac.Reference and ac.Type_Ligne='S'";
				req += " left join NOMENCLATURE_NC8 n on n.Code_NC8=fa.Code_NC8";
				req += " where br.Date_DEB="+(reedition?dateReedition:dateCourante);
				req += " and abr.Ref_Ligne=aff.Ref_Ligne and aff.Facture_Id=ff.Facture_Id";
				req += " and abr.BR_Id=br.BR_Id and abr.Ref_Ligne=ac.Ligne_Id";
				req += " and ac.Commande_Id=c.Commande_Id and abr.Facture=1";
				req += " and c.Code_Pays=p.Code_Pays";
				req += " group by ff.Facture_Id, Code_NC8 having round(Valeur)>0";
			} else {
				reqParam = "select Niveau_Obligation_Expe as Niveau_Obligation, Code_Regime_Expe as Code_Regime, Nature_Transaction_Expe as Nature_Transaction, Conditions_Liv_Expe as Conditions_Liv, Mode_Transport_Expe as Mode_Transport";
				reqParam += " from PARAM_DOSSIER where Dossier_Id='" + dossier_id + "'";
			
				req = "select Valeur, coalesce(Masse_Nette,0) as Masse_Nette, coalesce(Mesure_Poids,'K') as Mesure_Poids,";
				req += " Com.Code_NC8, coalesce(n.Libelle,'') as Libelle_Produit, Num_TVA_Intra, Code_Pays_Liv as Code_Pays,";
				req += " Num_Entier as Ref_Facture from";
				req += " (select (sum(if(cc.Edition_TTC=0,ac.Prix,ac.Prix/(1+ac.Taux_TVA/100))*(1-ac.Ristourne/100)*abl.Quantite) * (1-cc.Remise/100)) as Valeur,";
				req += " sum(coalesce(fa.Poids_Net,1)*abl.Quantite) as Masse_Nette, fa.Mesure_Poids, coalesce(fa.Code_NC8,'') as Code_NC8, cc.Num_TVA_Intra,";
				req += " bl.Code_Pays_Liv, fact.Num_Entier";
				req += " from PAYS p, "+ base +".BON_LIVRAISON bl, "+ base +".ARTICLE_BON_LIVRAISON abl, ";
				req += base +".FACTURE fact, ";
				req += base +".COM_CLIENT cc, "+ base +".ARTICLE_COM_CLIENT ac";
				req += " left join "+ base +".FICHE_ARTICLE fa on fa.Article_Id=ac.Reference and ac.Type_Ligne='S'";
				req += " where bl.Date_DEB="+(reedition?dateReedition:dateCourante);
				req += " and abl.Bon_Id=bl.Bon_Id and abl.Ref_Ligne=ac.Ligne_Id and ac.Com_Id=cc.Com_Id";
				req += " and bl.Facture_Id=fact.Facture_Id and bl.Code_Pays_Liv=p.Code_Pays";
				req += " group by fact.Facture_Id, Code_NC8 having round(Valeur)>0";
				req += " union";
				req += " select (sum(if(fact.Edition_TTC=0,af.Prix,af.Prix/(1+af.Taux_TVA/100))*(1-af.Ristourne/100)*af.Quantite) * (1-fact.Remise/100)) as Valeur,";
				req += " sum(coalesce(fa.Poids_Net,1)*af.Quantite) as Masse_Nette, fa.Mesure_Poids, coalesce(fa.Code_NC8,'') as Code_NC8, fact.Num_TVA_Intra,";
				req += " fact.Code_Pays_Liv, fact.Num_Entier";
				req += " from PAYS p, "+ base +".FACTURE fact, "+ base +".ARTICLE_FACTURE af";
				req += " left join "+ base +".FICHE_ARTICLE fa on fa.Article_Id=af.Reference and af.Type_Ligne='S'";
				req += " where fact.Date_DEB="+(reedition?dateReedition:dateCourante);
				req += " and af.Facture_Id=fact.Facture_Id";
				req += " and fact.Code_Pays_Liv=p.Code_Pays";
				req += " group by fact.Facture_Id, Code_NC8 having round(Valeur)>0";
				req += " union";
				req += " select (sum(if(cw.Edition_TTC=0,ac.Prix,ac.Prix/(1+ac.Taux_TVA/100))*(1-ac.Ristourne/100)*abl.Quantite) * (1-cw.Remise/100)) as Valeur,";
				req += " sum(coalesce(fa.Poids_Net,1)*abl.Quantite) as Masse_Nette, fa.Mesure_Poids, coalesce(fa.Code_NC8,'') as Code_NC8, cw.Num_TVA_Intra,";
				req += " bl.Code_Pays_Liv, fact.Num_Entier";
				req += " from PAYS p, "+ base +".BON_LIVRAISON bl, "+ base +".ARTICLE_BON_LIVRAISON abl, ";
				req += base +".FACTURE fact, "+ base +".COMMANDE_WEB cw, "+ base +".ARTICLE_COMMANDE_WEB ac";
				req += " left join "+ base +".FICHE_ARTICLE fa on fa.Article_Id=ac.Article_Id and ac.Type_Ligne='S'";
				req += " where bl.Date_DEB="+(reedition?dateReedition:dateCourante);
				req += " and abl.Bon_Id=bl.Bon_Id and abl.Ref_Ligne_Web=ac.Detail_Id and ac.Com_Web_Id=cw.Com_Web_Id";
				req += " and bl.Facture_Id=fact.Facture_Id and bl.Code_Pays_Liv=p.Code_Pays";
				req += " group by fact.Facture_Id, Code_NC8 having round(Valeur)>0) as Com";
				req += " left join NOMENCLATURE_NC8 n on n.Code_NC8=Com.Code_NC8 order by Ref_Facture";
			}
			rset = stt.executeQuery(reqParam);
			if (rset.next()) {
				this.niveauObligation=rset.getInt("Niveau_Obligation");
				this.declarationSimplifiee=(this.niveauObligation>2);
				this.codeRegime=rset.getInt("Code_Regime");
				String natureTransaction=rset.getString("Nature_Transaction");
				this.codeTransactionA=Integer.parseInt(natureTransaction.substring(0,1));
				this.codeTransactionB=Integer.parseInt(natureTransaction.substring(1,2));
				this.conditionsLiv=rset.getString("Conditions_Liv");
				this.codeTransport=rset.getInt("Mode_Transport");
			}
			rset.close();
			
			rset = stt.executeQuery(req);
			while (rset.next()) {
				LigneExportIdep lei = new LigneExportIdep();
				
				lei.setProvenanceDestination(rset.getString("Code_Pays"));
				lei.setValeur(rset.getDouble("Valeur"));
				lei.setReferenceProduit(rset.getString("Code_NC8"));
				lei.setCodeNGP9("");
				lei.setLibelleProduit(rset.getString("Libelle_Produit"));
				
				double poids = rset.getDouble("Masse_Nette");
				String uniteMesure = rset.getString("Mesure_Poids");
				if (uniteMesure.equals("G")) { poids = poids/1000; }
				else if (uniteMesure.equals("T")) { poids = poids*1000; }
				
				lei.setMasseNette(poids);
				lei.setValeurStatistique(rset.getDouble("Valeur"));
				lei.setUnitesSupplementaires(0);
				lei.setPaysOrigine(rset.getString("Code_Pays"));
				lei.setNumTVAPartenaire(this.declarationIntro?"":rset.getString("Num_TVA_Intra"));				
				lei.setRefFacture(rset.getString("Ref_Facture"));
				lei.setCodeDevise("EUR");
				lei.setValeurDevise(rset.getDouble("Valeur"));
				lei.setValeurStatDevise(rset.getDouble("Valeur"));

				lignes.add(lei);
			}
			rset.close();
			
			stt.close();
			s.closeConnection(con, null);
			
		} catch(Exception e) {
			e.printStackTrace();
		}			
	}
		
		
	public int getNbLignes() {
		return lignes.size();
	}



} // fin DataExportIdep
