/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Donnes pour l'edition de l'inventaire
 */

package org.opensi.facturation.actions.inventaire;

import java.sql.*;

import java.util.ArrayList;

import org.experlog.openeas.api.Session;
import org.opensi.util.tools.DateTime;


public class DataInventaire {

	protected ArrayList<LigneInventaire> lignes = new ArrayList<LigneInventaire>(10);

	protected String raisonSociale;
	protected String adresse;
	protected String compAdresse;
	protected String codePostal;
	protected String ville;
	protected String dateCourante;
	
	protected String dossierId;
	protected int inventaireId;
	protected String typeInventaire;
	protected String criteres = "";
	
	protected String typeEdition = "V"; // 'V'-> inventaire vierge; 'E'-> edition des carts; 'R'-> inventaire rempli
	
	
	public DataInventaire(Session s, int inventaireId, String marque, String famille, String sousFamille, String typeInventaire, String typeEdition) {	
		try {			
      			
			String base = s.getCookie().get("BaseDossier");
			this.dossierId = s.getCookie().get("Dossier_Id");			
			this.dateCourante = DateTime.formatTime(System.currentTimeMillis(), "dd/MM/yyyy");

			this.inventaireId = inventaireId;
			this.typeInventaire = typeInventaire;
			this.typeEdition = typeEdition;

			if (!marque.equals("")) this.criteres += " Marque : "+ marque;
			if (!famille.equals("")) this.criteres += (!criteres.equals("")?" - ":"") +" Famille : "+ famille;
			if (!sousFamille.equals("")) this.criteres += (!criteres.equals("")?" - ":"") +" Sous_Famille : "+ sousFamille;

			
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();
			
			String reqLeDossier = "select * from DOSSIER where Dossier_Id='"+ dossierId+ "'";

			ResultSet rset = stt.executeQuery(reqLeDossier);
			rset.next();
			this.raisonSociale = rset.getString("Raison_Sociale");
			this.adresse = rset.getString("Adresse");
			this.compAdresse = rset.getString("Comp_Adresse");
			this.codePostal = rset.getString("Code_Postal");
			this.ville = rset.getString("Ville");
			rset.close();

			
			String reqArticles = " select ai.Article_Id, fa.Designation, fa.Unite, ai.Qte_Theorique as Stock_Info, ai.Qte_Inventaire as Stock_Physique, ai.Valorisation";
			reqArticles += " from "+ base +".FICHE_ARTICLE fa,"+ base +".ARTICLE_INVENTAIRE ai";
			reqArticles += " where ai.Article_Id=fa.Article_Id and Inventaire_Id="+ inventaireId;
			if (typeEdition.equals("E")) {
				reqArticles += " and ai.Qte_Theorique<>ai.Qte_Inventaire";
			}
			if (typeInventaire.equalsIgnoreCase("C")) {
				if (!marque.equals("")) reqArticles += " and fa.Marque='"+ s.sqlEncode(marque) +"'";
				if (!famille.equals("")) reqArticles += " and fa.Famille='"+ s.sqlEncode(famille) +"'";
				if (!sousFamille.equals("")) reqArticles += " and fa.Sous_Famille='"+ s.sqlEncode(sousFamille) +"'";
			}							 
			reqArticles += " order by ai.Article_Id";
			
			rset = stt.executeQuery(reqArticles);
			while (rset.next()) {
				LigneInventaire li = new LigneInventaire();

				li.setRefArticle(rset.getString("Article_Id"));
				li.setDesignation(rset.getString("Designation"));
				li.setUnite(rset.getString("Unite"));
				li.setStockInfo(rset.getDouble("Stock_Info"));
				li.setStockPhysique(rset.getDouble("Stock_Physique"));
				li.setValorisation(rset.getDouble("Valorisation")*rset.getDouble("Stock_Physique"));

				lignes.add(li);
			}

			stt.close();
			s.closeConnection(con, null);
			
		} catch(Exception e) { e.printStackTrace();	}
	
	}
	
	
	public int getNbLignes() {
		return lignes.size();
	}


} // fin DataInventaire
