/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 * Edition de la mise  jour des tarifs spcifiques
 */

package org.opensi.facturation.actions.majtarifs;


import java.awt.Color;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

import org.opensi.util.pdf.MultiPagesDocument;
import org.opensi.util.pdf.PdfWidths;
import org.opensi.util.pdf.TableBuilder;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;



public class PdfMajTarifsSpec extends MultiPagesDocument {
	
	private static final Font fontLine = new Font(Font.HELVETICA, 8, Font.NORMAL);
	private static final Font fontColonne = new Font(Font.HELVETICA, 8, Font.BOLD);
	private static final Font fontTitre = new Font(Font.HELVETICA, 16, Font.BOLD, Color.WHITE);
	private static final Font fontLegale = new Font(Font.HELVETICA , 7, Font.NORMAL);

	private DataMajTarifs id;	   // les donnes dynamiques des tarifs

	private float[] widthsCols;
	private int nbCols;
	
	private DecimalFormat numFormat = new DecimalFormat("0.00");
	private DecimalFormat numFormatCap = new DecimalFormat("0.##");
	

	public PdfMajTarifsSpec(String tempdir, DataMajTarifs id) throws Exception {
		
		super(tempdir);
		this.id = id;
		
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		dfs.setGroupingSeparator(' ');

		numFormat.setDecimalFormatSymbols(dfs);
		numFormat.setNegativeSuffix("");
		numFormat.setNegativePrefix("-");
		numFormat.setPositiveSuffix(" ");
		numFormat.setGroupingUsed(true);
		numFormat.setGroupingSize(3);
		
		numFormatCap.setDecimalFormatSymbols(dfs);
		numFormatCap.setGroupingUsed(true);
		numFormatCap.setGroupingSize(3);
		
		PdfWidths pw = new PdfWidths();
		
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		if (id.element==4) {
			pw.addColonneVariableMin(1, 10);
			pw.addColonneVariableMin(1, 10);
		}
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		pw.addColonneVariableMin(1, 10);
		
		nbCols = pw.getNbCols();
		widthsCols = pw.getWidthsCols();

	}
	
	
	
	/* implmentation des fonctions de MultiPagesDocument */
	
	public PdfPTable firstHeader() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(makeEntete());
		tb.add(makeSpace(10));
		tb.add(makeTitre());
		tb.add(makeSpace(5));
		tb.add(makeEntetesColonnes());
		
		return tb.getTable();	
	}
	
	
	public PdfPTable middleHeader(int lastLine) throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(firstHeader());
		
		return tb.getTable();
	}
	
	
	public PdfPTable lastFooter() throws DocumentException {
		
		TableBuilder tb = new TableBuilder();
		
		tb.add(makeSpace(3));
		tb.add(makeInfosLegales());
		
		return tb.getTable();
	}
	
	
	public PdfPTable middleFooter(int nbLinesPage, int lastLine) throws DocumentException {
			
		TableBuilder tb = new TableBuilder();
		
		tb.add(lastFooter());

		return tb.getTable();		
	}
	
	
	public PdfPTable bodyLine(int numLine) throws DocumentException {
	
		LigneTarifSpecifique lts = id.lignesTarifsSpecifiques.get(numLine);
		
		return makeLine(lts);
	}
	
	
	public PdfPTable emptyLine(float emptyHeight) throws DocumentException {
		
		PdfPTable pptSpace = new PdfPTable(nbCols);
		pptSpace.setWidthPercentage(100);
		pptSpace.setWidths(widthsCols);
		
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.TOP);
		cell.setPaddingBottom(0);
		cell.setPaddingTop(0);
		cell.setFixedHeight(emptyHeight);		
		cell.setPhrase(new Phrase(""));		
	
		for (int i=0; i<nbCols; i++) {
			pptSpace.addCell(cell);
		}
	
		return pptSpace;
	}
	
	
	public int nbLines() {
		
		return id.getNbLignesTarifsSpec();
	}
	
	
	public int nbEx() {
	
		return 1;
	}
	
	
	public Image imageFond() {
	
		return null;
	}
	
	
	
	/* mthodes internes */

	private PdfPTable makeSpace(int space) {
		
		PdfPTable pptSpace = new PdfPTable(1);
		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setFixedHeight(space);
		cell.setPhrase(new Phrase(""));
	
		pptSpace.addCell(cell);
	
		return pptSpace;
	}
	
	
	private PdfPTable makeEntete() throws DocumentException {
		
		PdfPTable bandeau = new PdfPTable(1);
	
		bandeau.setWidthPercentage(100);
	
		PdfPTable entete = new PdfPTable(3);
	
		PdfPCell cellEntete = new PdfPCell(entete.getDefaultCell());
	
		cellEntete.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cellEntete.setBorder(PdfPCell.NO_BORDER);
		cellEntete.setMinimumHeight(10);
	
		cellEntete.setPhrase(new Phrase("Dossier : "+ id.dossierId, fontLine));
		entete.addCell(cellEntete);
	
		cellEntete.setPhrase(new Phrase(id.raisonSociale, fontLine));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);
		cellEntete.setColspan(1);
	
		cellEntete.setPhrase(new Phrase("Date : " + id.dateCourante, fontLine));
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		entete.addCell(cellEntete);
	
		cellEntete.setPhrase(new Phrase(id.adresse, fontLine));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);
	
		cellEntete.setPhrase(new Phrase("", fontLine));
	
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		entete.addCell(cellEntete);
	
		cellEntete.setPhrase(new Phrase(id.compAdresse, fontLine));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);
		cellEntete.setColspan(1);
	
		cellEntete.setPhrase(new Phrase("Page : "+ currentPage +" / "+ totalPages, fontLine));
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_LEFT);
		entete.addCell(cellEntete);
	
		cellEntete.setPhrase(new Phrase(id.codePostal +" "+ id.ville, fontLine));
		cellEntete.setColspan(2);
		cellEntete.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		entete.addCell(cellEntete);

		PdfPCell cellBandeau = new PdfPCell(entete);
		cellBandeau.setPadding(10);
	
		bandeau.addCell(cellBandeau);
		
		return bandeau;		
	}
	
	
	private PdfPTable makeTitre() {
		
		PdfPTable pptTitre = new PdfPTable(1);
		pptTitre.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptTitre.getDefaultCell());		
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setBackgroundColor(new Color(175,175,175));
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setFixedHeight(30);
		cell.setPaddingBottom(5);
		cell.setPhrase(new Phrase("MISE  A  JOUR  DES  TARIFS  SPECIFIQUES", fontTitre));
		
		pptTitre.addCell(cell);
		
		return pptTitre;	
	}
	
	
	
	private PdfPTable makeEntetesColonnes() throws DocumentException {

		PdfPTable pptCols = new PdfPTable(nbCols);
		pptCols.setWidthPercentage(100);
		pptCols.setWidths(widthsCols);

		PdfPCell cell = new PdfPCell(pptCols.getDefaultCell());
		cell.setMinimumHeight(30);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		
		cell.setPhrase(new Phrase("Code Article", fontColonne));
		pptCols.addCell(cell);		
		cell.setPhrase(new Phrase("R\u00E9f. Client", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Raison Sociale", fontColonne));
		pptCols.addCell(cell);
		
		if (id.element==4) {
			cell.setPhrase(new Phrase("Anc. Prix Achat (\u20AC)", fontColonne));
			pptCols.addCell(cell);				
			cell.setPhrase(new Phrase("Nv. Prix Achat (\u20AC)", fontColonne));
			pptCols.addCell(cell);
		}
		
		cell.setPhrase(new Phrase("Anc. Coeff", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Nv. Coeff", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Anc. Prix HT (\u20AC)", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Nv. Prix HT (\u20AC)", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Anc. Prix TTC (\u20AC)", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Nv. Prix TTC (\u20AC)", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Anc. Marge (%)", fontColonne));
		pptCols.addCell(cell);
		cell.setPhrase(new Phrase("Nv. Marge (%)", fontColonne));
		pptCols.addCell(cell);
		

		return pptCols;
	}
	



	private PdfPTable makeLine(LigneTarifSpecifique lts) throws DocumentException {

		PdfPTable pptLine = new PdfPTable(nbCols);
		pptLine.setWidthPercentage(100);
		pptLine.setWidths(widthsCols);

		PdfPCell cell = new PdfPCell(pptLine.getDefaultCell());
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);

		cell.setPhrase(new Phrase(lts.refArticle, fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(lts.refClient, fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(lts.raisonSociale, fontLine));
		pptLine.addCell(cell);
		
		cell.setHorizontalAlignment(PdfPCell.ALIGN_RIGHT);
		
		if (id.element==4) {
			cell.setPhrase(new Phrase(numFormat.format(lts.ancPrixAchat), fontLine));
			pptLine.addCell(cell);
			cell.setPhrase(new Phrase(numFormat.format(lts.nvPrixAchat), fontLine));
			pptLine.addCell(cell);
		}
		
		cell.setPhrase(new Phrase(numFormat.format(lts.ancCoeff), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.nvCoeff), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.ancPrixHT), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.nvPrixHT), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.ancPrixTTC), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.nvPrixTTC), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.ancMarge), fontLine));
		pptLine.addCell(cell);
		cell.setPhrase(new Phrase(numFormat.format(lts.nvMarge), fontLine));
		pptLine.addCell(cell);

		
		return pptLine;
	}
	
	
	private PdfPTable makeInfosLegales() {
		
		PdfPTable pptLegales = new PdfPTable(1);
		pptLegales.setWidthPercentage(100);
		
		PdfPCell cell = new PdfPCell(pptLegales.getDefaultCell());		
		
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPhrase(new Phrase(id.raisonSociale +" - "+ id.typeSociete + (id.capital>0?" au capital de "+ numFormatCap.format(id.capital) +" Euros":"") +" - immatricul\u00E9e au RCS "+ id.rcs +" "+ id.siren +" - N TVA : "+ id.tvaIntra, fontLegale));
		
		pptLegales.addCell(cell);
		
		return pptLegales;	
	}


} // fin PdfMajTarifsSpec
