/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

package org.opensi.facturation.actions.transcompta;

import java.sql.*;

import org.experlog.openeas.api.Session;
import org.experlog.openeas.api.Action;
import org.experlog.openeas.api.ServletRequest;

import java.util.HashMap;
import java.util.Iterator;

import org.opensi.compta.Ecriture;
import org.opensi.compta.Operation;

import org.opensi.util.tools.DateTime;
import org.opensi.util.tools.Arrondi;


public class TransfertAchats extends Action {


  private Connection con;
  private String base;

	private String compteAchatDefaultUE;
	private String compteAchatDefaultI;
	private String compteFournisseursDivers;

	private PreparedStatement psCpteFournisseur;
	private PreparedStatement psCpteArticleUE;
	private PreparedStatement psCpteFamilleUE;
	private PreparedStatement psCpteSousFamilleUE;
	private PreparedStatement psCpteArticleI;
	private PreparedStatement psCpteFamilleI;
	private PreparedStatement psCpteSousFamilleI;
	private PreparedStatement psCpteArticleN;
	private PreparedStatement psCpteFamilleN;
	private PreparedStatement psCpteSousFamilleN;
	private PreparedStatement psIntCpte;
	private PreparedStatement psMarqueFact;
	private PreparedStatement psMarqueAvoir;
	private PreparedStatement psBaseCompta;

	HashMap<Double, String> cptesATva = new HashMap<Double, String>(3);

	private Arrondi rd = new Arrondi(2);


  public boolean serverSide() { return true; }

  public boolean service(Session s, ServletRequest r, Object info) throws Exception {


    String codeJournal = r.getParameter("Code_Journal");
		String typePeriode = r.getParameter("Type_Periode");
		String fournisseur = r.getParameter("Fournisseur_Id");

		long debutPeriode;
		long finPeriode;

		if (typePeriode.equals("P")) {
			long periode = Long.parseLong(r.getParameter("Periode"));

			DateTime dtp = new DateTime(periode);
			dtp.setDay(1);
			debutPeriode = dtp.getDateInMillis();

			dtp.setDay(dtp.currentMaxDay());
			finPeriode = dtp.getDateFullTime();
		}
		else {
			debutPeriode = Long.parseLong(r.getParameter("Date_Debut"));
			finPeriode = Long.parseLong(r.getParameter("Date_Fin"));
		}


		String dossierId = s.getCookie().get("Dossier_Id");

		base = s.getCookie().get("BaseDossier");

    con = s.getConnection(null);

		String reqBaseCompta = "select Nom_Base from EXERCICE where Debut_Exercice<=? and Fin_Exercice>=? and Dossier_Id='"+ dossierId +"'";

		ResultSet rset;

		Statement stt = con.createStatement();

		String reqComptesDefaut = "select Numero_Compte_Achat_UE, Numero_Compte_Achat_I, Numero_Compte_Fournisseurs, Numero_Compte_Escompte_AC, Numero_Compte_Port_AC "
														+ "from PARAM_DOSSIER where Dossier_Id='"+ dossierId +"'";

		rset = stt.executeQuery(reqComptesDefaut);
		rset.next();

		compteAchatDefaultUE = rset.getString("Numero_Compte_Achat_UE");
		compteAchatDefaultI = rset.getString("Numero_Compte_Achat_I");
		compteFournisseursDivers = rset.getString("Numero_Compte_Fournisseurs");
    String compteLivraison = rset.getString("Numero_Compte_Port_AC");
    String compteEscompteAC = rset.getString("Numero_Compte_Escompte_AC");

		HashMap<Double, String> cptesTva = new HashMap<Double, String>(3);

		String reqCptesTva = "select Compte_TVA_Achat, Compte_Achat, Taux_TVA from "+ base +".TAUX_TVA where Code_Pays='FR'";		//  revoir au changement du mode de ventilation

		rset = stt.executeQuery(reqCptesTva);

		while (rset.next()) {
			cptesTva.put(new Double(rset.getDouble("Taux_TVA")), rset.getString("Compte_TVA_Achat"));
			cptesATva.put(new Double(rset.getDouble("Taux_TVA")), rset.getString("Compte_Achat"));
		}



		String reqCpteFournisseur = "select c.Numero_Compte from "+ base +".FICHE_FOURNISSEUR e,"+ base +".COMPTE c "
												 + "where e.Fournisseur_Id=? and e.Numero_Compte=c.Numero_Compte";


		String reqCpteArticleUE = "select f.Compte_Achat_UE from "+ base +".FICHE_ARTICLE f,"+ base +".COMPTE c "
										 			+ "where f.Article_Id=? and f.Compte_Achat_UE=c.Numero_Compte";

		String reqCpteArticleI = "select f.Compte_Achat_I from "+ base +".FICHE_ARTICLE f,"+ base +".COMPTE c "
										 			+ "where f.Article_Id=? and f.Compte_Achat_I=c.Numero_Compte";

		String reqCpteArticleN = "select ca.Compte_Achat from "+ base +".COMPTE_ARTICLE ca,"+ base +".TAUX_TVA t,"+ base +".COMPTE c "
														+ "where t.Code_TVA=ca.Code_TVA and c.Numero_Compte=ca.Compte_Achat and Article_Id=? and t.Taux_TVA=?";

		String reqCpteFamilleUE = "select f.Compte_Achat_UE from "+ base +".FAMILLE_ARTICLE f,"+ base +".COMPTE c,"+ base +".FICHE_ARTICLE a "
                     			+ "where a.Article_Id=? and f.Famille=a.Famille and f.Compte_Achat_UE=c.Numero_Compte";

		String reqCpteFamilleI = "select f.Compte_Achat_I from "+ base +".FAMILLE_ARTICLE f,"+ base +".COMPTE c,"+ base +".FICHE_ARTICLE a "
                     			+ "where a.Article_Id=? and f.Famille=a.Famille and f.Compte_Achat_I=c.Numero_Compte";

		String reqCpteFamilleN = "select cf.Compte_Achat from "+ base +".COMPTE_FAMILLE_ARTICLE cf,"+ base +".TAUX_TVA t,"+ base +".COMPTE c,"+ base +".FICHE_ARTICLE a "
														+ "where t.Code_TVA=cf.Code_TVA and c.Numero_Compte=cf.Compte_Achat and a.Famille=cf.Famille and a.Article_Id=? and t.Taux_TVA=?";

		String reqCpteSousFamilleUE = "select f.Compte_Achat_UE from "+ base +".SOUS_FAMILLE_ARTICLE f,"+ base +".COMPTE c,"+ base +".FICHE_ARTICLE a "
                     					+ "where a.Article_Id=? and f.Sous_Famille=a.Sous_Famille and f.Famille=a.Famille and f.Compte_Achat_UE=c.Numero_Compte";

		String reqCpteSousFamilleI = "select f.Compte_Achat_I from "+ base +".SOUS_FAMILLE_ARTICLE f,"+ base +".COMPTE c,"+ base +".FICHE_ARTICLE a "
                     					+ "where a.Article_Id=? and f.Sous_Famille=a.Sous_Famille and f.Famille=a.Famille and f.Compte_Achat_I=c.Numero_Compte";

		String reqCpteSousFamilleN = "select csf.Compte_Achat from "+ base +".COMPTE_SOUS_FAMILLE_ARTICLE csf,"+ base +".TAUX_TVA t,"+ base +".COMPTE c,"+ base +".FICHE_ARTICLE a "
														+ "where t.Code_TVA=csf.Code_TVA and c.Numero_Compte=csf.Compte_Achat and a.Famille=csf.Famille and csf.Sous_Famille=a.Sous_Famille and a.Article_Id=? and t.Taux_TVA=?";


		String reqIntCompte = "select Intitule from "+ base +".COMPTE where Numero_Compte=?";

		String majMarqueFact = "update "+ base +".FACTURE_FOURNISSEUR set Transferee=1 where Facture_Id=?";

		String majMarqueAvoir = "update "+ base +".AVOIR_FOURNISSEUR set Transfere=1 where Avoir_Id=?";



		psCpteFournisseur = con.prepareStatement(reqCpteFournisseur);
		psCpteArticleUE = con.prepareStatement(reqCpteArticleUE);
		psCpteArticleI = con.prepareStatement(reqCpteArticleI);
		psCpteArticleN = con.prepareStatement(reqCpteArticleN);
		psCpteFamilleUE = con.prepareStatement(reqCpteFamilleUE);
		psCpteFamilleI = con.prepareStatement(reqCpteFamilleI);
		psCpteFamilleN = con.prepareStatement(reqCpteFamilleN);
		psCpteSousFamilleUE = con.prepareStatement(reqCpteSousFamilleUE);
		psCpteSousFamilleI = con.prepareStatement(reqCpteSousFamilleI);
		psCpteSousFamilleN = con.prepareStatement(reqCpteSousFamilleN);
		psIntCpte = con.prepareStatement(reqIntCompte);
		psMarqueFact = con.prepareStatement(majMarqueFact);
		psMarqueAvoir = con.prepareStatement(majMarqueAvoir);
		psBaseCompta = con.prepareStatement(reqBaseCompta);




		// GENERATION DES ECRITURES DE FACTURES

		String reqVentilTvas = "select Taux_TVA, sum(Prix * Quantite * (1-Ristourne/100)) * Taux_TVA/100 as Montant_TVA "
												 + "from "+ base +".ARTICLE_FACTURE_FOURNISSEUR where Facture_Id=? and Taux_TVA>0 group by Taux_TVA order by Taux_TVA";

		String reqArtFact = "select * from "+ base +".ARTICLE_FACTURE_FOURNISSEUR where Facture_Id=?";


		PreparedStatement psVentilTvas = con.prepareStatement(reqVentilTvas);
		PreparedStatement psArtFact = con.prepareStatement(reqArtFact);



		String reqFactures = "select * from "+ base +".FACTURE_FOURNISSEUR where Numero>0 and Transferee=0 and Date_Facture<="+ finPeriode +" and Date_Facture>="+ debutPeriode
											 + " and Total_HT>0"+ (fournisseur.equals("")?"":" and Fournisseur_Id='"+ fournisseur +"'") +" order by Num_Entier";

		rset = stt.executeQuery(reqFactures);


		while (rset.next()) {

			long echeance = rset.getLong("Echeance");
			long dateFacture = rset.getLong("Date_Facture");

			int factureId = rset.getInt("Facture_Id");

			String fournisseurId = rset.getString("Fournisseur_Id");
			String numPiece = rset.getString("Num_Entier");
			String libelle = rset.getString("Raison_Sociale");
			String typeAchat = rset.getString("Type_Achat");

			double tRemise = rset.getDouble("Remise");
			double tEscompte = rset.getDouble("Escompte");
			double fraisPort = rset.getDouble("Frais_Port");
			double tauxTvaPort = rset.getDouble("Taux_TVA_Port");


			DateTime dt = new DateTime(dateFacture);
			dt.setDay(1);
			long dateEcriture = dt.getDateInMillis();


			Ecriture ecriture = new Ecriture(codeJournal, dateEcriture);

			HashMap<String, Double> vtvas = new HashMap<String, Double>(2);
			HashMap<String, Double> vachats = new HashMap<String, Double>(2);


			// ventilation des comptes de TVA et calcul du total TVA

			double totalTVA = 0;

			if (fraisPort>0 && tauxTvaPort>0) {

				String compteTVA = cptesTva.get(new Double(tauxTvaPort));
				double montantTVA = rd.round(fraisPort * tauxTvaPort/100);

				totalTVA += montantTVA;

				vtvas.put(compteTVA, new Double(montantTVA));
			}


			psVentilTvas.setInt(1, factureId);
			ResultSet rsVT = psVentilTvas.executeQuery();

			while (rsVT.next()) {

				double tauxTVA = rsVT.getDouble("Taux_TVA");
				double montantTVA = rd.round(rsVT.getDouble("Montant_TVA") * (1-tRemise/100));

				String compteTVA = cptesTva.get(new Double(tauxTVA));

				totalTVA += montantTVA;

				if (vtvas.containsKey(compteTVA)) {
					montantTVA += vtvas.get(compteTVA).doubleValue();
				}

				vtvas.put(compteTVA , new Double(rd.round(montantTVA)));
			}


			// parcours des articles de la facture pour calculer la ventilation des comptes d'achat

			double htFacture = 0;

			psArtFact.setInt(1, factureId);
      ResultSet rsAF = psArtFact.executeQuery();

      while (rsAF.next()) {

				String reference = rsAF.getString("Reference");
				String typeLigne = rsAF.getString("Type_Ligne");

				double quantite = rsAF.getDouble("Quantite");
				double puHT = rsAF.getDouble("Prix");
				double ristourne = rsAF.getDouble("Ristourne");
				double tauxTVA = rsAF.getDouble("Taux_TVA");


				String compteHT = (typeLigne.equalsIgnoreCase("S")?getCompteHT(reference, typeAchat, new Double(tauxTVA)):getCompteDefault(typeAchat, new Double(tauxTVA)));

				double montantHT = rd.round(quantite * puHT * (1-ristourne/100));

				htFacture += montantHT;

        if (vachats.containsKey(compteHT)) {
					montantHT += vachats.get(compteHT).doubleValue();
				}

				vachats.put(compteHT, new Double(rd.round(montantHT)));
			}


			// application du pourcentage de remise globale aux comptes d'achat et calcul du total HT
			// la remise globale est applique maintenant pour viter au maximum les carts

			double totalHT = fraisPort;

			Iterator li = vachats.keySet().iterator();

			while (li.hasNext()) {

				String compteHT = (String)li.next();

				double valeurCpte = vachats.get(compteHT).doubleValue();

				double valeurRem = rd.round(valeurCpte * (1-tRemise/100));

				totalHT += valeurRem;

				if (valeurRem>0)
					vachats.put(compteHT, new Double(valeurRem));
				else
					vachats.put(compteHT, new Double(0));
			}


			htFacture = rd.round(htFacture * (1-tRemise/100) + fraisPort);

			System.out.println("HT Facture = "+ htFacture);
			System.out.println("Somme HT Ventilee = "+ totalHT);

			// rajustement de la ventilation HT  partir du dernier compte HT ventil

			if (htFacture>totalHT) {
				ajusterHT(rd.round(htFacture-totalHT), 'P', vachats);
			}
			else if (htFacture<totalHT) {
				ajusterHT(rd.round(totalHT-htFacture), 'M', vachats);
			}


			double montantTTC = htFacture + totalTVA;


			// ajout des oprations  l'ecriture comptable

			// operations sur comptes d'achat

			li = vachats.keySet().iterator();

			while (li.hasNext()) {

				String compteHT = (String)li.next();
				double montantHT = vachats.get(compteHT).doubleValue();

				System.out.println("Montant HT : "+ montantHT);

				if (montantHT>0) {
					Operation op = new Operation();
					op.setDateOp(dateFacture);
					op.setNumPiece(numPiece);
					op.setDateEcheance(echeance);

					op.setNumeroCompte(compteHT);
					op.setLibelle(getIntitule(compteHT));
					op.setMontantD(montantHT);

					ecriture.addOp(op);
				}
			}

			// affectation des frais de port au compte de frais de livraison

			if (fraisPort>0) {

				Operation op = new Operation();
				op.setDateOp(dateFacture);
				op.setNumPiece(numPiece);
				op.setDateEcheance(echeance);

				op.setNumeroCompte(compteLivraison);
				op.setLibelle(getIntitule(compteLivraison));
				op.setMontantD(fraisPort);

				ecriture.addOp(op);
			}

			// operations sur comptes de TVA

			li = vtvas.keySet().iterator();

			while (li.hasNext()) {

				String compteTVA = (String)li.next();
				double montantTVA = vtvas.get(compteTVA).doubleValue();

				if (montantTVA>0) {
					Operation op = new Operation();
					op.setDateOp(dateFacture);
					op.setNumPiece(numPiece);
					op.setDateEcheance(echeance);

					op.setNumeroCompte(compteTVA);
					op.setLibelle(getIntitule(compteTVA));
					op.setMontantD(montantTVA);

					ecriture.addOp(op);
				}
			}


			// operation d'escompte

			double montantEscompte = rd.round(montantTTC * (tEscompte/100));

			if (montantEscompte>0) {
				Operation op = new Operation();
				op.setDateOp(dateFacture);
				op.setNumPiece(numPiece);
				op.setDateEcheance(echeance);

				op.setNumeroCompte(compteEscompteAC);
				op.setLibelle(getIntitule(compteEscompteAC));
				op.setMontantC(montantEscompte);

				ecriture.addOp(op);
			}


			// operation du compte fournisseur

			double montantNet = rd.round(montantTTC - montantEscompte);

			if (montantNet>0) {
				String compteFournisseur = getCompteFournisseur(fournisseurId);

				Operation op = new Operation();
				op.setDateOp(dateFacture);
				op.setNumPiece(numPiece);
				op.setDateEcheance(echeance);

				op.setNumeroCompte(compteFournisseur);
				op.setLibelle(getIntitule(compteFournisseur));
				op.setMontantC(montantNet);

				ecriture.addOp(op);
			}


			// insertion en base de l'ecriture comptable ssi quilibre

			if (ecriture.isStable()) {
				psBaseCompta.setLong(1, dateFacture);
				psBaseCompta.setLong(2, dateFacture);
				ResultSet rsExo = psBaseCompta.executeQuery();
				if (rsExo.next()) {
      		ecriture.insertDB(con, rsExo.getString("Nom_Base"), base);
					marquerFacture(factureId);
				}
      }
			else {
				System.out.println("\nTRANSFERT DES ACHATS : ECRITURE DESEQUILIBREE\n");
				ecriture.printEcriture();
				System.out.println("Montant Net --> "+ montantNet);
			}

		} // fin boucle factures




		// GENERATION DES ECRITURES D'AVOIRS

		reqVentilTvas = "select Taux_TVA, sum(Prix * Quantite) * Taux_TVA/100 as Montant_TVA "
											 + "from "+ base +".ARTICLE_AVOIR_FOURNISSEUR where Avoir_Id=? and Taux_TVA>0 group by Taux_TVA order by Taux_TVA";

		String reqArtAvoir = "select * from "+ base +".ARTICLE_AVOIR_FOURNISSEUR where Avoir_Id=?";


		psVentilTvas = con.prepareStatement(reqVentilTvas);
		PreparedStatement psArtAvoir = con.prepareStatement(reqArtAvoir);


		String reqAvoirs = "select * from "+ base +".AVOIR_FOURNISSEUR where Numero>0 and Transfere=0 and Date_Avoir<="+ finPeriode +" and Date_Avoir>="+ debutPeriode
										 + " and Total_HT>0"+ (fournisseur.equals("")?"":" and Fournisseur_Id='"+ fournisseur +"'") +" order by Num_Entier";

		rset = stt.executeQuery(reqAvoirs);


		while (rset.next()) {

			long dateAvoir = rset.getLong("Date_Avoir");

			int avoirId = rset.getInt("Avoir_Id");

			String fournisseurId = rset.getString("Fournisseur_Id");
			String numPiece = rset.getString("Num_Entier");
			String libelle = rset.getString("Raison_Sociale");
			String typeAchat = rset.getString("Type_Achat");

			//double tRemise = rset.getDouble("Remise");
			//double tEscompte = rset.getDouble("Escompte");
			//double fraisPort = rset.getDouble("Frais_Port");
			//double tauxTvaPort = rset.getDouble("Taux_TVA_Port");


			DateTime dt = new DateTime(dateAvoir);
			dt.setDay(1);
			long dateEcriture = dt.getDateInMillis();


			Ecriture ecriture = new Ecriture(codeJournal, dateEcriture);

			HashMap<String, Double> vtvas = new HashMap<String, Double>(2);
			HashMap<String, Double> vachats = new HashMap<String, Double>(2);


			// ventilation des comptes de TVA et calcul du total TVA

			double totalTVA = 0;

			/*if (fraisPort>0 && tauxTvaPort>0) {

				String compteTVA = cptesTva.get(new Double(tauxTvaPort));
				double montantTVA = rd.round(fraisPort * tauxTvaPort/100);

				totalTVA += montantTVA;

				vtvas.put(compteTVA, new Double(montantTVA));
			}*/


			psVentilTvas.setInt(1, avoirId);
			ResultSet rsVT = psVentilTvas.executeQuery();

			while (rsVT.next()) {

				double tauxTVA = rsVT.getDouble("Taux_TVA");
				double montantTVA = rd.round(rsVT.getDouble("Montant_TVA"));// * (1-tRemise/100));

				String compteTVA = cptesTva.get(new Double(tauxTVA));

				totalTVA += montantTVA;

				if (vtvas.containsKey(compteTVA)) {
					montantTVA += vtvas.get(compteTVA).doubleValue();
				}

				vtvas.put(compteTVA , new Double(rd.round(montantTVA)));
			}


			// parcours des articles de la facture pour calculer la ventilation des comptes d'achat

			double htAvoir = 0;

			psArtAvoir.setInt(1, avoirId);
      ResultSet rsAF = psArtAvoir.executeQuery();

      while (rsAF.next()) {

				String reference = rsAF.getString("Reference");
				String typeLigne = rsAF.getString("Type_Ligne");

				double quantite = rsAF.getDouble("Quantite");
				double puHT = rsAF.getDouble("Prix");
				//double ristourne = rsAF.getDouble("Ristourne");
				double tauxTVA = rsAF.getDouble("Taux_TVA");


				String compteHT = (typeLigne.equalsIgnoreCase("S")?getCompteHT(reference, typeAchat, new Double(tauxTVA)):getCompteDefault(typeAchat, new Double(tauxTVA)));

				double montantHT = rd.round(quantite * puHT);// * (1-ristourne/100));

				htAvoir += montantHT;

        if (vachats.containsKey(compteHT)) {
					montantHT += vachats.get(compteHT).doubleValue();
				}

				vachats.put(compteHT, new Double(rd.round(montantHT)));
			}


			// application du pourcentage de remise globale aux comptes d'achat et calcul du total HT
			// la remise globale est applique maintenant pour viter au maximum les carts

			double totalHT = 0;//fraisPort;

			Iterator li = vachats.keySet().iterator();

			while (li.hasNext()) {

				String compteHT = (String)li.next();

				double valeurCpte = vachats.get(compteHT).doubleValue();

				double valeurRem = rd.round(valeurCpte);// * (1-tRemise/100));

				totalHT += valeurRem;

				if (valeurRem>0)
					vachats.put(compteHT, new Double(valeurRem));
				else
					vachats.put(compteHT, new Double(0));
			}


			htAvoir = rd.round(htAvoir);// * (1-tRemise/100) + fraisPort);

			// rajustement de la ventilation HT  partir du dernier compte HT ventil

			if (htAvoir>totalHT) {
				ajusterHT(rd.round(htAvoir-totalHT), 'P', vachats);
			}
			else if (htAvoir<totalHT) {
				ajusterHT(rd.round(totalHT-htAvoir), 'M', vachats);
			}


			double montantTTC = htAvoir + totalTVA;


			// ajout des oprations  l'ecriture comptable

			// operations sur comptes d'achat

			li = vachats.keySet().iterator();

			while (li.hasNext()) {

				String compteHT = (String)li.next();
				double montantHT = vachats.get(compteHT).doubleValue();

				if (montantHT>0) {
					Operation op = new Operation();
					op.setDateOp(dateAvoir);
					op.setNumPiece(numPiece);

					op.setNumeroCompte(compteHT);
					op.setLibelle(getIntitule(compteHT));
					op.setMontantC(montantHT);

					ecriture.addOp(op);
				}
			}

			// affectation des frais de port au compte de frais de livraison

			/*if (fraisPort>0) {

				Operation op = new Operation();
				op.setDateOp(dateAvoir);
				op.setNumPiece(numPiece);

				op.setNumeroCompte(compteLivraison);
				op.setLibelle(getIntitule(compteLivraison));
				op.setMontantC(fraisPort);

				ecriture.addOp(op);
			}*/

			// operations sur comptes de TVA

			li = vtvas.keySet().iterator();

			while (li.hasNext()) {

				String compteTVA = (String)li.next();
				double montantTVA = vtvas.get(compteTVA).doubleValue();

				if (montantTVA>0) {
					Operation op = new Operation();
					op.setDateOp(dateAvoir);
					op.setNumPiece(numPiece);

					op.setNumeroCompte(compteTVA);
					op.setLibelle(getIntitule(compteTVA));
					op.setMontantC(montantTVA);

					ecriture.addOp(op);
				}
			}


			// operation d'escompte

			/*double montantEscompte = rd.round(montantTTC * (tEscompte/100));

			if (montantEscompte>0) {
				Operation op = new Operation();
				op.setDateOp(dateAvoir);
				op.setNumPiece(numPiece);

				op.setNumeroCompte(compteEscompteAC);
				op.setLibelle(getIntitule(compteEscompteAC));
				op.setMontantD(montantEscompte);

				ecriture.addOp(op);
			}*/


			// operation du compte fournisseur

			double montantNet = rd.round(montantTTC);// - montantEscompte);

			if (montantNet>0) {
				String compteFournisseur = getCompteFournisseur(fournisseurId);

				Operation op = new Operation();
				op.setDateOp(dateAvoir);
				op.setNumPiece(numPiece);

				op.setNumeroCompte(compteFournisseur);
				op.setLibelle(getIntitule(compteFournisseur));
				op.setMontantD(montantNet);

				ecriture.addOp(op);
			}


			// insertion en base de l'ecriture comptable ssi quilibre

			if (ecriture.isStable()) {
				psBaseCompta.setLong(1, dateAvoir);
				psBaseCompta.setLong(2, dateAvoir);
				ResultSet rsExo = psBaseCompta.executeQuery();
				if (rsExo.next()) {
      		ecriture.insertDB(con, rsExo.getString("Nom_Base"), base);
					marquerAvoir(avoirId);
				}
      }
			else {
				System.out.println("\nTRANSFERT DES ACHATS : ECRITURE DESEQUILIBREE\n");
				ecriture.printEcriture();
			}

		} // fin  boucle avoirs

		s.closeConnection(con, null);

		return true;
  }





	private void marquerFacture(int factureId) throws SQLException {

		psMarqueFact.setInt(1, factureId);
		psMarqueFact.executeUpdate();
	}


	private void marquerAvoir(int avoirId) throws SQLException {

		psMarqueAvoir.setInt(1, avoirId);
		psMarqueAvoir.executeUpdate();
	}


	private void ajusterHT(double montant, char sens, HashMap<String, Double> vachats) throws SQLException {

		Iterator li = vachats.keySet().iterator();

		while (li.hasNext() && montant>0) {

			String compteHT = (String)li.next();

			double montantHT = vachats.get(compteHT).doubleValue();

			if (sens=='P') {
				montantHT += montant;
				montant = 0;
			}
			else {
				montantHT = (montantHT>montant?montantHT - montant:0);
				montant -= montantHT;
			}

			if (montantHT>0)
				vachats.put(compteHT, new Double(montantHT));
			else
				vachats.put(compteHT, new Double(0));
		}
	}


	private String getIntitule(String numCompte) throws SQLException {

		psIntCpte.setString(1, numCompte);
		ResultSet rset = psIntCpte.executeQuery();
		return rset.next()?rset.getString("Intitule"):"-";
	}


	private String getCompteFournisseur(String fournisseurId) throws SQLException {

		if (fournisseurId.equals("")) {
			return compteFournisseursDivers;
		}
		else {
			psCpteFournisseur.setString(1, fournisseurId);
			ResultSet rset = psCpteFournisseur.executeQuery();

    	if (rset.next()) {
      	return rset.getString("Numero_Compte");
    	}
			else {
				return compteFournisseursDivers;
			}
		}
	}


  private String getCompteFamilleI(String reference) throws SQLException {

    psCpteFamilleI.setString(1, reference);
    ResultSet rset = psCpteFamilleI.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat_I");
    }
		else {
			return compteAchatDefaultI;
		}
  }


	private String getCompteFamilleUE(String reference) throws SQLException {

    psCpteFamilleUE.setString(1, reference);
    ResultSet rset = psCpteFamilleUE.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat_UE");
    }
		else {
			return compteAchatDefaultUE;
		}
  }


	private String getCompteFamilleN(String reference, Double tauxTVA) throws SQLException {

    psCpteFamilleN.setString(1, reference);
		psCpteFamilleN.setDouble(2, tauxTVA);
    ResultSet rset = psCpteFamilleN.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat");
    }
		else {
			return cptesATva.get(tauxTVA);
		}
  }


  private String getCompteSousFamilleI(String reference) throws SQLException {

    psCpteSousFamilleI.setString(1, reference);
    ResultSet rset = psCpteSousFamilleI.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat_I");
    }
		else {
			return getCompteFamilleI(reference);
		}
  }


	private String getCompteSousFamilleUE(String reference) throws SQLException {

    psCpteSousFamilleUE.setString(1, reference);
    ResultSet rset = psCpteSousFamilleUE.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat_UE");
    }
		else {
			return getCompteFamilleUE(reference);
		}
  }


	private String getCompteSousFamilleN(String reference, Double tauxTVA) throws SQLException {

    psCpteSousFamilleN.setString(1, reference);
		psCpteSousFamilleN.setDouble(2, tauxTVA);
    ResultSet rset = psCpteSousFamilleN.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat");
    }
		else {
			return getCompteFamilleN(reference, tauxTVA);
		}
  }


  private String getCompteArticleI(String reference) throws SQLException {

		psCpteArticleI.setString(1, reference);
    ResultSet rset = psCpteArticleI.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat_I");
    }
		else {
			return getCompteSousFamilleI(reference);
		}
  }


	private String getCompteArticleUE(String reference) throws SQLException {

		psCpteArticleUE.setString(1, reference);
    ResultSet rset = psCpteArticleUE.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat_UE");
    }
		else {
			return getCompteSousFamilleUE(reference);
		}
  }


	private String getCompteArticleN(String reference, Double tauxTVA) throws SQLException {

		psCpteArticleN.setString(1, reference);
		psCpteArticleN.setDouble(2, tauxTVA);
    ResultSet rset = psCpteArticleN.executeQuery();

    if (rset.next()) {
      return rset.getString("Compte_Achat");
    }
		else {
			return getCompteSousFamilleN(reference, tauxTVA);
		}
  }


	private String getCompteHT(String reference, String typeAchat, Double tauxTVA) throws SQLException {

		if (typeAchat.equalsIgnoreCase("I")) {
			return getCompteArticleI(reference);
		}
		else if (typeAchat.equalsIgnoreCase("U")) {
			return getCompteArticleUE(reference);
		}
		else {
			return getCompteArticleN(reference, tauxTVA);
		}
	}


	private String getCompteDefault(String typeAchat, Double tauxTVA) {

		if (typeAchat.equalsIgnoreCase("I")) {
			return compteAchatDefaultI;
		}
		else if (typeAchat.equalsIgnoreCase("U")) {
			return compteAchatDefaultUE;
		}
		else {
			return cptesATva.get(tauxTVA);
		}
	}


} // fin TransfertAchats.java

