/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *   Export d'critures comptables
 */

package org.opensi.facturation.actions.transferts;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DecimalFormatSymbols;
import java.util.Calendar;

import org.experlog.openeas.api.Action;
import org.experlog.openeas.api.ServletRequest;
import org.experlog.openeas.api.Session;


public class ExportExpe extends Action {


	private long debutPeriode;
	private long finPeriode;
	private String base;
	private String periode;
	private String [] expediteur;
	private String [] heure;
	private boolean colis;
	private boolean web;
	private boolean affaire;
	
	
	private DecimalFormatSymbols dfs = new DecimalFormatSymbols();	


  public boolean serverSide() { return true; }

  public boolean service(Session s, ServletRequest r, Object info) throws Exception {


		dfs.setDecimalSeparator('.');

		this.base = s.getCookie().get("BaseDossier");
		this.periode = r.getParameter("Periode");
		this.expediteur = r.getParameter("Expediteur").split(",");
		this.heure = r.getParameter("Heure").split(":");
		this.colis = r.getParameter("Colis").equals("true");
		this.web = r.getParameter("Web").equals("true");
		this.affaire = r.getParameter("Affaire").equals("true");
		String dateDebut = r.getParameter("Date_Debut");
		String dateFin = r.getParameter("Date_Fin");

		Calendar calendar = Calendar.getInstance();
		calendar.clear();


		if (periode.equals("dd")) {
			calendar.set(2000+Integer.parseInt(dateDebut.substring(6,8)), Integer.parseInt(dateDebut.substring(3,5))-1, Integer.parseInt(dateDebut.substring(0,2)),0,0,0);
			debutPeriode = calendar.getTimeInMillis();
			calendar.set(2000+Integer.parseInt(dateFin.substring(6,8)), Integer.parseInt(dateFin.substring(3,5))-1, Integer.parseInt(dateFin.substring(0,2))+1,0,0,0);
			finPeriode = calendar.getTimeInMillis();
		}
		else if (periode.equals("A")) {
			calendar = Calendar.getInstance();
			calendar.set(Calendar.HOUR_OF_DAY,Integer.parseInt(heure[0]));
			calendar.set(Calendar.MINUTE,Integer.parseInt(heure[1]));
			debutPeriode = calendar.getTimeInMillis();
		}
		else if (periode.equals("M")) {
			calendar = Calendar.getInstance();
			calendar.set(Calendar.HOUR_OF_DAY,0);
			calendar.set(Calendar.DAY_OF_MONTH,1);
			debutPeriode = calendar.getTimeInMillis();
		}

		String fichier = s.getAppConfiguration().get("opensi.absdir") +"/iobuffer/test_export.test";
		
		FileWriter fw = new FileWriter(fichier);
		
		BufferedWriter bw = new BufferedWriter(fw);
		
		Connection con = s.getConnection(null);

		makeExpedition(bw,con);
		
		bw.close();
		
		s.closeConnection(con, null);
		
		r.addParameter("FichierExport", "test_export.test");
		r.addParameter("ActionError", "");
		
		try {
			File file = new File(fichier);
			file.deleteOnExit();
		}
		catch(SecurityException se) {
			s.trace("Impossible d'enregistrer le fichier  supprimer");
			se.printStackTrace();
		}
		
		return true;
			
	}
	
	private void makeExpedition(BufferedWriter bw, Connection con) throws Exception {
		
		Statement stt = con.createStatement();		
		
		String bons = "select bl.*, p.Nom_FR";
		bons +=" from "+ base +".BON_LIVRAISON bl, PAYS p";
		bons+=" where bl.Code_Pays_Liv=p.Code_Pays and ";
		if (periode.equals("A") || periode.equals("M")) {
			bons+=" bl.Date_Liv>="+debutPeriode+" and ";
		}
		else if (periode.equals("dd")) {
			bons+=" bl.Date_Liv>="+debutPeriode+" and bl.Date_Liv<"+finPeriode+" and ";
		}

		if (web && !affaire) {
			bons+=" bl.Com_Web_id!=0 and ";			
		}
		else if (affaire && !web) {
			bons+=" bl.Com_Web_id=0 and ";			
		}
		if (colis) {
			bons+=" bl.Bon_Id not in (select Bon_Id from "+ base +".COLIS) and ";						
		}
		bons+=" bl.Num_Entier!='non gnr' and bl.Etat='V' order by bl.Num_Entier";

		ResultSet rset = stt.executeQuery(bons);
		String requete="select * from " + base + ".COMMANDE_WEB where Com_Web_Id=?";
		if (expediteur.length>0) {
			requete+="  and (Mode_Expedition='"+expediteur[0]+"'";
			for (int i=1;i<expediteur.length;i++) {
				requete+=" or Mode_Expedition='"+expediteur[i]+"'";
			}
		}
		requete+=")";

		PreparedStatement psInfosSup = con.prepareStatement(requete);
		
		while (rset.next()) {

			if (rset.getInt("Com_Web_Id")>0) {
				psInfosSup.setString(1, rset.getString("Com_Web_Id"));
				ResultSet rsIS = psInfosSup.executeQuery();
	
				if (rsIS.next()) {	
					bw.write("\""+rset.getString("Num_Entier")+"\";\"");
					bw.write("\";\"");
					bw.write(rset.getString("Nom_Liv")+"\";\"");
					bw.write(rset.getString("Adresse_Liv")+"\";\"");
					bw.write(rset.getString("Comp_Adresse_Liv")+"\";\"");			
					bw.write(rset.getString("Adresse_3_Liv")+"\";\"");
					bw.write(rset.getString("CP_Liv")+"\";\"");
				
					bw.write(rset.getString("Ville_Liv")+"\";\"");			
					bw.write(rset.getString("Nom_FR")+"\"");					
					bw.newLine();
				}
			}
			else {
				bw.write("\""+rset.getString("Num_Entier")+"\";\"");
				bw.write("\";\"");
				bw.write(rset.getString("Nom_Liv")+"\";\"");
				bw.write(rset.getString("Adresse_Liv")+"\";\"");
				bw.write(rset.getString("Comp_Adresse_Liv")+"\";\"");			
				bw.write(rset.getString("Adresse_3_Liv")+"\";\"");
				bw.write(rset.getString("CP_Liv")+"\";\"");			

				bw.write(rset.getString("Ville_Liv")+"\";\"");			
				bw.write(rset.getString("Nom_FR")+"\"");
				bw.newLine();
			}
		}
		bw.flush();
		stt.close();
	}
} // fin ExportCompta
