/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *   Export des expditions
 */

package org.opensi.facturation.expedition;

import org.experlog.openeas.api.Session;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;


public class DataExpedition {

	// les lignes de l'export
	protected ArrayList<LigneExpedition> lignes = new ArrayList<LigneExpedition>(10);
	
	protected String [] expediteur;
	protected boolean colis;
	protected boolean web;
	protected boolean affaire;
	protected String refCommande;
	protected String clientId;
	protected String civInterLiv;
	protected String nomInterLiv;
	protected String telInterLiv;
	protected String emailInterLiv;
	protected String codePorteLiv;
	protected String codePaysLiv;
	protected double poids;	
	protected String compteClient;
	protected String codeProduit;
	
	protected boolean relaisColis;
	protected String numRelaisColis;
	protected String nomRelaisColis;
	protected String nomPrenomDestFinal;
	protected String numTelDestFinal;	
	
	
	public DataExpedition(Session s, String periode, String numClient, long debutPeriode, long finPeriode, String[] expediteur, boolean colis, boolean web, boolean affaire) {
		
		try {
			Connection con = s.getConnection(null);
			Statement stt = con.createStatement();

			String base = s.getCookie().get("BaseDossier");
			this.compteClient=numClient;
			
			this.expediteur = expediteur;
			this.colis = colis;
			this.web = web;
			this.affaire = affaire;
			
			
			String bons = "select bl.*, p.Nom_FR from "+ base +".BON_LIVRAISON bl, PAYS p where bl.Code_Pays_Liv=p.Code_Pays and bl.Commande_Id=0 and";
			if (periode.equals("A") || periode.equals("M")) {
				bons+=" bl.Date_Liv>="+debutPeriode+" and ";
			} else if (periode.equals("dd")) {
				bons+=" bl.Date_Liv>="+debutPeriode+" and bl.Date_Liv<"+finPeriode+" and ";
			}
			if (web && !affaire) {
				bons+=" bl.Com_Web_id!=0 and ";			
			} else if (affaire && !web) {
				bons+=" bl.Com_Web_id=0 and ";			
			}
			if (colis) {
				bons+=" bl.Bon_Id not in (select Bon_Id from "+ base +".COLIS) and ";						
			}
			bons+=" bl.Num_Entier!='non gnr' and bl.Etat='V' order by bl.Num_Entier";
			ResultSet rset = stt.executeQuery(bons);
			
			String requeteCW="select c.Civ_Courte as Civ_Courte_Liv, cw.Com_Web_Id, cw.Com_Site_Id, cw.Client_Web_Id, cw.Tel_Inter_Liv, cw.Email_Inter_Liv, cw.Code_Porte, cw.Code_Pays_Liv, cw.Mode_Expedition, cw.Nom_Inter_Liv, cw.Societe_Liv, cw.Societe_Client, cw.Nom_Inter, cw.Prenom_Inter, cw.Tel_Inter from " + base + ".COMMANDE_WEB cw, CIVILITE c where cw.Civ_Inter_Liv=c.Civ_Id and cw.Com_Web_Id=?";
			if (expediteur.length>0) {
				requeteCW+="  and (cw.Mode_Expedition='"+expediteur[0]+"'";
				for (int i=1;i<expediteur.length;i++) {
					requeteCW+=" or cw.Mode_Expedition='"+expediteur[i]+"'";
				}
				requeteCW+=")";
			}
			PreparedStatement psInfosSupCW = con.prepareStatement(requeteCW);
			
			String requeteA="select c.Civ_Courte as Civ_Courte_Liv, a.Affaire_Id, a.Num_Entier, a.Client_Id, u.Login as Login_Createur, a.Code_Pays_Liv, a.Tel_Inter_Liv, a.Email_Inter_Liv, a.Nom_Inter_Liv from " + base + ".AFFAIRE a, CIVILITE c, UTILISATEUR u where a.Util_C=u.Utilisateur_Id and a.Civ_Inter_Liv=c.Civ_Id and a.Affaire_Id=?";
			PreparedStatement psInfosSupA = con.prepareStatement(requeteA);
			
			String requeteABLCW="select sum(f.Poids_Brut* abl.Quantite) as poids from "+ base +".FICHE_ARTICLE f, "+ base +".ARTICLE_BON_LIVRAISON abl ";
			requeteABLCW += ", "+ base +".ARTICLE_COMMANDE_WEB acw ";
			requeteABLCW += "where ";
			requeteABLCW += "f.Article_Id = acw.Article_Id and acw.Detail_Id = abl.Ref_Ligne_Web and Type_Ligne='S' and ";
			requeteABLCW += "abl.Bon_Id=?";
			
			String requeteABLA="select sum(f.Poids_Brut* abl.Quantite) as poids from "+ base +".FICHE_ARTICLE f, "+ base +".ARTICLE_BON_LIVRAISON abl ";
			requeteABLA += ", "+ base +".ARTICLE_COM_CLIENT acc ";
			requeteABLA += "where ";
			requeteABLA += "f.Article_Id = acc.Reference and Type_Ligne='S' and acc.Ligne_Id = abl.Ref_Ligne and ";
			requeteABLA += "abl.Bon_Id=?";
			
			PreparedStatement psInfosSupABLCW = con.prepareStatement(requeteABLCW);
			PreparedStatement psInfosSupABLA = con.prepareStatement(requeteABLA);
			
			while (rset.next()) {
				this.relaisColis = false;
				this.numRelaisColis = "";
				this.nomRelaisColis = "";
				this.nomPrenomDestFinal = "";
				this.numTelDestFinal = "";
				
				if (rset.getInt("Com_Web_Id")>0) {
					
					psInfosSupCW.setString(1, rset.getString("Com_Web_Id"));
					ResultSet rsISCW = psInfosSupCW.executeQuery();
		
					if (rsISCW.next()) {
						this.poids=0;
						psInfosSupABLCW.setInt(1, rset.getInt("Bon_Id"));
						ResultSet rsISABLCW = psInfosSupABLCW.executeQuery();
						if (rsISABLCW.next()) {
							this.poids = rsISABLCW.getDouble("poids");
						}
						
						this.refCommande = rsISCW.getString("Com_Site_Id");
						this.clientId = rsISCW.getString("Client_Web_Id");
						this.emailInterLiv = rsISCW.getString("Email_Inter_Liv");
						this.codePorteLiv = rsISCW.getString("Code_Porte");
						this.codePaysLiv = rsISCW.getString("Code_Pays_Liv");
						this.relaisColis = (rsISCW.getString("Mode_Expedition").equals("TNTRelais"));
						this.codeProduit = (this.relaisColis?"JD":"J");
						this.civInterLiv = rsISCW.getString("Civ_Courte_Liv");
						this.nomInterLiv = rsISCW.getString("Nom_Inter_Liv");
						this.telInterLiv = rsISCW.getString("Tel_Inter_Liv");
						this.numRelaisColis = (this.relaisColis?formaterChaine(rsISCW.getString("Nom_Inter_Liv"),5):"");
						this.nomRelaisColis = (this.relaisColis?rsISCW.getString("Societe_Liv"):"");
						if (this.relaisColis) {
							if (!rsISCW.getString("Nom_Inter").equals("")) {
								this.nomPrenomDestFinal = formaterChaine(rsISCW.getString("Nom_Inter"),17) + " " + formaterChaine(rsISCW.getString("Prenom_Inter"),12);
							} else {
								this.nomPrenomDestFinal = formaterChaine(rsISCW.getString("Societe_Client"),17);
							}
						} else { this.nomPrenomDestFinal = ""; }
						this.numTelDestFinal = (this.relaisColis?formaterTel(rsISCW.getString("Tel_Inter")):"");
						this.ajouterLigne(rset);
					}
				}
				else {
					this.poids=0;
					psInfosSupABLA.setInt(1, rset.getInt("Bon_Id"));
					ResultSet rsISABLA = psInfosSupABLA.executeQuery();
					if (rsISABLA.next()) {
						this.poids = rsISABLA.getDouble("poids");
					}
					
					psInfosSupA.setString(1, rset.getString("Affaire_Id"));
					ResultSet rsISA = psInfosSupA.executeQuery();
					rsISA.next();
					
					this.refCommande = rsISA.getString("Num_Entier");
					this.clientId = rsISA.getString("Client_Id");
					this.civInterLiv = rsISA.getString("Civ_Courte_Liv");
					this.nomInterLiv = rsISA.getString("Nom_Inter_Liv");
					this.telInterLiv = rsISA.getString("Tel_Inter_Liv");
					this.emailInterLiv = rsISA.getString("Email_Inter_Liv");
					this.codePorteLiv = "";
					this.codePaysLiv = rsISA.getString("Code_Pays_Liv");
					this.codeProduit = "J";
					this.ajouterLigne(rset);
				}
			}
			
			rset.close();
			stt.close();
			s.closeConnection(con, null);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void ajouterLigne(ResultSet rset) {
		try {
			LigneExpedition le = new LigneExpedition();
			
			le.setNumEntier(rset.getString("Num_Entier"));
			le.setNomLiv(this.relaisColis?"RELAIS COLIS":rset.getString("Nom_Liv"));
			le.setAdresseLiv(this.relaisColis?this.nomRelaisColis:rset.getString("Adresse_Liv"));
			le.setCompAdresseLiv(this.relaisColis?rset.getString("Adresse_Liv"):rset.getString("Comp_Adresse_Liv"));
			le.setAdresse3Liv(rset.getString("Adresse_3_Liv"));
			le.setCPLiv(rset.getString("CP_Liv"));
			le.setVilleLiv(rset.getString("Ville_Liv"));
			le.setPaysLiv(rset.getString("Nom_FR"));
			le.setCivInterLiv(this.civInterLiv);
			le.setNomInterLiv(this.nomInterLiv);
			le.setTelInterLiv(this.telInterLiv);
			le.setEmailInterLiv(this.emailInterLiv);
			le.setCodePorteLiv(this.codePorteLiv);
			le.setInfosLiv("");
			le.setCompteClient(this.compteClient);
			le.setTypeExpedition('C');
			le.setPoids(this.poids<0.1?0.1:this.poids<999.9?this.poids:999.9);
			le.setRefClient(rset.getString("Num_Entier"));
			le.setRefCommande(this.refCommande);
			le.setValeurDeclaree(0); // doit tre  0 si relais colis
			le.setNbColis(1);
			le.setCodeDestinataire(this.relaisColis?this.numRelaisColis:this.clientId);			
			le.setCodePaysLiv(this.codePaysLiv);
			le.setCodeProduit(this.codeProduit);
			le.setInstruction(this.relaisColis?this.nomPrenomDestFinal:"");
			le.setCompInstruction(this.relaisColis?this.numTelDestFinal:"");
			le.setZoneUtilisateur(formaterChaine(this.civInterLiv + " " +this.nomInterLiv,13) + "  " + formaterTel(this.telInterLiv));
			le.setMontantReglement(0);
			le.setDateDepartColis("");
			le.setNomExpediteur("");
			le.setAdresseExpediteur("");
			le.setCompAdresseExpediteur("");
			le.setCPExpediteur("");
			le.setVilleExpediteur("");
			le.setCodePaysExpediteur("");			
			
			lignes.add(le);
		} catch(Exception e) {
			e.printStackTrace();
		}		
	}
	
	public String formaterChaine(String chaine, int longueur_max) {
		String nouv_chaine = chaine;
		if (nouv_chaine.length() < longueur_max) {
			for (int i=nouv_chaine.length(); i<longueur_max; i++)
				nouv_chaine += " ";
		} else {
			nouv_chaine = nouv_chaine.substring(0,longueur_max);
		}
		return nouv_chaine;
	}
	
	public String formaterTel(String chaine) {
		String nouv_chaine = "";

		for (int i=0; i<chaine.length(); i++) {
			if (chaine.charAt(i)>='0' && chaine.charAt(i)<='9') {
				nouv_chaine += chaine.charAt(i);
			}
		}
		
		return formaterChaine(nouv_chaine,10);
	}
	
	public ArrayList<LigneExpedition> getLignes() {
		return lignes;
	}
		
		
	public int getNbLignes() {
		return lignes.size();
	}
	
} // fin DataExpedition
