/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/


package org.opensi.facturation.reglements.clients;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.*;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;

import org.experlog.openeas.api.Action;
import org.experlog.openeas.api.ServletRequest;
import org.experlog.openeas.api.Session;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;


public class GenPdfRembourser extends Action {


	private static final Font fontAdrFacture = new Font(Font.HELVETICA, 11, Font.BOLD);
	private static final Font fontAdrSociete = new Font(Font.HELVETICA, 10, Font.NORMAL);
	private static final Font fontAdrSocieteGras = new Font(Font.HELVETICA, 11, Font.BOLD);
	private static final Font fontWeb = new Font(Font.HELVETICA, 10, Font.ITALIC);
	private static final Font fontLegale = new Font(Font.HELVETICA , 8, Font.NORMAL);
	private static final Font fontTexte = new Font(Font.HELVETICA, 11, Font.NORMAL);

	private DecimalFormat numFormat = new DecimalFormat("0.00");
	private DecimalFormat numFormatCap = new DecimalFormat("0.##");
	private SimpleDateFormat formatDate = new SimpleDateFormat("dd/MM/yyyy");

	private String absdir;
	private String dossierId;
	private String base;

	// le logo contient les coordonn\u00E9es de l'entreprise
	private boolean logoAdr;

	// coordonn\u00E9es et infos l\u00E9gales de l'entreprise
	private String raisonSociale;
	private String adresseSociete1;
	private String adresseSociete2;
	private String codePostal;
	private String ville;
	private String tel;
	private String fax;
	private String email;
	private String web;
	private String siren;
	private String typeSociete;
	private double capital;
	private String tvaIntra;
	private String rcs;

	private String Reglement_Id;
	private String Avoir_Id;
	private boolean reglement;
	private String Motif;
	private long Date;
	private String Texte;
	private String Mode_Reg;
	private String montantaff;

	private PreparedStatement psReglement;
	private PreparedStatement psAvoir;
	private PreparedStatement psDestinataireAvoir;
	private PreparedStatement psDestinataireReglement;
	private PreparedStatement psProvFact;

	public boolean serverSide() { return true; }

	public boolean service(Session s, ServletRequest r, Object info) throws Exception {

		this.dossierId = s.getCookie().get("Dossier_Id");
		base = s.getCookie().get("BaseDossier");
		Reglement_Id = r.getParameter("Reglement_Id");
		Avoir_Id = r.getParameter("Avoir_Id");
		montantaff = r.getParameter("Montant_Impute");
		reglement=Reglement_Id!=null;
		Motif = r.getParameter("Motif");
		Mode_Reg = r.getParameter("Mode_Reg");
		if (r.getParameter("Date")!=null) {
			Date = new Long(r.getParameter("Date")).longValue();
		}
		else {
			Date = 0;
		}
		Texte = r.getParameter("Texte");

		this.absdir = s.getAppConfiguration().get("opensi.absdir");

		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		dfs.setGroupingSeparator(' ');

		numFormat.setDecimalFormatSymbols(dfs);
		numFormat.setNegativeSuffix("-");
		numFormat.setNegativePrefix("");
		numFormat.setPositiveSuffix(" ");
		numFormat.setGroupingUsed(true);
		numFormat.setGroupingSize(3);

		numFormatCap.setDecimalFormatSymbols(dfs);
		numFormatCap.setGroupingUsed(true);
		numFormatCap.setGroupingSize(3);

		Connection con = s.getConnection(null);

		String reqDestinataireAvoir = "select ef.Denomination, ef.Adresse_1, ef.Adresse_2, ef.Code_Postal, ef.Ville, p.Nom_FR as Pays"
													 + " from "+ base +".AVOIR a,"+ base +".ADRESSE_CLIENT ef, PAYS p where ef.Code_Pays=p.Code_Pays and ef.Defaut_Fact=1 and Avoir_Id=? and a.Client_Id=ef.Client_Id";

		String reqDestinataireReglement = "select ef.Denomination, ef.Adresse_1, ef.Adresse_2, ef.Code_Postal, ef.Ville, p.Nom_FR as Pays"
													 + " from "+ base +".REGLEMENT_CLIENT r,"+ base +".ADRESSE_CLIENT ef, PAYS p where ef.Code_Pays=p.Code_Pays and ef.Defaut_Fact=1 and r.Reglement_Id=? and r.Client_Id=ef.Client_Id";

		String reqProvFact = "select Date_Facture,f.Num_Entier from "+ base +".FACTURE f,"+ base +".ECHEANCE_CLIENT e,"+ base +".REGLEMENT_CLIENT r where r.Client_Id=f.Client_Id";
					reqProvFact += " and Reglement_Id=?";
					reqProvFact += " and e.Document_Id=f.Facture_Id";
					reqProvFact += " and e.Provenance='Facture'";
					reqProvFact += " order by Date_Facture DESC";

		String reqReglement = "select * from " + base + ".REGLEMENT_CLIENT r";
		reqReglement += " where Reglement_Id=?";


		String reqAvoir = "select * from " + base + ".AVOIR r";
		reqAvoir += " where Avoir_Id=?";

		psReglement = con.prepareStatement(reqReglement);
		psAvoir = con.prepareStatement(reqAvoir);
		psDestinataireAvoir = con.prepareStatement(reqDestinataireAvoir);
		psDestinataireReglement = con.prepareStatement(reqDestinataireReglement);
		psProvFact = con.prepareStatement(reqProvFact);

		Statement stt = con.createStatement();

		String reqLogo = "select Logo_Adr from PARAM_DOSSIER where Dossier_Id='"+ dossierId +"'";

		ResultSet rset = stt.executeQuery(reqLogo);

		rset.next();

		this.logoAdr = rset.getInt("Logo_Adr")==1;

		rset.close();

	 	// coordonn\u00E9es de l'entreprise

		String reqSociete = "select Ville_RCS, Num_TVA_Intra, Type_Societe, Montant_Capital, Num_SIRET, Raison_Sociale,"
											+ " Adresse, Comp_Adresse, Code_Postal, Ville, Telephone, Fax, Email, Site_Web "
											+ "from DOSSIER where Dossier_Id='"+ dossierId +"'";

		rset = stt.executeQuery(reqSociete);

		rset.next();

		this.raisonSociale = rset.getString("Raison_Sociale");
		this.adresseSociete1 = rset.getString("Adresse");
		this.adresseSociete2 = rset.getString("Comp_Adresse");
		this.codePostal = rset.getString("Code_Postal");
		this.ville = rset.getString("Ville");
		this.tel = rset.getString("Telephone");
		this.fax = rset.getString("Fax");
		this.email = rset.getString("Email");
		this.web = rset.getString("Site_Web");
		this.siren = formatSiren(rset.getString("Num_SIRET"));
		this.typeSociete = rset.getString("Type_Societe");
		this.capital = rset.getDouble("Montant_Capital");
		this.tvaIntra = rset.getString("Num_TVA_Intra");
		this.rcs = rset.getString("Ville_RCS");

		rset.close();

		Document document = new Document(PageSize.A4, 20, 20, 20, 20);
		String chemin = s.getAppConfiguration().get("opensi.absdir") + "/pdf/";

		String fichierPdf = "rembourser_"+ r.getParameter("id") +".pdf";

		PdfWriter.getInstance(document, new FileOutputStream(chemin + fichierPdf));

		Phrase foot=new Phrase(raisonSociale +" - "+ typeSociete +" au capital de "+ numFormatCap.format(capital) +" Euros - immatricul\u00E9e au RCS "+ rcs +" "+ siren +" - N\u00B0 TVA : "+ tvaIntra, fontLegale);
		HeaderFooter footer = new HeaderFooter(foot,false);
		footer.setAlignment(HeaderFooter.ALIGN_CENTER);
		footer.setBorder(HeaderFooter.TOP);
		footer.setBorderWidth(1);
 		document.setFooter(footer);
		document.open();

		Lettre(document);
		s.closeConnection(con, null);

		document.close();
		try {
			File filePdf = new File(chemin, fichierPdf);
			filePdf.deleteOnExit();
		} catch(SecurityException se) {
			s.trace("Impossible d'enregistrer le fichier \u00E0 supprimer");
			se.printStackTrace();
		}

		return true;

	} // fin serviceNum_Entier


	private void Lettre(Document document) throws Exception {

		ResultSet rset;
		if (reglement) {
		psReglement.setInt(1, Integer.parseInt(Reglement_Id));
		rset = psReglement.executeQuery();
		}
		else {
		psAvoir.setInt(1, Integer.parseInt(Avoir_Id));
		rset = psAvoir.executeQuery();
		}
		if (rset.next()) {

			document.newPage();
			document.add(makeHeader());
		 	document.add(makeDebutLettre());

			if (reglement) {
				rset.close();
		 		document.add(makeLettreReglement());
 			}
			else {
				long Date_avo = rset.getLong("Date_Avoir");
				String Numero= rset.getString("Num_Entier");
				rset.close();
		 		document.add(makeLettreAvoir(Numero,Date_avo));
			}
			rset.close();
		 	document.add(makeFinLettre());
		}
	}


	private PdfPTable makeHeader() throws Exception {

		PdfPTable pptHeader = new PdfPTable(1);

		pptHeader.setWidthPercentage(100);

		PdfPCell cell;

		cell = new PdfPCell(makeEntete());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);
		pptHeader.addCell(cell);

		cell = new PdfPCell(makeSpace(50));
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setPadding(0);
		pptHeader.addCell(cell);

		return pptHeader;
	}


	private PdfPTable makeEntete() throws Exception {

		PdfPTable pptEntete = new PdfPTable(2);

		pptEntete.setWidthPercentage(100);
		int[] widths = {55,45};
		pptEntete.setWidths(widths);

		PdfPCell cellSociete = new PdfPCell(makeInfosSociete());
		cellSociete.setBorder(PdfPCell.NO_BORDER);

		PdfPCell cellFacture = new PdfPCell(makeInfosFacture());
		cellFacture.setBorder(PdfPCell.NO_BORDER);

		pptEntete.addCell(cellSociete);
		pptEntete.addCell(cellFacture);

		return pptEntete;
	}


	private PdfPTable makeSpace(int space) throws DocumentException {

		PdfPTable pptSpace = new PdfPTable(1);

		PdfPCell cell = new PdfPCell(pptSpace.getDefaultCell());
		cell.setBorder(PdfPCell.NO_BORDER);
		cell.setFixedHeight(space);
		cell.setPhrase(new Phrase(""));

		pptSpace.addCell(cell);

		return pptSpace;
	}


	private PdfPTable makeInfosSociete() throws Exception {

		PdfPTable pptInfos = new PdfPTable(1);

		PdfPCell cell;

		try {
			Image logo = Image.getInstance(absdir+ "/logos/"+ dossierId +".jpg");
			if (logo.getDpiX()!=0 && logo.getDpiY()!=0)
				logo.scaleAbsolute(logo.plainWidth() * 72 / logo.getDpiX(), logo.plainHeight() * 72 / logo.getDpiY());

			cell = new PdfPCell(logo, false);

			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);

			pptInfos.addCell(cell);

			cell.setImage(null);

			cell.setMinimumHeight(5);
			cell.setPhrase(new Phrase("", fontAdrSociete));
			pptInfos.addCell(cell);
			cell.setMinimumHeight(0);
		}
		catch (IOException e) {
			cell = new PdfPCell(pptInfos.getDefaultCell());
			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		}

		if (!logoAdr) {
			cell.setPhrase(new Phrase(raisonSociale, fontAdrSocieteGras));
			pptInfos.addCell(cell);
			cell.setPhrase(new Phrase(adresseSociete1, fontAdrSociete));
			pptInfos.addCell(cell);
			if (!adresseSociete2.equals("")) {
				cell.setPhrase(new Phrase(adresseSociete2, fontAdrSociete));
				pptInfos.addCell(cell);
			}
			cell.setPhrase(new Phrase(codePostal +" "+ ville, fontAdrSociete));
			pptInfos.addCell(cell);

			String telfax = "";

			if (tel.length()>0) {
				telfax = "T\u00E9l: "+ tel;
				if (fax.length()>0) {
					telfax += " - Fax: "+ fax;
				}
			}
			else if (fax.length()>0) {
				telfax = "Fax: "+ fax;
			}

			if (telfax.length()>0) {
				cell.setPhrase(new Phrase(telfax, fontAdrSociete));
				pptInfos.addCell(cell);
			}

			if (!email.equals("")) {
				cell.setPhrase(new Phrase("e-mail: "+ email, fontAdrSociete));
				pptInfos.addCell(cell);
			}
			if (!web.equals("")) {
				cell.setPhrase(new Phrase(web, fontWeb));
				pptInfos.addCell(cell);
			}
		}

		cell.setPhrase(new Phrase(""));
		pptInfos.addCell(cell);

		return pptInfos;
	}


	private PdfPTable makeInfosFacture() throws SQLException {

		PdfPTable pptInfosFacture = new PdfPTable(1);
		PdfPCell cell = new PdfPCell(pptInfosFacture.getDefaultCell());

		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setPaddingBottom(4);
		cell.setBorder(PdfPCell.NO_BORDER);
		ResultSet rset;
		if (reglement) {
			psDestinataireReglement.setInt(1, Integer.parseInt(Reglement_Id));
			rset = psDestinataireReglement.executeQuery();
		}
		else {
			psDestinataireAvoir.setInt(1, Integer.parseInt(Avoir_Id));
			rset = psDestinataireAvoir.executeQuery();
		}
		if (rset.next()) {

			String societeFact = rset.getString("Denomination");
			String adresseFact1 = rset.getString("Adresse_1");
			String adresseFact2 = rset.getString("Adresse_2");
			String codePostalFact = rset.getString("Code_Postal");
			String villeFact = rset.getString("Ville");
			String paysFact = rset.getString("Pays");

			rset.close();


			cell.setPhrase(new Phrase("", fontTexte));
			cell.setFixedHeight(120);
			pptInfosFacture.addCell(cell);
			cell.setMinimumHeight(0);

			cell.setPhrase(new Phrase(societeFact, fontAdrFacture));
			pptInfosFacture.addCell(cell);

			cell.setPhrase(new Phrase(adresseFact1, fontAdrFacture));
			pptInfosFacture.addCell(cell);

			if (!adresseFact2.equals("")) {
				cell.setPhrase(new Phrase(adresseFact2, fontAdrFacture));
				pptInfosFacture.addCell(cell);
			}

			cell.setPhrase(new Phrase(codePostalFact+" "+villeFact, fontAdrFacture));
			pptInfosFacture.addCell(cell);

			if (!paysFact.equals("")) {
				cell.setPhrase(new Phrase(paysFact, fontAdrFacture));
				pptInfosFacture.addCell(cell);
			}

			cell.setPhrase(new Phrase("", fontTexte));
			pptInfosFacture.addCell(cell);
		}
		return pptInfosFacture;
	}

	public String formatSiren(String siret) {
		if (siret.length()>=9) {
			return siret.substring(0,3) +" "+ siret.substring(3,6) +" "+ siret.substring(6,9);
		}
		else {
			return siret;
		}
	}
	private PdfPTable makeDebutLettre() throws SQLException {

		PdfPTable tab = new PdfPTable(2);
		tab.setWidthPercentage(100);

		PdfPCell cel = new PdfPCell(tab.getDefaultCell());
		cel.setBorder(PdfPCell.NO_BORDER);

		cel.setPhrase(new Phrase("Objet : Lettre de remboursement", fontTexte));
		tab.addCell(cel);

		cel.setPhrase(new Phrase("Date : "+ formatDate.format(new Date(System.currentTimeMillis())), fontTexte));
		tab.addCell(cel);

		cel.setColspan(2);
		cel.setPhrase(new Phrase(""));
		cel.setFixedHeight(80);
		tab.addCell(cel);
		cel.setMinimumHeight(0);

		String texte = "Madame, Monsieur,\n\n\n\n";
		texte += "Par la pr\u00E9sente, nous avons le plaisir de vous confirmer le remboursement de "+ montantaff +" \u20AC TTC,";
		texte += "\ncorrespondant ";
		cel.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED);
		cel.setPhrase(new Phrase(texte, fontTexte));
		tab.addCell(cel);
		return tab;
	}


	private PdfPTable makeLettreAvoir(String Numero,long date) throws SQLException {

		PdfPTable tab = new PdfPTable(1);
		tab.setWidthPercentage(100);

		PdfPCell cel = new PdfPCell(tab.getDefaultCell());
		cel.setBorder(PdfPCell.NO_BORDER);

		String texte = "\u00E0 l'avoir n\u00B0 "+Numero+ " du "+formatDate.format(new Date(date));
		texte+= ", par "+Mode_Reg+".";
		cel.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED);
		cel.setPhrase(new Phrase(texte, fontTexte));
		tab.addCell(cel);
		return tab;
	}

	private PdfPTable makeLettreReglement() throws SQLException {

		PdfPTable tab = new PdfPTable(1);
		tab.setWidthPercentage(100);

		PdfPCell cel = new PdfPCell(tab.getDefaultCell());
		cel.setBorder(PdfPCell.NO_BORDER);
		String texte="";
		if (Motif.equals("autre")) {
			texte = Texte;
		}
		else {
			if (Motif.equals("retmarch")) {
				texte = "au retour de marchandise du "+formatDate.format(new Date(Date));
			}
			else if (Motif.equals("com")) {
				texte = "\u00E0 votre commande non honor\u00E9e du "+formatDate.format(new Date(Date));
			}
			else if (Motif.equals("trop")) {
				psProvFact.setInt(1, Integer.parseInt(Reglement_Id));
				ResultSet rset = psProvFact.executeQuery();
				if (rset.next()) {
					texte = "au trop percu sur la facture n\u00B0 "+rset.getString("f.Num_Entier")+ " du "+formatDate.format(new Date(rset.getLong("f.Date_Facture")));
				}
				else {
					texte = "\u00E0 un trop percu";
				}
			}
			else if (Motif.equals("recl")) {
				texte = "\u00E0 la somme mentionn\u00E9e sur votre lettre de r\u00E9clamation du "+formatDate.format(new Date(Date));
			}
			texte+= ", par "+Mode_Reg+".";
		}
		cel.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED);
		cel.setPhrase(new Phrase(texte, fontTexte));
		tab.addCell(cel);
		return tab;
	}

	private PdfPTable makeFinLettre() throws SQLException {

		PdfPTable tab = new PdfPTable(1);
		tab.setWidthPercentage(100);

		PdfPCell cel = new PdfPCell(tab.getDefaultCell());
		cel.setBorder(PdfPCell.NO_BORDER);

		String texte = "\n\n\nEn vous remerciant de votre confiance,";

		texte += "\n\n\nNous vous prions d'agr\u00E9er, Madame, Monsieur, l'assurance de notre consid\u00E9ration distingu\u00E9e.";

		texte += "\n\n\n\nLa comptabilit\u00E9";
		cel.setHorizontalAlignment(PdfPCell.ALIGN_JUSTIFIED);
		cel.setPhrase(new Phrase(texte, fontTexte));
		tab.addCell(cel);

		return tab;
	}

} // fin GenPdfRelances
