/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *  Mise a jour des informations des articles  l'exception du prix, du stock et du on-line/off-line
 */

package org.opensi.webmanager.fonctions_periodiques;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Vector;

import org.opensi.webmanager.utils.AppelServices;
import org.opensi.webmanager.utils.ArticleFactureStruct;
import org.opensi.webmanager.utils.FactureStruct;
import org.opensi.webmanager.utils.ListeArticleFactureStruct;
import org.opensi.webmanager.utils.PersonneStruct;
import org.opensi.webmanager.utils.ToolsMAJ;



/**
 * Procdure de mise  jour des informations des articles (description...)
 */
public class MAJFactures extends MAJ {

	Vector<String> listeId;
	
	/**
	 * Prepare le lancement de la mise  jour.
	 * 
	 * @param nbArticles
	 *          nombre d'articles maximum par envoi
	 * @param timeoutService
	 *          temps de rponse maximum du web service
	 * @param lenbRapports
	 *          Nombre de rapports a conserver.
	 */
	public MAJFactures(int nbArticles, int timeoutService, int lenbRapports,String fichier, String nomtype) {
		super(nbArticles,timeoutService,lenbRapports,fichier,nomtype);
	}
	public MAJFactures(int nbArticles, int timeoutService, int lenbRapports) {
		super(nbArticles,timeoutService,lenbRapports);
	}

	/**
	 * Lance la mise  jour.
	 * 
	 * La mise  jour est faite en plusieurs tapes: On regarde si il y a des
	 * articles  mettre  jour, puis on cre la liste d'article a envoyer au
	 * serveur, si tout s'est bien pass, on enregistre dans OpenSi que les
	 * informations de ces articles sont maintenant  jour.
	 * 
	 * La connexion  la base de donne n'est ni ouverte ni ferme.
	 * 
	 * @param serveur
	 *          adresse du web service
	 * @param dossierRoot
	 *          chemin absolu vers le seveur de OpenSi (opensi-serveur)
	 * @param dossierId
	 *          identifiant du dossier
	 * @param con
	 *          Connexion  la base de donne
	 * @param base
	 *          Nom de la base de donne FACT_qquechose
	 * @param site_id
	 *          identifiant du site
	 * @return vrai si la mise  jour s'est bien passe, faux sinon.
	 */
	public boolean lancerMAJ(String serveur, String dossierRoot, String dossierId, Connection con, String base, String site_id) {
		java.util.Date date = new java.util.Date();
		Timestamp stamp = new Timestamp(date.getTime());
		dateDebut = "" + stamp;

		appelService(serveur, con, base);

		date = new java.util.Date();
		stamp = new Timestamp(date.getTime());
		dateFin = "" + stamp;

		try {
			ToolsMAJ.ecrireLog(aEcrire,type,"Je met la date");

			String requete = "UPDATE " + base + ".MAJ_AUTO SET Derniere_Maj=" + date.getTime() + " WHERE Site_Id=" + site_id
					+ " AND Type_Maj='FACTU'";
			Statement state = con.createStatement();
			state.executeUpdate(requete);
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		ecrireRapport(dossierRoot, dossierId, site_id,type,ToolsMAJ.TYPE_MAJ_FACTU);

		return appelToutServiceReussi;

	}

	/**
	 * Envoie les informations au web service
	 */
	private void appelService(String serveur, Connection con, String base) {
		try {
			// On rcupre toutes les infos  l'exception du stock et du prix et
			// de on-line/off-line

			String reqInfo = "";
			reqInfo += "SELECT  *";
			reqInfo += " FROM " + base + ".FACTURE f, " + base + ".COMMANDE_WEB cw, " + base + ".CLIENT_WEB cli ";
			reqInfo += " WHERE f.Envoyee_Au_Site=0 AND f.Com_Web_Id>0 AND cli.Client_Web_Id=cw.Client_Web_Id";

			Statement state = con.createStatement();
			ResultSet resInfo = state.executeQuery(reqInfo);

			Vector<Object> liste = new Vector<Object>();
			listeId = new Vector<String>();

			int compteur = 0;
			while (resInfo.next()) {
				FactureStruct factureTemp = new FactureStruct();
				PersonneStruct personne=new PersonneStruct();
				
				factureTemp.setString(factureTemp.ID_COMMANDE,resInfo.getString("cw.Com_Site_Id"));
				factureTemp.setDouble(factureTemp.MONTANT_TOTAL,resInfo.getDouble("f.Montant_TTC"));
				factureTemp.setDouble(factureTemp.MONTANT_TVA,resInfo.getDouble("f.Montant_TTC") - resInfo.getDouble("f.Total_HT"));
				factureTemp.setDouble(factureTemp.FRAIS_PORT,resInfo.getDouble("f.Frais_Port"));
				factureTemp.setDouble(factureTemp.REMISE,resInfo.getDouble("f.Remise"));
				factureTemp.setString(factureTemp.NUM_FACTURE,resInfo.getString("f.Num_Entier"));
				factureTemp.setLong(factureTemp.DATE_FACTURE,resInfo.getLong("f.Date_Facture"));
				factureTemp.setString(factureTemp.MODE_REGLEMENT,resInfo.getString("f.Mode_Reg"));
				factureTemp.setString(factureTemp.ID_CLIENT,resInfo.getString("cli.Client_Site_Id"));
				factureTemp.setString(factureTemp.NUM_MOIS,resInfo.getString("f.Numero"));
				
				personne.setString(personne.CIV,resInfo.getString("f.Civ_Inter"));
				personne.setString(personne.NOM,resInfo.getString("f.Nom_Inter"));
				personne.setString(personne.PRENOM,resInfo.getString("f.Prenom_Inter"));
				personne.setString(personne.ENTREPRISE,resInfo.getString("f.Raison_Sociale"));
				personne.setString(personne.ADRESSE,resInfo.getString("f.Adresse_Fact"));
				personne.setString(personne.COMPADRESSE,resInfo.getString("f.Comp_Adresse_Fact"));
				personne.setString(personne.ADRESSE3,resInfo.getString("f.Adresse_3_Fact"));
				personne.setString(personne.CP,resInfo.getString("f.CP_Fact"));
				personne.setString(personne.VILLE,resInfo.getString("f.Ville_Fact"));
				
				factureTemp.setStruct(factureTemp.PERS_FACT, personne.getStruct());
				
				String reqprod="SELECT *";
				reqprod+=" FROM " + base + ".ARTICLE_FACTURE";
				reqprod+=" WHERE Facture_Id="+resInfo.getInt("Facture_Id"); 

				ResultSet resProd = state.executeQuery(reqprod);

				ListeArticleFactureStruct listeArticle=new ListeArticleFactureStruct();

				while(resProd.next())
				{
					if (resProd.getString("Reference")!=null)
					{					
						ArticleFactureStruct article=new ArticleFactureStruct(resProd.getString("Reference"));
			
						article.setDouble(article.QUANTITE,resProd.getDouble("Quantite"));
						article.setDouble(article.PRIX_U,resProd.getDouble("Prix"));
						article.setDouble(article.TVA,resProd.getDouble("Taux_TVA"));
						article.setString(article.DESIGNATION,resProd.getString("Designation"));

						listeArticle.addArticle(article);
					}
				}
				factureTemp.setStruct(factureTemp.LISTE_ARTICLES,listeArticle.getListeStruct());				
				liste.add(factureTemp.getStruct());

				listeId.add(resInfo.getString("Facture_Id"));

				// On envoi tout les nbArticlesParEnvoi articles
				if ((compteur % nbArticlesParEnvoi) == 0 && compteur > 0) {
					ToolsMAJ.ecrireLog(aEcrire,type,"Appel du service pour " + nbArticlesParEnvoi + " articles.");
					envoyerMaj(serveur, liste.toArray());

					appelToutServiceReussi &= appelServiceReussi;

					if (appelServiceReussi) {
						ToolsMAJ.ecrireLog(aEcrire,type,"Je valide ");
						validerDansOpenSi(con, base);
					}
					else {
						nbEchecs += nbArticlesParEnvoi;
					}

					liste = new Vector<Object>();
					listeId = new Vector<String>();// pour valider
				}
				compteur++;

			}

			nbArticles = compteur;

			// la fin de la liste
			if (nbArticles > 0) {
				ToolsMAJ.ecrireLog(aEcrire,type,"Appel du service pour " + nbArticlesParEnvoi + " articles.");
				envoyerMaj(serveur, liste.toArray());

				if (appelServiceReussi) {
					ToolsMAJ.ecrireLog(aEcrire,type,"Je valide ");
					validerDansOpenSi(con, base);
				}
				else {
					nbEchecs += nbArticlesParEnvoi;
				}
			}
			ToolsMAJ.ecrireLog(aEcrire,type,"fini");

		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"ERREUR " + e);
		}

	}

	private void envoyerMaj(String serveur, Object[] liste) {

		appelServiceReussi = true;

		try {
			Object resultat = AppelServices.majFactures(serveur, liste);
			Object[] rep = (Object[]) resultat;
			Object[] rapport = (Object[]) rep[0];
			lesRapports.addRapport(rapport);
		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"ERREUR:" + e);
			appelServiceReussi = false;
		}
	}
	
	/*
	 * Valide la mise  jour dans OpenSi: change la date de rfrence info par
	 * celle de l'article utilise prcedemment
	 */
	private void validerDansOpenSi(Connection con, String base) {
		try {

			String listeId_str = new String();

			if (listeId.size() > 0) {
				listeId_str += (listeId.size() > 0)?"'" + ToolsMAJ.sqlEncode(listeId.elementAt(0)) + "'":"";
			}

			for (int i = 1; i < listeId.size(); i++) {
				listeId_str += ",'" + ToolsMAJ.sqlEncode(listeId.elementAt(i)) + "'";
			}

			String req = "UPDATE " + base + ".FACTURE SET Envoyee_Au_Site=1";
			req += " WHERE Facture_Id IN(" + listeId_str + ")";

			ToolsMAJ.ecrireLog(aEcrire,type,"La requete:" + req);

			Statement state = con.createStatement();
			state.executeUpdate(req);

		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"ERREUR " + e);
		}
	}
}
