/******************************************************************************/
/* OpenSi : Outils libres de gestion d'entreprise                             */
/* Copyright (C) 2003 Speedinfo.fr S.A.R.L.                                   */
/* Contact: contact@opensi.org                                                */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

/**
 *  Rcupration des commandes
 */

package org.opensi.webmanager.fonctions_periodiques;

import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.HashMap;

import org.opensi.facturation.GenDocNumber;
import org.opensi.facturation.StockManager;
import org.opensi.util.tools.DateTime;
import org.opensi.webmanager.utils.AppelServices;
import org.opensi.webmanager.utils.ArticleCommandeStruct;
import org.opensi.webmanager.utils.ClientStruct;
import org.opensi.webmanager.utils.CommandeStruct;
import org.opensi.webmanager.utils.MultiRapports;
import org.opensi.webmanager.utils.PersonneStruct;
import org.opensi.webmanager.utils.RedactionRapport;
import org.opensi.webmanager.utils.ToolsMAJ;

/**
 * Procdure de rcupration des commandes.
 */
public class UploadCommandes {

	boolean appelServiceReussi;
	boolean appelToutServiceReussi;
	Object[] rapport;
	String dateDebut;
	String dateFin;
	Object[] listeCommandes;
	int nbRecupReussi;
	MultiRapports lesRapports;
	HashMap<Double, Integer> code_tva;
	int nbCommandes;
	private File aEcrire;
	private String type;
	private String reqRoot;
	private ResultSet resRoot;

	/**
	 * Initialise la rcupration des commandes.
	 */
	public UploadCommandes(String nomFichier, String nomtype) {
		appelServiceReussi = true;
		aEcrire = new File(nomFichier);// le fichier de log
		this.type=nomtype;
		lesRapports = new MultiRapports();
	}
	public UploadCommandes() {
		aEcrire = new File("Lancer_Maintenant.log");// le fichier de log
		this.type="";
		appelServiceReussi = true;
		lesRapports = new MultiRapports();
	}

	/**
	 * Lance la rcupration des commandes.
	 * 
	 * La connexion  la base de donne n'est ni ouverte ni ferme.
	 * 
	 * @param serveur
	 *          adresse du web service
	 * @param dossierRoot
	 *          chemin absolu vers le serveur de OpenSi (opensi-serveur)
	 * @param dossierId
	 *          identifiant du dossier
	 * @param con
	 *          Connexion  la base de donne
	 * @param nomBase
	 *          Nom de la base de donne FACT_qquechose
	 * @param site_id
	 *          identifiant du site
	 * @return vrai si la mise  jour s'est bien passe, faux sinon.
	 */
	public boolean lancerUpload(String serveur, String dossierRoot, String dossierId, Connection con, String nomBase, String site_id) {
		// //ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Debut ");

		nbRecupReussi = 0;
		nbCommandes = 0;
		appelToutServiceReussi = true;
		java.util.Date date = new java.util.Date();
		Timestamp stamp = new Timestamp(date.getTime());
		dateDebut = "" + stamp;

		boolean continuer = true;

		// Pour la gestion des codes de TVA
		code_tva = new HashMap<Double, Integer>();
		try {
			String req = "SELECT Code_TVA, Taux_TVA FROM " + nomBase + ".TAUX_TVA";
			Statement state = con.createStatement();
			ResultSet resTva = state.executeQuery(req);

			while (resTva.next()) {
				code_tva.put(new Double(resTva.getDouble("Taux_Tva")), new Integer(resTva.getInt("Code_TVA")));
			}
		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] ERREUR1:" + e);
		}

		while (continuer) {
			appelServiceReussi = true;

			continuer = appelService(serveur);
			appelToutServiceReussi = appelToutServiceReussi && appelServiceReussi;
			if (appelServiceReussi) {

				ToolsMAJ.ecrireLog(aEcrire,type,"enregistrement");
				enregistrer(con, nomBase, site_id, dossierId);
				nbRecupReussi++;
			}

		}

		date = new java.util.Date();
		stamp = new Timestamp(date.getTime());
		dateFin = "" + stamp;

		try {
			String requete = "UPDATE " + nomBase + ".MAJ_AUTO SET Derniere_Maj=" + date.getTime() + " WHERE Site_Id=" + site_id
					+ " AND Type_Maj='COM'";
			Statement state = con.createStatement();
			state.executeUpdate(requete);
		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommande]ERREUR2" + e);
		}

		ecrireRapport(dossierRoot, dossierId, site_id);

		// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Fin ");

		return appelToutServiceReussi;

	}

	private boolean appelService(String serveur) {

		Object resultat = null;
		boolean resultFct = false;

		try {
			resultat = AppelServices.uploadCommandes(serveur);

			Object[] rep = (Object[]) resultat; // valeur
			Object[] rapport = (Object[]) rep[0];// rapport

			lesRapports.addRapport(rapport);

			Object[] listeComPlusBool = (Object[]) rep[1];

			// Je sais que c'est une liste de commandes
			listeCommandes = (Object[]) listeComPlusBool[0];

			// Boolen indiquant si il faut continuer ou non
			Boolean fautContinuer = (Boolean) listeComPlusBool[1];
			if (fautContinuer == null) {
				resultFct = false;
				appelServiceReussi = false;
			}
			else {
				resultFct = ((Boolean) listeComPlusBool[1]).booleanValue() && listeCommandes.length>0;
			}
		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] ERREUR3:" + e);
			appelServiceReussi = false;
		}

		return resultFct;

	}

	private void ecrireRapport(String dossierRoot, String dossierId, String site_id) {

		try {
			boolean noErreurRapport = true;
			RedactionRapport ecriveur = new RedactionRapport(ToolsMAJ.TYPE_UPLOAD_COMMANDES, 10, dossierRoot, dossierId, site_id);

			ecriveur.ajouterLigne(dateDebut + ":La rcupration des commandes a t lance", true);

			if (nbRecupReussi < 1) {
				ecriveur.ajouterLigne("\nLe site distant n'as pas pu etre atteind. Aucunes commande recupre.", false);
			}
			else {
				// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommande] Nb de commandes:"+nbCommandes);

				// if(nbCommandes==0)
				// {
				// ecriveur.ajouterLigne(dateFin+":Aucune nouvelle
				// commande.",true);
				// }
				// else
				// {

				String lesDetails = "";

				int cptErreur = 0;
				int cptOk = 0;

				while (lesRapports.next()) {
					if (lesRapports.aReussi()) {
						lesDetails += "\n<br>" + lesRapports.getMessageUtilisateur();
						cptOk++;
					}
					else {
						lesDetails += "\n<br><span class=\"echec\">" + lesRapports.getMessageUtilisateur() + "</span>";
						cptErreur++;
					}

					noErreurRapport = noErreurRapport && (lesRapports.aReussi());
				}

				if (cptErreur >= 0) {
					ecriveur.ajouterLigne(dateFin + ":Toutes les commandes ont t rcuprs", true);
				}
				else {
					ecriveur.ajouterLigne(dateFin + ":" + cptErreur + " commandes n'ont pas t rcupres.", false);
					ecriveur.ajouterLigne(dateFin + ":" + cptOk + " commandes ont t rcupres.", true);
				}

				if (!lesDetails.equals("")) {
					ecriveur.ajouterLigne(" ****************** DETAILS **************", true);
					ecriveur.ajouterLigne(lesDetails, true);
				}
				// }

			}

			ecriveur.fermerFichier();

		}
		catch (java.io.IOException e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes(ecrireRapport)] ERREUR4:" + e);
		}

	}

	/* *********************************************************************** */
	/* * Enregistrement dans la base * */
	/* *********************************************************************** */

	private void enregistrer(Connection con, String base, String site_id, String dossierId) {
		try {
	
			Statement state = con.createStatement();
			Statement state2 = con.createStatement();

			nbCommandes += listeCommandes.length;
			for (int j = 0; j < listeCommandes.length; j++) {

				// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Une nouvelle commande");
				CommandeStruct com = new CommandeStruct((Object[]) listeCommandes[j]);

				ClientStruct client = new ClientStruct(com.getStruct(com.CLIENT));
				String clientId = client.getSqlString(client.CLIENTID,10);
				PersonneStruct personneClient = new PersonneStruct(client.getStruct(client.PERSONNE));
				ClientStruct client_Liv = new ClientStruct(com.getStruct(com.CLIENT_LIV));
				PersonneStruct personneLiv = new PersonneStruct(client_Liv.getStruct(client_Liv.PERSONNE));
				String com_id = com.getSqlString(com.ID,20);
				double com_totalHT = com.getDouble(com.TOTAL_HT);
				double com_totalTTC = com.getDouble(com.TOTAL_TTC);
				double com_tauxRemise = com.getDouble(com.TAUX_REMISE);
				double com_remise = com.getDouble(com.REMISE);
				double com_fraisPort = com.getDouble(com.FRAIS_PORT);
				double com_tauxTVAFraisPort = com.getDouble(com.TAUX_TVA_PORT);
				String com_modePaiement = com.getSqlString(com.MODE_PAIEMENT,20);
				int com_statut_Paiement = com.getInteger(com.STATUT_PAIE);
				String com_date_string = com.getSqlString(com.DATE,16);
				long com_date = DateTime.parseDate(com_date_string, "dd/MM/yyyy/HH/mm");
				int com_validation = com.getInteger(com.VALIDATION);
				String com_email_manager=com.getSqlString(com.EMAIL_MANAGER,64);
				String com_modeExpedition=com.getSqlString(com.MODE_EXPEDITION,60);
				String com_commentaires=com.getSqlString(com.COMMENTAIRES);
				String com_commentaires_2=com.getSqlString(com.COMMENTAIRES_2);
				int com_editionTTC = com.getInteger(com.EDITION_TTC);
				int com_editionCom = com.getInteger(com.EDITION_COM);
				String com_typeVente=com.getSqlString(com.TYPE_VENTE,1);
				String com_codePorte=com.getSqlString(com.CODE_PORTE,30);
				String com_numTransaction=com.getSqlString(com.NUM_TRANSACTION,20);
				String com_origine=com.getSqlString(com.ORIGINE,30);
				String com_infosCommande=com.getSqlString(com.INFOS_COMMANDE,100);
				String com_codePaysLiv=com.getSqlString(com.CODE_PAYS_LIV,3);
				String com_codePays=com.getSqlString(com.CODE_PAYS,3);
				int com_affaire = com.getInteger(com.AFFAIRE);
				int com_assujetti = com.getInteger(com.ASSUJETTI);
				int com_envoi_multiple = com.getInteger(com.ENVOI_MULTIPLE);
				int com_bloque = com.getInteger(com.BLOQUE);

			
				double com_tauxIndicatif = com.getDouble(com.TAUX_INDICATIF);

				String liv_civ=personneLiv.getSqlString(personneLiv.CIV,1);

				String liv_nom=personneLiv.getSqlString(personneLiv.NOM,30);
				String liv_prenom=personneLiv.getSqlString(personneLiv.PRENOM,30);
				String liv_entreprise=personneLiv.getSqlString(personneLiv.ENTREPRISE,50);
				String liv_adresse=personneLiv.getSqlString(personneLiv.ADRESSE,80);
				String liv_compadresse=personneLiv.getSqlString(personneLiv.COMPADRESSE,50);
				String liv_adresse3=personneLiv.getSqlString(personneLiv.ADRESSE3,50);
				String liv_cp=personneLiv.getSqlString(personneLiv.CP,10);
				String liv_ville=personneLiv.getSqlString(personneLiv.VILLE,50);
				String liv_code_pays=personneLiv.getSqlString(personneLiv.CODE_PAYS,3);
				String liv_tel=client_Liv.getSqlString(client_Liv.TEL,20);
				String liv_fax=client_Liv.getSqlString(client_Liv.FAX,20);
				String liv_email=client_Liv.getSqlString(client_Liv.EMAIL,64);
				
				
				
				String client_id=client.getSqlString(client.ID,60);
				int client_tarif=client.getInteger(client.TARIF);
				String client_civ=personneClient.getSqlString(personneClient.CIV,1);
				String client_nom=personneClient.getSqlString(personneClient.NOM,30);
				String client_prenom=personneClient.getSqlString(personneClient.PRENOM,30);
				String client_entreprise=personneClient.getSqlString(personneClient.ENTREPRISE,50);
				String client_adresse=personneClient.getSqlString(personneClient.ADRESSE,80);
				String client_compadresse=personneClient.getSqlString(personneClient.COMPADRESSE,50);
				String client_adresse3=personneClient.getSqlString(personneClient.ADRESSE3,50);
				String client_cp=personneClient.getSqlString(personneClient.CP,10);
				String client_ville=personneClient.getSqlString(personneClient.VILLE,50);
				String client_code_pays=personneClient.getSqlString(personneClient.CODE_PAYS,3);
				String client_tel=client.getSqlString(client.TEL,20);
				String client_fax=client.getSqlString(client.FAX,20);
				String client_email=client.getSqlString(client.EMAIL,64);
				String client_pass=client.getSqlString(client.PASS,20);
				
				int numClient = 0;
				int numCommande = 0;
				
				String fact_civ_string="";
				String reqCiv = "SELECT * FROM CIVILITE WHERE Civ_Id="+ client_civ;
				ResultSet resCiv = state.executeQuery(reqCiv);
				if (resCiv.next()) {// Le site est slectionn
					fact_civ_string=resCiv.getString("Civ_Courte");
				}
				String liv_civ_string="";
				reqCiv = "SELECT * FROM CIVILITE WHERE Civ_Id="+ liv_civ;
				resCiv = state.executeQuery(reqCiv);
				if (resCiv.next()) {// Le site est slectionn
					liv_civ_string=resCiv.getString("Civ_Courte");
				}
				int logRoot = 0;
				
				reqRoot = "select Utilisateur_Id from UTILISATEUR where Login='root'";
				resRoot = state.executeQuery(reqRoot);
				if (resRoot.next()) {// Le site est slectionn
					logRoot=resRoot.getInt("Utilisateur_Id");
				}
				
				if (com_affaire==1 && !clientId.equals("")) {
					int affaireId=0;
					/* **************** */
					/* * FICHE CLIENT * */
					/* **************** */
					// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Cration de la fiche client");
					DateTime dt = new DateTime();
					long now = dt.getTimeInMillis();
					
					String Type_Fact="C";
					String reqClient = "SELECT Type_Fact FROM " + base + ".FICHE_CLIENT WHERE Client_Id='"+ clientId+"'";
					ResultSet resClient = state.executeQuery(reqClient);
					if (resClient.next()) {
						Type_Fact=resClient.getString("Type_Fact");
					}

					GenDocNumber gdn = new GenDocNumber(now,con,base,dossierId);
					String Num_Entier = gdn.getNumeroAffaire();
					int Numero = gdn.getNumeroPeriode();

					String insertup = "insert into " + base + ".AFFAIRE ";
					insertup +=	"(Numero ,Num_Entier ,Client_Id ,Nom_Affaire ,Util_R,Util_M ,Util_C ,Date_Creation ,Date_Maj,Date_Cloture ,"+
          "Civ_Inter ,Nom_Inter ,Prenom_Inter ,Tel_Inter ,Fax_Inter ,Email_Inter ,Raison_Sociale ,Adresse_Fact ,Comp_Adresse_Fact ,Adresse_3_Fact ,CP_Fact ,Ville_Fact ,Code_Pays ,Commentaires ,Imp_Com  ,Etat ,Type_Fact ,"+
          "Civ_Inter_Liv ,Nom_Inter_Liv ,Prenom_Inter_Liv ,Tel_Inter_Liv ,Fax_Inter_Liv ,Email_Inter_Liv,Nom_Liv ,Adresse_Liv ,Comp_Adresse_Liv ,Adresse_3_Liv,CP_Liv ,Ville_Liv ,Code_Pays_Liv)";
					insertup +=	"values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
					PreparedStatement psinsertup = con.prepareStatement(insertup);
					
					psinsertup.setInt(1, Numero);
					psinsertup.setString(2, Num_Entier);
					psinsertup.setString(3, clientId);
					psinsertup.setString(4, "");
					
					// les 3 champs suivants prennent la valeur de l'utilisateur PVD
					// car seul PVD utilise cette fonction
					psinsertup.setInt(5, 158);
					psinsertup.setInt(6, logRoot);
					psinsertup.setInt(7, logRoot);
					
					
					psinsertup.setLong(8, now);
					psinsertup.setLong(9, now);
					psinsertup.setLong(10, 0);
					
					if (client_entreprise.equals("")) {
						psinsertup.setString(11, "0");
						psinsertup.setString(12, "");
						psinsertup.setString(13, "");
						psinsertup.setString(17, fact_civ_string +" "+client_nom+" "+client_prenom);
					}
					else {
						psinsertup.setString(11, client_civ);
						psinsertup.setString(12, client_nom);
						psinsertup.setString(13, client_prenom);
						psinsertup.setString(17, client_entreprise);
					}
					
					psinsertup.setString(14, client_tel);
					psinsertup.setString(15, client_fax);
					psinsertup.setString(16, client_email);
					
					psinsertup.setString(18, client_adresse);
					psinsertup.setString(19, client_compadresse);
					psinsertup.setString(20, client_adresse3);
					psinsertup.setString(21, client_cp);
					psinsertup.setString(22, client_ville);
					psinsertup.setString(23, com_codePays);
					psinsertup.setString(24, "");
					psinsertup.setInt(25, 0);
					psinsertup.setString(26, "N");
					psinsertup.setString(27, Type_Fact);
					

					if (liv_entreprise.equals("")) {
						psinsertup.setString(28, "0");
						psinsertup.setString(29, "");
						psinsertup.setString(30, "");
						psinsertup.setString(34, liv_civ_string +" "+liv_nom+" "+liv_prenom);
					}
					else {
						psinsertup.setString(28, liv_civ);
						psinsertup.setString(29, liv_nom);
						psinsertup.setString(30, liv_prenom);
						psinsertup.setString(34, liv_entreprise);
					}
					
					psinsertup.setString(31, liv_tel);
					psinsertup.setString(32, liv_fax);
					psinsertup.setString(33, liv_email);
					
					psinsertup.setString(35, liv_adresse);
					psinsertup.setString(36, liv_compadresse);
					psinsertup.setString(37, liv_adresse3);
					psinsertup.setString(38, liv_cp);
					psinsertup.setString(39, liv_ville);
					psinsertup.setString(40, com_codePaysLiv);
					
					psinsertup.execute();
					
					ResultSet rsnum = psinsertup.getGeneratedKeys();
					
					if (rsnum.next()) {
						affaireId = rsnum.getInt(1);
					}
					String ajoutCom = "INSERT INTO " + base + ".COM_CLIENT (" +
							"Affaire_Id ,Date_Com ,Remise ,Escompte ,Frais_Port ,Commentaires ,"+
              "Imp_Com ,Date_Creation ,Date_Maj ,Util_M ,Util_C ,Ref_Com ,Num_Com_Site ,Etat ,Total_HT ,Montant_TTC ,"+
              "Delai ,Soldee ,Acompte ,Montant_Remise ,Edition_TTC, Taux_TVA_Port,Code_Tarif ,Assujetti_TVA,Num_TVA_Intra) ";
					ajoutCom += " select ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,Code_Tarif ,Assujetti_TVA,Num_TVA_Intra from " + base + ".FICHE_CLIENT where Client_Id=?";
					PreparedStatement psajoutCom = con.prepareStatement(ajoutCom);

					psajoutCom.setInt(1, affaireId);
					psajoutCom.setLong(2, now);
					psajoutCom.setDouble(3, com_tauxRemise);
					psajoutCom.setDouble(4, 0);
					psajoutCom.setDouble(5, com_fraisPort);
					psajoutCom.setString(6, com_commentaires);
					
					psajoutCom.setInt(7, 0);
					psajoutCom.setLong(8, now);
					psajoutCom.setLong(9, now);
					psajoutCom.setInt(10, logRoot);
					psajoutCom.setInt(11, logRoot);
					psajoutCom.setString(12, com_id);
					psajoutCom.setString(13, com_id);
					psajoutCom.setString(14, "N");
					psajoutCom.setDouble(15, com_totalHT);
					psajoutCom.setDouble(16, com_totalTTC);
					
					psajoutCom.setString(17, "0");
					psajoutCom.setInt(18, 0);
					psajoutCom.setInt(19, 0);
					psajoutCom.setDouble(20, com_remise);
					psajoutCom.setInt(21, 0);
					psajoutCom.setDouble(22, com_tauxTVAFraisPort);
					psajoutCom.setString(23, clientId);
					
					psajoutCom.execute();
					
					rsnum = psajoutCom.getGeneratedKeys();
					
					if (rsnum.next()) {
						numCommande = rsnum.getInt(1);
					}
					
					/* ************************** */
					/* * DETAIL DE COMMANDE * */
					/* ************************** */
					// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Cration des dtails de la
					// commande");
					Object[] liste = com.getStruct(com.LISTE_ARTICLES);

					String insertartex = "insert into " + base + ".ARTICLE_COM_CLIENT (" +
							"Com_Id ,Rank ,Reference ,Type_Ligne ,Designation ,Quantite ,Prix ,Ristourne ,Taux_TVA ,Code_TVA ,"+
							"Libelle , Commentaire, Commentaire_Avant)";
					insertartex +=	"values (?,?,?,?,?,?,?,?,?,?,?,?,?)";
					PreparedStatement psartex = con.prepareStatement(insertartex);
					
					String insertart = "insert into " + base + ".ARTICLE_COM_CLIENT (" +
					"Com_Id ,Rank ,Reference ,Type_Ligne ,Designation ,Quantite ,Prix ,Ristourne ,Taux_TVA ,Code_TVA ,"+
					"Libelle,Commentaire ,Commentaire_Avant)";
					insertart +=	"values (?,?,?,?,?,?,?,?,?,?,?,?,?)";
			    PreparedStatement psart = con.prepareStatement(insertart);
				
					for (int i = 0; i < liste.length; i++) {
	
						ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes]Un prod");
	
						ArticleCommandeStruct article = new ArticleCommandeStruct((Object[]) liste[i]);
	
						// Recherche du code TVAcom_assujetti
						Integer the_code = code_tva.get(new Double(article.getDouble(article.TVA)));
						if (the_code == null) {
							// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] TVA: Le code est null");
							the_code = new Integer(4);
						}
	
						ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] TVA retenue:" + the_code);
						String article_existe = "SELECT Article_Id, Article_Web_Id from " + base + ".FICHE_ARTICLE_WEB where Site_Id=" + site_id + " AND Article_Id='";
						article_existe += article.getSqlString(article.REFERENCE ,40) + "'";
						System.out.println(article_existe);
						ResultSet resArticleExiste = state.executeQuery(article_existe);
						if (resArticleExiste.next()) {// L'article existe	
							
							psartex.setInt(1, numCommande);
							psartex.setInt(2, (i+1));
							psartex.setString(3, article.getSqlString(article.REFERENCE,40));
							psartex.setString(4, "S");
							psartex.setString(5, article.getSqlString(article.DESIGNATION,100));
							psartex.setDouble(6, article.getDouble(article.QUANTITE));
							psartex.setDouble(7, article.getDouble(article.PRIX_U));
							psartex.setDouble(8, 0);
							psartex.setDouble(9, article.getDouble(article.TVA));
							psartex.setInt(10, the_code);
							
							psartex.setString(11, "");
							psartex.setString(12, article.getSqlString(article.COMMENTAIRE));
							psartex.setString(13, "");
							psartex.execute();
						}
						else {
							psartex.setInt(1, numCommande);
							psartex.setInt(2, (i+1));
							psartex.setString(3, article.getSqlString(article.REFERENCE,40));
							psartex.setString(4, "I");
							psartex.setString(5, article.getSqlString(article.DESIGNATION,100));
							psartex.setDouble(6, article.getDouble(article.QUANTITE));
							psartex.setDouble(7, article.getDouble(article.PRIX_U));
							psartex.setDouble(8, 0);
							psartex.setDouble(9, article.getDouble(article.TVA));
							psartex.setInt(10, the_code);
							
							psartex.setString(11, "");
							psartex.setString(12, article.getSqlString(article.COMMENTAIRE));
							psartex.setString(13, "");
							psartex.execute();
						}
					}				
				}
				else {
					/* **************** */
					/* * FICHE CLIENT * */
					/* **************** */
					// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Cration de la fiche client");
	
					DateTime dt = new DateTime();
					long now = dt.getTimeInMillis();

					String insertup = "insert into " + base + ".CLIENT_WEB ";
					insertup +=	"(Client_Id,Client_Site_Id,Site_Id,Civ,Nom,Prenom,Entreprise,Adresse_1, Adresse_2,Adresse_3, CP,Ville,Code_Pays,Tel,Fax,Email,Password) ";
					insertup +=	"values ('',?,?,?,?,?,?,?,?,?,?,?,?,?,?,lcase(?),?) ";
					insertup +=	"on duplicate key update Civ=?,Nom=?,Prenom=?,Entreprise=?,Adresse_1=?, Adresse_2=?,Adresse_3=?, CP=?,Ville=?,Code_Pays=?,Tel=?,Fax=?,Email=lcase(?),Password=?;";
					PreparedStatement psinsertup = con.prepareStatement(insertup);
					
					psinsertup.setString(1, client_id);
					psinsertup.setString(2, site_id);
					if (client_entreprise.equals("")) {
						psinsertup.setString(3, "0");
						psinsertup.setString(4, "");
						psinsertup.setString(5, "");
						psinsertup.setString(6, fact_civ_string +" "+client_nom+" "+client_prenom);
					}
					else {
						psinsertup.setString(3, client_civ);
						psinsertup.setString(4, client_nom);
						psinsertup.setString(5, client_prenom);
						psinsertup.setString(6, client_entreprise);
					}
	
					psinsertup.setString(7, client_adresse);
					psinsertup.setString(8, client_compadresse);
					psinsertup.setString(9, client_adresse3);
					psinsertup.setString(10, client_cp);
					psinsertup.setString(11, client_ville);
					psinsertup.setString(12, client_code_pays);
					psinsertup.setString(13, client_tel);
					psinsertup.setString(14, client_fax);
					psinsertup.setString(15, client_email);
					psinsertup.setString(16, client_pass);
					
					if (client_entreprise.equals("")) {
						psinsertup.setString(17, "0");
						psinsertup.setString(18, "");
						psinsertup.setString(19, "");
						psinsertup.setString(20, fact_civ_string +" "+client_nom+" "+client_prenom);
					}
					else {
						psinsertup.setString(17, client_civ);
						psinsertup.setString(18, client_nom);
						psinsertup.setString(19, client_prenom);
						psinsertup.setString(20, client_entreprise);
					}
					psinsertup.setString(21, client_adresse);
					psinsertup.setString(22, client_compadresse);
					psinsertup.setString(23, client_adresse3);
					psinsertup.setString(24, client_cp);
					psinsertup.setString(25, client_ville);
					psinsertup.setString(26, client_code_pays);
					psinsertup.setString(27, client_tel);
					psinsertup.setString(28, client_fax);
					psinsertup.setString(29, client_email);	
					psinsertup.setString(30, client_pass);
					psinsertup.execute();
					
					String reqCli = "SELECT Client_Web_Id FROM " + base + ".CLIENT_WEB WHERE Site_Id="+ site_id+" and Client_Site_Id='"+client_id+"'";
					ResultSet resCli = state.executeQuery(reqCli);
					if (resCli.next()) {// Le site est slectionn
						numClient=resCli.getInt("Client_Web_Id");
					}
					// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] id du client:"+numClient);
	
					/* **************** */
					/* * COMMANDE * */
					/* **************** */
					String Journal="";
					String Piece="";
					String Banque="";
					
					String reqSite = "SELECT * FROM " + base + ".SITE_WEB WHERE Site_Id="+ site_id;
					ResultSet resSite = state.executeQuery(reqSite);
					if (resSite.next()) {// Le site est slectionn
						Journal=resSite.getString("Journal_Banque_CB");
						Piece=resSite.getString("Num_Piece_CB");
						Banque=	resSite.getString("Banque_Client_CB");				
					}
	
					// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Cration de la fiche commande");
					String ajoutCom = "INSERT INTO " + base + ".COMMANDE_WEB ";
					ajoutCom += "(Client_Web_Id,Site_Id,Com_Site_Id, Num_Com_Site,Total_HT,Total_TTC,Remise,Montant_Remise";
					ajoutCom += ",Frais_Port,Civ_Inter,Nom_Inter,Prenom_Inter,Societe_Client,Adresse_Client_1, Adresse_Client_2 , Adresse_Client_3";
					ajoutCom += ",Ville_Client,CP_Client,Civ_Inter_Liv,Nom_Inter_Liv,Prenom_Inter_Liv,Societe_Liv ";
					ajoutCom += ",Adresse_Liv_1, Adresse_Liv_2,Adresse_Liv_3,Ville_Liv,CP_Liv,Mode_Paiement";
					ajoutCom += ",Statut_Paiement,Statut_Commande,Journal_Banque,Num_Piece,Banque_Client,Date_Reglement";
					ajoutCom += ",Date_Commande,Date_Validation,Tel_Inter,Fax_Inter,Email_Inter,Tel_Inter_Liv,Fax_Inter_Liv,Email_Inter_Liv,Mail_Manager";
					ajoutCom += ",Mode_Expedition,Commentaire,Commentaire_2,Edition_TTC,Imp_Com,Statut_Modifie,Code_Porte,Num_Transaction,Origine,Infos_Commande";
					ajoutCom += ",Code_Pays,Code_Pays_Liv,Taux_TVA_Port,Taux_Indicatif,Assujetti_TVA, Date_Creation, Date_Maj, Montant_Regle, Envoi_Multiple,Util_R, Bloque";
					ajoutCom += ") VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,lcase(?),?,?,lcase(?),?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
					PreparedStatement psajoutCom = con.prepareStatement(ajoutCom);
					
					psajoutCom.setInt(1, numClient);
					psajoutCom.setString(2, site_id);
					psajoutCom.setString(3, com_id);
					psajoutCom.setString(4, com_id);
					psajoutCom.setDouble(5, com_totalHT);
					psajoutCom.setDouble(6, com_totalTTC);
					psajoutCom.setDouble(7, com_tauxRemise);
					psajoutCom.setDouble(8, com_remise);
					psajoutCom.setDouble(9, com_fraisPort);
					if (client_entreprise.equals("")) {
						psajoutCom.setString(10, "0");
						psajoutCom.setString(11, "");
						psajoutCom.setString(12, "");
						psajoutCom.setString(13, fact_civ_string +" "+client_nom+" "+client_prenom);
					}
					else {
						psajoutCom.setString(10, client_civ);
						psajoutCom.setString(11, client_nom);
						psajoutCom.setString(12, client_prenom);
						psajoutCom.setString(13, client_entreprise);
					}
					psajoutCom.setString(14, client_adresse);
					psajoutCom.setString(15, client_compadresse);
					psajoutCom.setString(16, client_adresse3);
					psajoutCom.setString(17, client_ville);
					psajoutCom.setString(18, client_cp);
	
					if (liv_entreprise.equals("")) {
						psajoutCom.setString(19, "0");
						psajoutCom.setString(20, "");
						psajoutCom.setString(21, "");
						psajoutCom.setString(22, liv_civ_string +" "+liv_nom+" "+liv_prenom);
					}
					else {
						psajoutCom.setString(19, liv_civ);
						psajoutCom.setString(20, liv_nom);
						psajoutCom.setString(21, liv_prenom);
						psajoutCom.setString(22, liv_entreprise);
					}
					psajoutCom.setString(23, liv_adresse);
					psajoutCom.setString(24, liv_compadresse);
					psajoutCom.setString(25, liv_adresse3);
					psajoutCom.setString(26, liv_ville);
					psajoutCom.setString(27, liv_cp);
					psajoutCom.setString(28, com_modePaiement);
					psajoutCom.setInt(29, com_statut_Paiement);
	
					if (com_statut_Paiement==1) {
						psajoutCom.setString(30, "C");
						psajoutCom.setString(31, Journal);
						psajoutCom.setString(32, Piece);
						psajoutCom.setString(33, Banque);
						psajoutCom.setLong(34, com_date);
						psajoutCom.setDouble(61, com_totalTTC);
					}
					else {
						if (com_validation==0) {
							psajoutCom.setString(30, "M");
						}
						else if (com_validation==1) {
							psajoutCom.setString(30, "N");
						}
						else if (com_validation==2) {
							psajoutCom.setString(30, "C");
						}
						else if (com_validation==3) {
							psajoutCom.setString(30, "E");
						}
						else {
							psajoutCom.setString(30, "N");
						}
						psajoutCom.setString(31, "");
						psajoutCom.setString(32, "");
						psajoutCom.setString(33, "");
						psajoutCom.setLong(34, 0);
						psajoutCom.setDouble(61, 0.0);
					}
					psajoutCom.setLong(35, com_date);
					psajoutCom.setLong(36, com_date);
					psajoutCom.setString(37, client_tel);
					psajoutCom.setString(38, client_fax);
					psajoutCom.setString(39, client_email);
					psajoutCom.setString(40, liv_tel);
					psajoutCom.setString(41, liv_fax);
					psajoutCom.setString(42, liv_email);
					psajoutCom.setString(43, com_email_manager);
					psajoutCom.setString(44, com_modeExpedition);
					psajoutCom.setString(45, com_commentaires);
					psajoutCom.setString(46, com_commentaires_2);
					psajoutCom.setInt(47, com_editionTTC);
					psajoutCom.setInt(48, com_editionCom);
					psajoutCom.setInt(49, 1);
					psajoutCom.setString(50, com_codePorte);
					psajoutCom.setString(51, com_numTransaction);
					psajoutCom.setString(52, com_origine);
					psajoutCom.setString(53, com_infosCommande);
					psajoutCom.setString(54, com_codePays.equals("")?"FR":com_codePays);
					psajoutCom.setString(55, com_codePaysLiv.equals("")?"FR":com_codePaysLiv);
					psajoutCom.setDouble(56, com_tauxTVAFraisPort);
					psajoutCom.setDouble(57, com_tauxIndicatif);
					psajoutCom.setInt(58, com_assujetti);
					psajoutCom.setLong(59, now);
					psajoutCom.setLong(60, now);
					psajoutCom.setLong(62, com_envoi_multiple);
					psajoutCom.setInt(63, logRoot);
					psajoutCom.setLong(64, com_bloque);
					psajoutCom.execute();
					
					ResultSet rsnum = psajoutCom.getGeneratedKeys();
			
					if (rsnum.next()) {
						numCommande = rsnum.getInt(1);
					}
	
					/* ************************** */
					/* * DETAIL DE COMMANDE * */
					/* ************************** */
					// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] Cration des dtails de la
					// commande");
					Object[] liste = com.getStruct(com.LISTE_ARTICLES);
					StockManager stockm = new StockManager(con,base);
					for (int i = 0; i < liste.length; i++) {
	
						ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes]Un prod");
	
						ArticleCommandeStruct article = new ArticleCommandeStruct((Object[]) liste[i]);
	
						// Recherche du code TVA
						Integer the_code = code_tva.get(new Double(article.getDouble(article.TVA)));
						if (the_code == null) {
							// ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] TVA: Le code est null");
							the_code = new Integer(4);
						}
	
						ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] TVA retenue:" + the_code);
						String article_existe = "SELECT Article_Id, Article_Web_Id from " + base + ".FICHE_ARTICLE_WEB where Site_Id=" + site_id + " AND Article_Id_Site='";
						article_existe += article.getSqlString(article.REFERENCE,40) + "'";
						ResultSet resArticleExiste = state.executeQuery(article_existe);
						if (resArticleExiste.next()) {// L'article existe	
							
							String addDetail = "INSERT INTO " + base
									+ ".ARTICLE_COMMANDE_WEB(Com_Web_Id,Type_Ligne,Rank,Article_Web_Id,Designation,Quantite,Prix,Taux_TVA,Code_TVA, Article_Id,Commentaire, Commentaire_Avant)";
							addDetail += "VALUES(";
							addDetail += "" + numCommande + ",'S',"+(i+1)+",'" + resArticleExiste.getInt("Article_Web_Id") + "','" + article.getSqlString(article.DESIGNATION,100) + "'";
							addDetail += "," + article.getDouble(article.QUANTITE) + "," + article.getDouble(article.PRIX_U) + "," + article.getDouble(article.TVA) + "," + the_code;
							addDetail += ",(SELECT Article_Id from " + base + ".FICHE_ARTICLE_WEB where Site_Id=" + site_id + " AND Article_Id_Site='"
									+ article.getSqlString(article.REFERENCE,40) + "'),'"+article.getSqlString(article.COMMENTAIRE)+"',''";
							addDetail += ")";
							ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] req detail:" + addDetail);
							state2.executeUpdate(addDetail);
	
							if ((com_statut_Paiement==1 && com_validation!=3) || com_validation==2) {
								stockm.incComClients(resArticleExiste.getString("Article_Id"), article.getDouble(article.QUANTITE));
							}
						}
						else {
							String addDetail = "INSERT INTO " + base
									+ ".ARTICLE_COMMANDE_WEB(Com_Web_Id,Rank,Type_Ligne,Article_Web_Id,Designation,Quantite,Prix,Taux_TVA,Code_TVA, Article_Id,Commentaire, Commentaire_Avant)";
							addDetail += "VALUES(";
							addDetail += "" + numCommande + ","+(i+1)+",'I',0,'" + article.getSqlString(article.DESIGNATION,100) + "'";
							addDetail += "," + article.getDouble(article.QUANTITE) + "," + article.getDouble(article.PRIX_U) + "," + article.getDouble(article.TVA) + "," + the_code;
							addDetail += ",'"+article.getSqlString(article.REFERENCE,40) + "','"+article.getSqlString(article.COMMENTAIRE)+"','')";
							ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] req detail:" + addDetail);
							state.executeUpdate(addDetail);
						}
					}
				}
			}
		}
		catch (Exception e) {
			ToolsMAJ.ecrireLog(aEcrire,type,"[UploadCommandes] ERREUR5:" + e);
			e.printStackTrace();

		}
	}
}
